/*  ==================================================================	*
 *				Editor mined				*
 *				Part 1					*
 *			for documentation see mined.doc			*
 *  ==================================================================	*/

#include "mined.h"

/* #define DEBUG */


/*  ==================================================================	*
 *			Definitions specific for mined1.c		*
 *  ==================================================================	*/

# ifdef unix
# define helpcommand "man mined"
# define helpfilename "%s/mined.help"
# endif

# ifdef vms
# define helpcommand "help mined"
# define helpfilename "%s/mined.help"
# endif

# ifdef msdos
# define helpcommand "more < %smined.hlp"
# define helpfilename "%s/mined.hlp"
# endif

# ifdef unix
#  ifdef sysV
#  define printcommand "lp %s"
#  else
#  define printcommand "lpr %s"
#  endif
# endif

# ifdef vms
# define printcommand "print %s"
# endif

# ifdef msdos
# define printcommand "copy %s prn: > nul:"
# endif

#define mark_file	"@mined.mar"


/*  ==================================================================	*
 *				Screen stuff				*
 *  ==================================================================	*/

#define clearscreen()	clear_screen (); top_line_scrolled = True;

/*  ==================================================================	*
 *				Data section				*
 *  ==================================================================	*/

LINE * header;			/* Head of line list */
LINE * tail;			/* Last line in line list */
LINE * cur_line;		/* Current line in use */
LINE * top_line;		/* First line of screen */
LINE * bot_line;		/* Last line of screen */
char * cur_text;		/* Current char on current line in use */
int last_y;			/* Last y of screen. Usually SCREENMAX */
int x = 0, y = 0;		/* x, y coordinates on screen */
int line_number;		/* current line # determined by file_status */
int lines_per_page = 0;		/* assumption for file_status */
int open_linum;			/* line # to re-position to */
int open_col;			/* column to re-position to */
LINE * open_line;		/* line to re-position to */

short YMAX, XMAX;
short MENU = 1;
mousebutton mouse_button, mouse_lastbutton;
FLAG report_release = False;
int mouse_shift = 0;
int mouse_xpos, mouse_ypos, mouse_lastxpos, mouse_lastypos;

FLAG quickmenu = True;		/* Right mouse button pops up menu */
FLAG scrollbar = True;		/* use scrollbar */
char linesplitmarker = '\023';	/* Leave 1 char. space BEFORE screen [] ! */
char screen [screen_BUFL + 1];	/* I/O buffer for "writes" and "reads" */
int total_lines = 0;		/* Number of lines in file */
long total_chars = -1L;		/* Number of characters in file */
FLAG modified = False;		/* Set when file is modified */
FLAG plus_opt = False;		/* Set if options start with '+' */
FLAG viewonly = False;		/* Set when view only mode is selected */
FLAG append_flag = False;	/* Set when buffer should be appended to */
FLAG restricted = False;	/* Set when edited file shall not be switched */
FLAG overwriteOK = False;	/* Set if current file is OK for overwrite */
FLAG tab_left = True;		/* Set if moving up/down on TAB should go left */
FLAG writable;			/* Set if file cannot be written */
int JUSlevel = 0;		/* keep justified while typing? */
int JUSmode = 0;		/* 1: paragraphs end at empty line */
FLAG loading = True;		/* Loading a file? Init True for error handling */
FLAG quit = False;		/* Set when quit character is typed */
FLAG intr_char = False;		/* Set when intr character is typed */
FLAG winchg = False;		/* Set when window size has changed */
FLAG isscreenmode = False;	/* Set when screen mode is on */
FLAG stat_visible;		/* Set if status_line is visible */
FLAG top_line_scrolled = False;	/* was menu line scrolled away? */
FLAG fstat_always = False;	/* Permanent file status display wanted ? */
FLAG always_disp_code = False;	/* Permanent char code display on status line? */
FLAG waitingforinput = False;	/* Set while waiting for the next command key */
FLAG rpipe = False;		/* Set if file should be read from stdin */
FLAG wpipe = False;		/* Set if file should be written to stdout */
FLAG multiexit = True;		/* Should exit command go to next file? */
FLAG wordnonblank = False;	/* handle all non-blank sequences as words */
FLAG proportional = False;	/* Enable support for proportional fonts? */
FLAG transdeutsch = False;	/* Transform input {->, ..., ~-> ? */
FLAG auto_transdeutsch = False;	/* Auto detect transdeutsch mode from file ? */
FLAG auto_UTF_detect = True;	/* Auto detect UTF-8 encoding from file ? */
FLAG translate_output = False;	/* Transform output diacritics to strings */
char * transout;		/* Output transformation table */
char * dimansi;			/* special character dimming ANSI sequence */
char * ctrlansi;		/* control character display ANSI sequence */
char * uniansi;			/* Unicode character display ANSI sequence */
char * menuansi;		/* menu line ANSI sequence */
char * diagansi;		/* dialog (bottom status) line ANSI sequence */
int translen;			/* length of " */
FLAG controlQS = False;		/* must respect ^Q/^S handshake ? */
FLAG insert_mode = True;	/* insert or overwrite */
character control_prefix = '\026';	/* ^V/^P character to prefix control chars */
character erase_char = '\010';	/* effective (configured) char for erase left */
FLAG Chinese = False;		/* set if two-byte characters are enabled */
FLAG utf8_screen = False;	/* screen driven in UTF-8 mode ? */
FLAG utf8_input = False;	/* keyboard input in UTF-8 mode ? */
FLAG utf8_text = False;		/* text in UTF-8 representation ? */
FLAG page_scroll = False;	/* use scroll for page up/down */
FLAG page_stay = False;		/* stay at edge of screen after page up/down */
#ifdef pc
int display_delay = 9;		/* delay between display lines */
#else
int display_delay = -1;		/* Unix terminals are slow enough anyway */
#endif
#ifdef msdos
char RET_opt = 'r';		/* handle RET chars: ignore / newline */
#else
char RET_opt = ' ';		/* handle RET chars: ignore / newline */
#endif
long chars_saved;		/* Nr of chars in buffer */
int input_fd = STD_IN;		/* File descriptors for terminal dialog */
int output_fd = STD_ERR;
unsigned int out_count = 0;	/* Index in output buffer */
char file_name [maxLINE_LEN];	/* Name of file in use */
char text_buffer [MAX_CHARS];	/* for get_line, modifications, build_string */
int escape_delay = 300;		/* wait to detect escape sequence */
int hop_flag = 0;		/* Counter for the HOP function */
char TABchar = ' ';		/* Char to be shown in place of tab chars */
char TABchar0 = '\0';		/* Char to be shown at start of tab chars */
char TABchar2 = '\0';		/* Char to be shown at end of tab chars */
char TABcharmid = '\0';		/* Char to be shown in middle of tab chars */
char SHIFT_BEG = '\0';		/* Char indicating that line continues left */
char RET_MARK = '\0';		/* Char indicating end of line */
char PARA_MARK = '\0';		/* Char indicating end of paragraph */
char RET_BLANK = '\0';		/* Char to fill the end of line with */
char RET_BLANK2 = '\0';		/* Char to fill last position of line with */
char * UTF_TAB = NIL_PTR;	/* Char to be shown in place of tab chars */
char * UTF_TAB0 = NIL_PTR;	/* Char to be shown at start of tab chars */
char * UTF_TAB2 = NIL_PTR;	/* Char to be shown at end of tab chars */
char * UTF_TABmid = NIL_PTR;	/* Char to be shown in middle of tab chars */
char * UTF_RET = NIL_PTR;	/* Char indicating end of line */
char * UTF_PARA = NIL_PTR;	/* Char indicating end of paragraph */
char * UTF_RETblank = NIL_PTR;	/* Char to fill the end of line with */
char * UTF_RETblank2 = NIL_PTR;	/* Char to fill last position of line with */
char UNI_MARK = '';		/* char to be shown in place of Unicode char */
FLAG paradisp = False;		/* Shall paragraph end be distinguished? */
#ifdef vms
unsigned int fprot = 0;		/* To be used for file creatings */
unsigned int bufprot = 0;	/* To be used for paste buffer file */
unsigned int exeprot = 0;	/* To be used for creation of executables */
#else
unsigned int fprot = 0644;	/* To be used for file creatings */
unsigned int bufprot = 0600;	/* To be used for paste buffer file */
unsigned int exeprot = 0111;	/* To be used for creation of executables */
#endif
unsigned int xprot = 0;
int fnami;			/* Parameter index of current file name */
int fnami_min, fnami_max, fnami_cnt;
/* char * (* fnamv) []; */
char * * fnamv;			/* Copy of argv. Points to program params */

char * inisearch = NIL_PTR;	/* Optional startup search string */

int first_left_margin = 0;
int next_left_margin = 0;
int right_margin = 71;

#ifdef unix
/* window headline and icon text setting */
char * window_string_code;
char * mined_modf = " (*)";
#endif

/*
 * Yank variables.
 */
char * temp_dir;
char yank_file [maxLINE_LEN];
char yankie_file [maxLINE_LEN];
char panic_file [maxLINE_LEN];
char mined_dir [maxLINE_LEN];	/* startup directory to locate help file */


/*  ==================================================================	*
 *			Forward declarations				*
 *  ==================================================================	*/

void RDheader ();


/*  ==================================================================	*
 *			text string routines				*
 *  ==================================================================	*/

int
UTF_len (c)
  char c;
{
	if ((c & 0x80) == 0x00)
		return 1;
	else if ((c & 0xE0) == 0xC0)
		return 2;
	else if ((c & 0xF0) == 0xE0)
		return 3;
	else if ((c & 0xF8) == 0xF0)
		return 4;
	else if ((c & 0xFC) == 0xF8)
		return 5;
	else if ((c & 0xFE) == 0xFC)
		return 6;
	else /* ignore illegal UTF-8 code */
		return 1;
}

/*
 * inChinese (string, charpoi) determines if charpoi points to the second 
 * byte of a multi-byte character within string
 */
int
inChinese (string, charpoi)
  character * string;
  character * charpoi;
{
  while (string < charpoi) {
	if (multichar (* string)) {
		string ++;
		if (* string != '\n' /* would be an error */) string ++;
	} else
		string ++;
  }
  return string > charpoi;
}

/*
 * precede_char () moves the character pointer within line "begin_line" 
 * left by 1 character
 */
void
precede_char (poipoi, begin_line)
  char * * poipoi;
  char * begin_line;
{
  char * utf_search;
  char * utf_prev;

  if (utf8_text == True) {
	utf_search = begin_line;
	utf_prev = utf_search;
	while (utf_search < * poipoi) {
		utf_prev = utf_search;
		advance_utf8 (& utf_search);
	}
	(* poipoi) = utf_prev;
  } else if ((* poipoi) != begin_line) {
	(* poipoi) --;
	if (Chinese == True && inChinese (begin_line, * poipoi))
		(* poipoi) --;
  }
}

/*
 * Advance pointer and counter to next character.
 * Handle tab characters and Chinese 2-byte characters correctly.
 */
void
advance_utf8 (poipoi)
  char * * poipoi;
{
	int follow = UTF_len (* * poipoi) - 1;
	(* poipoi) ++;
	while (follow > 0 && (* * poipoi & 0xC0) == 0x80) {
		(* poipoi) ++;
		follow --;
	}
}
void
advance_char_col (poipoi, colpoi)
  char * * poipoi;
  int * colpoi;
{
  if (is_tab (* * poipoi)) {
	(* poipoi) ++;
	if (colpoi != (int *) NIL_PTR) * colpoi = tab (* colpoi);
  }
  else if (utf8_text == True) {
	advance_utf8 (poipoi, colpoi);
	if (colpoi != (int *) NIL_PTR) (* colpoi) ++;
  }
  else if (Chinese == True && multichar (* * poipoi)) {
	(* poipoi) ++;
	if (* * poipoi != '\0') (* poipoi) ++;
	if (colpoi != (int *) NIL_PTR) {
		(* colpoi) ++;
		(* colpoi) ++;
	}
  }
  else {
	(* poipoi) ++;
	if (colpoi != (int *) NIL_PTR) (* colpoi) ++;
  }
}
void
advance_char (poipoi)
  char * * poipoi;
{
  advance_char_col (poipoi, (int *) NIL_PTR);
}


/*  ==================================================================	*
 *			Text buffer routines				*
 *  ==================================================================	*/

int old_x = 0;	/* previous x position */

/*
 * Find_x () returns the x coordinate belonging to address.
 * (Tabs are expanded).
 */
int
find_x (line, address)
  LINE * line;
  char * address;
{
  char * textp = line->text;
  register int x_left = get_shift (line->shift_count) * - SHIFT_SIZE;
  int x_in_line = 0;	/* must start from 0 to calculate correct 
			tab positions (since SHIFT_SIZE is not guaranteed 
			to be a multiple of 8) */
	/* Alright, SHIFT_SIZE is now guaranteed to be a multiple of 8 
	   due to lots of display problems related to that matter.
	   Leave this code anyway. */

  while (textp != address && * textp != '\0') {
	advance_char_col (& textp, & x_in_line);
  }
  return x_in_line + x_left;
}

/*
 * Find_address () returns the pointer in the line with given offset.
 * (Tabs are expanded).
 * find_address is only called by move_it ()
get_shift (cnt)		is	((cnt) & DUMMY_MASK) ; DUMMY_MASK is 0x7F
tab (cnt)		is	(((cnt) + 8) & ~07)
is_tab (c)		is	((c) == '\t')
 */
char *
find_address (line, new_x, cur_x)
  LINE * line;
  int new_x;
  int * cur_x;
{
  char * textp = line->text;
  int tx = get_shift (line->shift_count) * - SHIFT_SIZE;
  char * prev_poi;
  int prev_x;

  while (tx < new_x && * textp != '\n') {
	if (is_tab (* textp)) {
		if (new_x == old_x /* (* cur_x) */ - 1 && tab (tx) > new_x)
			break;	/* Moving left over tab */
		else if (tab_left == True && new_x == old_x && tab (tx) > new_x)
			break;	/* Moving up/down on tab */
		else
			tx = tab (tx);
		textp ++;
	} else {
		prev_poi = textp;
		prev_x = tx;
		advance_char_col (& textp, & tx);
		if (tx > new_x && new_x < * cur_x) {
		/* moving left into multi-width character, stay before it */
			textp = prev_poi;
			tx = prev_x;
			new_x = tx;
		}
	}
  }
  * cur_x = tx;
  return textp;
}

/*
 * move_to: move to given coordinates on screen.
 * move_y: move to given line on screen, staying in last explicit column.
 * move_address: move to given line at given text position.
 * The caller must check that scrolling is not needed.
 * If new x-position is < 0 or > XBREAK, move_it () will check if
 * the line can be shifted. If it can it sets (or resets) the shift_count
 * field of the current line accordingly. By this mechanism, the
 * pseudo-x-positions LINE_START / LINE_END (a very small / big value)
 * perform the appropriate positioning actions.
 * Move also sets cur_text to the right char.
 * "If we're moving to the same x coordinate, try to move the the x-coordinate
 * used on the other previous call." -- This worked erroneously and was 
 * replaced by an explicit old_x variable and move_y call.
 * move_address is directly called by move_next/previous_word(), re_search(), RDwin()
 */
void
move_it (new_x, new_address, new_y)
  register int new_x;
  int new_y;
  char * new_address;
{
  register LINE * line = cur_line;	/* For building new cur_line */
  int shift = 0;			/* How many shifts to make */
  char * utf_search;
  char * utf_prev;

/*  static int rel_x = 0;	*/	/* Remember relative x position */
/*	This was used as a trick to stay virtually in the previous column 
	even when moving across shorter lines; but it had >= 2 errors.
	Renamed to old_x, made globally accessible and explicitly used 
	by appropriate calls to avoid these problems. TW */
  int tx = x;

/* Check for illegal values */
  if (new_y < 0 || new_y > last_y)
	return;

/* Adjust y-coordinate and cur_line */
  if (new_y < y)
	while (y != new_y) {
		y --;
		line = line->prev;
	}
  else
	while (y != new_y) {
		y ++;
		line = line->next;
	}

/* Set or unset relative x-coordinate */
  if (new_address == NIL_PTR) {
	new_address = find_address (line, new_x, & tx);
	new_x = tx;
  }
  else
	/* rel_x = */ new_x = find_x (line, new_address);

/* Adjust on character boundary */
  if (Chinese == True) {
	if (inChinese (line->text, new_address)) {
	/* adjust position which is currently within a multi-byte character */
		if (new_x >= x) {
			new_x ++;	/* Chinese char has double width */
			new_address ++;
		} else {
			new_x --;	/* Chinese char has double width */
			new_address --;
		}
	}
  } else if (utf8_text == True) {
	if ((* new_address & 0xC0) == 0x80) {	/* UTF-8 sequence byte? */
	/* adjust position which is currently within a UTF-8 character */
	/*  adjust new_x for multi-width characters ? */
		utf_search = line->text;
		utf_prev = utf_search;
		while (utf_search < new_address) {
			utf_prev = utf_search;
			advance_utf8 (& utf_search);
		}
		if (utf_search != new_address)
			if (new_x >= x)
				new_address = utf_search;
			else
				new_address = utf_prev;
	}
  }

/* Adjust shift_count if new_x lower than 0 or higher than XBREAK */
/* Allow adjustment also if new_x == 0 to enable left shift mark */
  if (new_x <= 0 || new_x >= XBREAK) {
	if (new_x > XBREAK || (new_x == XBREAK && * new_address != '\n'))
		shift = (new_x - XBREAK) / SHIFT_SIZE + 1;
	else {
		shift = new_x / SHIFT_SIZE;
		if (new_x % SHIFT_SIZE)
			shift --;
		if (new_x == 0 && line->shift_count != 0 && SHIFT_BEG != '\0')
			shift --;
	}

	if (shift != 0) {
		line->shift_count += shift;
		new_x = find_x (line, new_address);
		if (new_x == 0 && line->shift_count != 0 && SHIFT_BEG != '\0')
		{	line->shift_count --;
			new_x = find_x (line, new_address);
		}
		set_cursor (0, y);
		line_print (line);
		/* rel_x = new_x; */
	}
  }

/* Assign and position cursor */
  x = new_x;
  cur_text = new_address;
  cur_line = line;
  set_cursor_xy ();
}

void
move_y (ny)
  register int ny;
{
  move_it (old_x, NIL_PTR, ny);
}

void
move_to (nx, ny)
  register int nx;
  register int ny;
{
  old_x = x;
  move_it (nx, NIL_PTR, ny);
  old_x = x;
}

void
move_address (nadd, ny)
  register char * nadd;
  register int ny;
{
  old_x = x;
  move_it (0, nadd, ny);
  old_x = x;
}

void
move_address_w_o_RD (nadd, ny)
  register char * nadd;
  register int ny;
{
  old_x = x;
  move_it (0, nadd, ny);
  old_x = x;
}

/*
 * Initialize is called when a another file is edited. It free's the allocated 
 * space and sets modified back to False and fixes the header/tail pointer.
 */
void
initialize ()
{
  register LINE * line, * next_line;

/* Delete the whole list */
  for (line = header->next; line != tail; line = next_line) {
	next_line = line->next;
	free_space (line->text);
	free_header (line);
  }

/* header and tail should point to itself */
  line->next = line->prev = line;
  x = y = 0;
  rpipe = modified = False;
}

/*
 * Get_line reads one line from filedescriptor fd. If EOF is reached on fd,
 * get_line () returns ERRORS, else it returns the length of the string.
 */
char * get_l_err1;
char * get_l_err2;
char last_char = '\0';
long count_ascii;	/* count ASCII replacement candidate chars */
long count_nonascii;	/* count non-ASCII replacement corresponding chars */
long count_good_utf;	/* count good UTF-8 sequences */
long count_bad_utf;	/* count bad UTF-8 sequences */
long count_read_op;	/* count 1st read operation by get_line */
FLAG UTF_BOM;		/* UTF-8 BOM found at beginning of file? */
FLAG Unicode_BOM;	/* Unicode BOM found at beginning of file? */
FLAG Unicode_little_endian;	/* Unicode words reversed? */

void
reset_get_line ()
{
  get_l_err1 = NIL_PTR;
  get_l_err2 = NIL_PTR;
  count_ascii = 0;
  count_nonascii = 0;
  count_good_utf = 0;
  count_bad_utf = 0;
  count_read_op = 0;
  UTF_BOM = False;
  Unicode_BOM = False;
}

void
show_get_l_error (get_l_err)
  char * get_l_err;
{
  if (get_l_err != NIL_PTR) {
	ring_bell ();
	error (get_l_err, " -- type a blank");
	while (readchar () != ' ' && quit == False) {
		ring_bell ();
		flush ();
	}
  }
}
void
show_get_l_errors ()
{
  show_get_l_error (get_l_err1);
  show_get_l_error (get_l_err2);
  clear_status ();
}

static char * UTF16buf = NIL_PTR;
static char * next_byte;
static char * fini_byte;
static unsigned int read_bytes;
static unsigned int surrogate = 0;

int
UTF_transform (UTF8buf, maxbufl, next_byte_poi, fini_byte)
  char * UTF8buf;
  int maxbufl;
  character * * next_byte_poi;
  character * fini_byte;
{
  register char * ptr = UTF8buf;
  int read_chars = 0;
  unsigned long unichar;

  while (read_chars + 4 < maxbufl && * next_byte_poi < fini_byte) {
	unichar = * * next_byte_poi;
	(* next_byte_poi) ++;
	if (* next_byte_poi < fini_byte) {
		if (Unicode_little_endian == True) {
			unichar |= (* * next_byte_poi) << 8;
		} else {
			unichar = (unichar << 8) | (* * next_byte_poi);
		}
		(* next_byte_poi) ++;
	} else if (Unicode_little_endian == False) {
		unichar = 0;
	}

	if ((unichar & 0xFC00) == 0xD800) {
	/* high surrogates */
		surrogate = (unichar - 0xD7C0) << 10;
	} else if ((unichar & 0xFC00) == 0xDC00) {
	/* low surrogates */
		unichar = surrogate | (unichar & 0x03FF);
		surrogate = 0;
		* ptr ++ = 0xF0 | (unichar >> 18);
		* ptr ++ = 0x80 | ((unichar >> 12) & 0x3F);
		* ptr ++ = 0x80 | ((unichar >> 6) & 0x3F);
		* ptr ++ = 0x80 | (unichar & 0x3F);
		read_chars += 4;
	} else if (unichar < 0x80) {
		* ptr ++ = unichar;
		read_chars ++;
	} else if (unichar < 0x800) {
		* ptr ++ = 0xC0 | (unichar >> 6);
		* ptr ++ = 0x80 | (unichar & 0x3F);
		read_chars += 2;
	} else {
		* ptr ++ = 0xE0 | (unichar >> 12);
		* ptr ++ = 0x80 | ((unichar >> 6) & 0x3F);
		* ptr ++ = 0x80 | (unichar & 0x3F);
		read_chars += 3;
	}
  }

  return read_chars;
}

int
get_line (fd, buffer)
  int fd;
  register char buffer [MAX_CHARS];
{
  static char * last = NIL_PTR;
  static char * current = NIL_PTR;
  static int read_chars;
  register char * cur_pos = current;
  char * begin = buffer;
  char * fini = (char *) (buffer + MAX_CHARS - 2) /* leave space for '\n\0' */;
  register FLAG ignore1char;

  do {
	do {
	    ignore1char = False;
	    if (cur_pos == last) {
		if (Unicode_BOM == True) {
		    if (next_byte >= fini_byte) {
			if ((read_bytes = read (fd, UTF16buf, screen_BUFL)) <= 0) {
				read_chars = read_bytes;
				break;
			}
			next_byte = UTF16buf;
			fini_byte = & UTF16buf [read_bytes];
		    }
		    read_chars = UTF_transform (screen, screen_BUFL, & next_byte, fini_byte);
		} else {
			if ((read_chars = read (fd, screen, screen_BUFL)) <= 0)
				break;
		}
		last = & screen [read_chars];
		cur_pos = screen;

		if (count_read_op == 0) {
			if (strncmp (screen, "﻿", 3) == 0) {
				/* UTF-8 BOM */
				UTF_BOM = True;
			} else if (strncmp (screen, "\376\377", 2) == 0
				   && auto_UTF_detect == True) {
				/* UTF-16 BOM */
				Unicode_BOM = True;
				Unicode_little_endian = False;
				UTF_BOM = True;
			} else if (strncmp (screen, "\377\376", 2) == 0
				   && auto_UTF_detect == True) {
				/* little-endian reversed UTF-16 BOM */
				Unicode_BOM = True;
				Unicode_little_endian = True;
				UTF_BOM = True;
			}
			if (Unicode_BOM == True) {
			/* move UTF-16 input to UTF-16 buffer */
				if (UTF16buf == NIL_PTR) {
					UTF16buf = alloc (screen_BUFL);
					if (UTF16buf == NIL_PTR) {
						get_l_err1 = "Cannot allocate memory for UTF-16 buffer";
						viewonly = True;
						return ERRORS;
					}
				}
				memcpy (UTF16buf, screen, read_chars);
				read_bytes = read_chars;
				next_byte = UTF16buf;
				fini_byte = & UTF16buf [read_bytes];

				read_chars = UTF_transform (screen, screen_BUFL, & next_byte, fini_byte);
				last = & screen [read_chars];

				modified = True;
			}
			count_read_op = 1;
		}
	    }

	    switch (* cur_pos) {
		case '\0':
			get_l_err1 = "File contains NUL characters (^@) - changed to DEL characters";
			* cur_pos = '\177';
			break;
		case '{':
		case '|':
		case '}':
		case '[':
		case '\\':
		case ']':
		case '~':
			count_ascii ++;
			break;
		case '':
		case '':
		case '':
		case '':
		case '':
		case '':
		case '':
			count_nonascii ++;
			break;
	    }

	/* UTF-8 auto-detection */
	    if ((* cur_pos & 0xC0) == 0x80) {
		if ((last_char & 0xC0) == 0xC0) {
			count_good_utf ++;
		} else if ((last_char & 0x80) == 0x00) {
			count_bad_utf ++;
		}
	    } else if ((last_char & 0xC0) == 0xC0) {
		count_bad_utf ++;
	    }

#ifdef msdos
	    if (* cur_pos == '\n' && last_char != '\r')
		set_modified ();
#endif
	    if (RET_opt == 'R' && * cur_pos == '\n' && last_char == '\r') {
		last_char = * cur_pos;
		cur_pos ++;
		ignore1char = True;
		set_modified ();
	    }
	    else
		last_char = * cur_pos;

	    if (* cur_pos == '\r' ) {
		if (RET_opt == 'R') {
			* cur_pos = '\n';
			set_modified ();
		}
		else if (RET_opt == 'r') {
			cur_pos ++;
			ignore1char = True;
#ifndef msdos
			set_modified ();
#endif
		}
	    }
	} while (ignore1char == True);
	if (cur_pos == last) break;
	if (buffer == fini && * cur_pos != '\n') {
		get_l_err2 = "Line(s) too long - split";
		* buffer ++ = '\n';
		cur_pos --;
		* cur_pos = linesplitmarker;
			/* To provide safe space for this operation, the 
			   screen array (which cur_pos points into) is 
			   preceded by the char variable linesplitmarker, 
			   which also holds the marker character */
		break;
	}
  } while ((* buffer ++ = * cur_pos ++) != '\n');

  current = cur_pos;
  if (read_chars <= 0) {
	if (buffer == begin)
		return ERRORS;
	if (* (buffer - 1) != '\n')
		if (loading == True) /* Add '\n' to last line of file */
			* buffer ++ = '\n';
		else {
			* buffer = '\0';
			return NO_LINE;
		}
  }

  * buffer = '\0';
  return (int) (buffer - begin);
}

/*
 * get_open_pos and save_open_pos look up and save the current file 
 * position in @mined.mar . For save_open_pos, line_number must be up-to-date
 */
void
get_open_pos (fn)
  char * fn;
{
  int mark_fd = open (mark_file, O_RDONLY | O_BINARY, 0);
  char * post;
  FLAG modif = modified;

  open_linum = 0;
  open_col = 0;
  if (mark_fd >= 0) {
	while ((get_line (mark_fd, text_buffer)) != ERRORS) {
		if (strncmp (fn, text_buffer, strlen (fn)) == 0) {
			post = text_buffer + strlen (fn);
			if (* post == ' ') {
				lines_per_page = 0;
				sscanf (post, "%d%d%d", 
				   & open_linum, & open_col, & lines_per_page);
			}
		}
	}
	close (mark_fd);
	clear_buffer ();
  }
  modified = modif; /* don't let the line number file affect the modified flag */
}

int
get_cur_col ()
{
  int cur_column = 0;
  char * cpoi = cur_line->text;

  while (* cpoi != '\0' && cpoi != cur_text) {
	advance_char_col (& cpoi, & cur_column);
  }
  return cur_column;
}

void
save_open_pos (fn, force)
  char * fn;
  int force;
{
  int mark_fd;
  char marktext [maxLINE_LEN];
  int cur_column;

  if (fn [0] != '\0') {
	if (force > 0)
		mark_fd = open (mark_file, O_WRONLY | O_APPEND | O_CREAT, fprot);
	else	mark_fd = open (mark_file, O_WRONLY | O_APPEND, fprot);
	if (mark_fd >= 0) {
		cur_column = get_cur_col ();
		build_string (marktext, "%s %d %d %d\n",
			      fn, line_number, cur_column, lines_per_page);
		write (mark_fd, marktext, strlen (marktext));
		close (mark_fd);
	}
  }
}

/*
 * Load_file loads the file with given name or the input pipe into memory.
 * If the file couldn't be opened, just an empty line is installed.
 * Buffer pointers are initialized.
 */
void
load_file (file, with_display)
  char * file;
  FLAG with_display;
{
  register LINE * line = header;
  register int len;
  long nr_of_bytes = 0L;
  long nr_of_chars = 0L;
  int fd = -1;		/* Filedescriptor for file */
  FLAG locked = False;	/* lockf performed ? */

  total_lines = 0;	/* Zero lines to start with */
  open_linum = 0;	/* clear init. line number info */

  clearscreen ();
  flush ();
  overwriteOK = False;
  writable = True;
  if (file == NIL_PTR) {
	if (rpipe == False)
		status_msg ("No file");
	else {
		fd = 0;
		file = "standard input";
	}
	file_name [0] = '\0';
  }
  else {
	copy_string (file_name, file);	/* Save file name */
	if (access (file, 0 /* F_OK */) < 0) {	/* Cannot access file */
		status_line ("New file ", file);
		overwriteOK = False;
	}
	else if ((fd = open (file, O_RDWR | O_BINARY, 0)) >= 0) {
		overwriteOK = True;
		writable = True;
/*
		if (lockf (fd, F_LOCK, 0) < 0) {
			status_line ("No lock for ", file);
		}
		else	locked = True;
*/
		get_open_pos (file_name);
	}
	else if ((fd = open (file, O_RDONLY | O_BINARY, 0)) < 0)
		error ("Cannot open: " /*, file */, serror ());
	else {	overwriteOK = True;
		writable = False;
		get_open_pos (file_name);
	}
  }

/* Read file */
  loading = True;		/* Loading file, so set flag */
  reset_get_line ();

  if (fd >= 0) {
	status_line ("Reading ", file);
	while (line != NIL_LINE
		&& (len = get_line (fd, text_buffer)) != ERRORS) {
		line = line_insert (line, text_buffer, len);
		nr_of_bytes += (long) len;
		nr_of_chars += char_count (text_buffer);
	}
	if (total_lines == 0 && line != NIL_LINE)   /* The file was empty! */
		line = line_insert (line, "\n", 1);
	clear_buffer (); /* Clear output buffer: out_count = 0; */
	cur_line = header->next;
	if (locked == False)
		(void) close (fd);		/* Close file */

	if (auto_transdeutsch == True) {
		if (count_nonascii > count_ascii)
			transdeutsch = True;
		else	transdeutsch = False;
	}

	if (auto_UTF_detect == True) {
		if (UTF_BOM == True || count_good_utf > count_bad_utf)
			utf8_text = True;
		else	utf8_text = False;
	}

	if (line != NIL_LINE) {
	   show_get_l_errors ();
	   fstatus ("Read", nr_of_bytes, nr_of_chars);
	}
  }
  else				/* Just install a "\n" */
	line = line_insert (line, "\n", 1);

  if (line == NIL_LINE) {
	sleep (2) /* give time to read allocation error msg */;
	viewonly = True;
  }

  reset (header->next, 0);	/* Initialize pointers */
  move_to (0, 0);
  loading = False;		/* Stop loading, reset flag */

  if (open_linum > 0) {
	open_line = proceed (header->next, open_linum - 1);
	move_to (0, find_y_w_o_RD (open_line));
	{	/* ??? */
		int cur_column = 0;
		char * cpoi = cur_line->text;
		while (* cpoi != '\n' && cur_column < open_col) {
			advance_char_col (& cpoi, & cur_column);
		}
		move_address (cpoi, y);
		open_col = 0;
	}
	if (with_display == True)
		display (0, top_line, last_y, 0);
	fstatus ("Read", nr_of_bytes, nr_of_chars);
	while (open_col > 0) {
		MRT ();
		open_col --;
	}
	Markn (-1);
  } else if (with_display == True) {
	display (0, top_line, last_y, 0);
	move_to (0, 0);
  }

#ifdef unix
  RDheader ();
#endif
}

#ifdef msdos
#include "mineddir.h"
struct ffblk ffblock;
FLAG ffblockvalid = False;
char wildfile [maxLINE_LEN];
char drive [9], dir [maxLINE_LEN], name [13], ext [5];
#endif

void
load_wild_file (file, with_display)
  char * file;
  FLAG with_display;
{
#ifdef msdos
  char dumdrive [9], dumdir [maxLINE_LEN];
#endif

#ifdef msdos
  ffblockvalid = False;
#endif
  if (file == NIL_PTR) load_file (file, with_display);
  else {
#ifdef msdos
/*	(void) fnsplit (fnamv [fnami], drive, dir, name, ext);
	if (findfirst (fnamv [fnami], & ffblock, 0) == 0) {
*/
	(void) fnsplit (file, drive, dir, name, ext);
	if (findfirst (file, & ffblock, 0) == 0) {
		ffblockvalid = True;
		(void) fnsplit (ffblock.ff_name, dumdrive, dumdir, name, ext);
		fnmerge (wildfile, drive, dir, name, ext);
		load_file (wildfile, with_display);
	}
	else
#endif
	load_file (file, with_display);
  }
}

/*-------------------------------------------------------------------------*/

/*
 * Ask user if named file should be overwritten.
 */
FLAG
checkoverwrite (name)
char * name;
{
  character c;

  if (access (name, 0 /* F_OK */) < 0)	/* Cannot access file */
	return True;	/* thus no danger of unwanted damage */

  status_line (name [0] ? name : "[buffer]" ,
			": OK to overwrite? (y/n)");
	/* previously only basename (name) was printed */
  c = promptyn ();
  clear_status ();
  if (c == 'y')
	return True;
  else if (c == 'n')
	return False;
  else {
/*	quit = False;	abort character has been given */
	return False;
  }
}

/*
 * Attach new file name to buffer
 */
void
NN ()
{
  char file [maxLINE_LEN];	/* Buffer for new file name */

  if (restricted == True)
	{restrictederr (); return;}
  if (get_file ("Enter new file name:", file) == ERRORS)
	return;

  overwriteOK = False;
  writable = True;
  copy_string (file_name, file);	/* Save new file name */
#ifdef unix
  if (modified == True) RDheader ();
#endif
  set_modified ();	/* cf. CHDI command */
  clear_status ();
}

/*
 * Write text in memory to file.
 */
/* Call graph for writing functions:
	panic --\
		 > QUED --\
	ESC q --/	   > ask_save --\
	ESC e ---> EDIT --/		 \
	ESC v ---> VIEW -/		  \
	ESC w -----------------------------> WT
	ESC z -----------> SUSP ----------/
	ESC ESC ---------> EXED ---------/
*/
long write_count;	/* number of bytes written */
long chars_written;	/* number of chars written */
void
write_file (fd)
  int fd;
{
  register LINE * line;

  write_count = 0L;
  chars_written = 0L;
  clear_buffer (); /* out_count = 0; */
  for (line = header->next; line != tail; line = line->next) {
	if (line->shift_count & DUMMY) {
		if (line->next == tail && line->text [0] == '\n')
			continue;
	}
	if (writestring (fd, line->text) == ERRORS) {
		write_count = -1L;
		chars_written = -1L;
		break;
	}
	write_count += (long) length_of (line->text);
	chars_written += (long) char_count (line->text);
  }

  if (write_count > 0L && flush_buffer (fd) == ERRORS) {
	write_count = -1L;
	chars_written = -1L;
  }

  (void) close (fd);
}

int
wrt_text (forceflag)
  FLAG forceflag;
{
  char file [maxLINE_LEN];	/* Buffer for new file name */
  int fd;			/* Filedescriptor of file */
  int ret;

  if (wpipe == True) {
    fd = STD_OUT;
    status_line ("Writing ", "to standard output");
    wpipe = False; /* no further write to same stream possible */
  }
  else {
    if (forceflag == False && modified == False) {
	if (file_name [0] != '\0') {
		fstatus ("(Write not necessary)", -1L, -1L);
		save_open_pos (file_name, hop_flag);
	}
	else	status_msg ("Write not necessary.");
	return FINE;
    }

    /* Check if file_name is valid and if file can be written */
    if (file_name [0] == '\0' || writable == False) {
	overwriteOK = False;
	if ((ret = get_file ("Enter file name:", file)) != FINE)
		return ret;
	copy_string (file_name, file);		/* Save file name */
    }
    if (overwriteOK == False) {
	if (checkoverwrite (file_name) == True)
		overwriteOK = True;
	else {	if (quit == False)
			writable = False;
		return ERRORS;
	}
    }
    if ((fd = creat (file_name, fprot | xprot)) < 0) {	/* Empty file */
	error ("Cannot create or write: " /*, file_name */, serror ());
	writable = False;
	return ERRORS;
    }
    else
	writable = True;

    status_line ("Writing ", file_name);
  }

  write_file (fd);

  if (write_count == -1L)
	return ERRORS;

  modified = False;
#ifdef unix
  RDheader ();
#endif
  rpipe = False;	/* File name is now assigned */

/* Display how many chars (and lines) were written */
  fstatus ("Wrote", write_count, chars_written);
/*  fstatus ("Wrote", -1L); */
  save_open_pos (file_name, hop_flag);
  return FINE;
}

void
WT ()
{
  (void) wrt_text (False);
}

void
WTU ()
{
  (void) wrt_text (True);
}

/*
 * Save current file pos
 */
void
SAVPOS ()
{
  fstatus ("remembering file position", -1L, -1L);
  save_open_pos (file_name, 1);
}

/*
 * Ask the user if he wants to save the file or not.
 */
int
ask_save ()
{
  register character c;

  status_line (file_name [0] ? file_name : "[buffer]" ,
			" has been modified. Save? (y/n)");
	/* previously only basename (file_name) was printed */
  c = promptyn ();
  clear_status ();
  if (c == 'y')
	return wrt_text (False);
  else if (c == 'n')
	return FINE;
  else {
	quit = False;	/* abort character has been given */
	return ERRORS;
  }
}

int
panicwrite ()
{
  int fd;
  fd = creat (panic_file, fprot);
  write_file (fd);
  if (write_count == -1L)
	return ERRORS;
  else	return FINE;
}

/*
 * Edit/view another file. If the current file has been modified, 
 * ask whether the user wants to save it.
 * (We could allow to switch between edit and view mode without changing 
 * the file, but we would have to consider carefully the relationship 
 * between viewonly and modified.)
 */
void
edit_file (prompt, vomode)
  char * prompt;
  FLAG vomode;
{
  char new_file [maxLINE_LEN];	/* Buffer to hold new file name */

  if (modified == True && viewonly == False && ask_save () != FINE)
	return;

/* Get new file name */
  if (get_file (prompt, new_file) == ERRORS)
	return;

  viewonly = vomode;

/* Free old linked list, initialize global variables and load new file */
  initialize ();
  clearscreen ();
  load_wild_file (new_file [0] == '\0' ? NIL_PTR : new_file, True);
}

void
edit_this_file (new_file, vomode)
  char * new_file;
  FLAG vomode;
{
  if (modified == True && viewonly == False && ask_save () != FINE)
	return;

  viewonly = vomode;

/* Free old linked list, initialize global variables and load new file */
  initialize ();
  clearscreen ();
  load_wild_file (new_file [0] == '\0' ? NIL_PTR : new_file, True);
}

void
EDIT ()
{
  if (restricted == True)
	{restrictederr (); return;}
  edit_file ("Edit file:", False);
}

void
VIEW ()
{
  if (restricted == True)
	{restrictederr (); return;}
  edit_file ("View file:", True);
}

void
EDITmode ()
{
  viewonly = False;
  FSTAT ();
}

void
VIEWmode ()
{
  if (modified == False) {
	viewonly = True;
	FSTAT ();
  } else error ("Cannot view only - already modified", NIL_PTR);
}

void
quit_mined ()
{
  delete_yank_file ();
  set_cursor (0, YMAX);
  putchar ('\n');
  raw_mode (OFF);
  exit (0);
}

void
edit_nth_file (n)
  int n;
{
  int number, index;

  if (modified == True && viewonly == False && ask_save () != FINE)
	return;

  if (n == -1) {
	index = get_number ("Edit which file (enter number) ...", '\0', & number);
	if (index == ERRORS) return;
	n = number - 1 + fnami_min;
  }
  if (n < fnami_min) n = fnami_min;
  if (n > fnami_max) n = fnami_max;

/* Free old linked list, initialize global variables and load new file */
  initialize ();
  clearscreen ();

  fnami = n;
  if (fnami < fnami_min) load_wild_file (NIL_PTR, True);
  else	load_wild_file (fnamv [fnami], True);
}

void
nextfile (exitiflast)
	FLAG exitiflast;
{
#ifdef msdos
  char dumdrive [9], dumdir [maxLINE_LEN];
#endif

  if (hop_flag > 0) edit_nth_file (fnami_max);
  else
#ifdef msdos
	if (modified == True && viewonly == False && ask_save () != FINE)
		return;

	if ((ffblockvalid == True) && (findnext (& ffblock)) == 0) {
	/* Free old linked list, initialize global variables and load new file */
	  initialize ();
	  clearscreen ();
	  (void) fnsplit (ffblock.ff_name, dumdrive, dumdir, name, ext);
	  fnmerge (wildfile, drive, dir, name, ext);
	  load_file (wildfile, True);
	  return;
	}
	ffblockvalid = False;
#endif
	if (fnami >= fnami_max || fnami == 0) {
		if (exitiflast == True) quit_mined ();
		else error ("Already at last file", NIL_PTR);
	}
	else edit_nth_file (fnami + 1);
}

void
NXTFILE ()
{
  nextfile (False);
}

void
PRVFILE ()
{
  if (hop_flag > 0) edit_nth_file (fnami_min);
  else	if (fnami == fnami_min) {
#ifdef msdos
		error ("Already at first file name", NIL_PTR);
#else
		error ("Already at first file", NIL_PTR);
#endif
		return;
	}
	else edit_nth_file (fnami - 1);
}

void
NTHFILE ()
{
  edit_nth_file (-1);
}

/*
 * Leave editor. If the file has changed, ask if the user wants to save it.
 */
void
QUED ()
{
  if (modified == True && viewonly == False && ask_save () != FINE)
	return;

  quit_mined ();
}

/*
 * Exit editing current file. If the file has changed, save it.
 * Edit next file if there is one.
 */
void
EXFILE ()
{
  if (modified == True)
	if (wrt_text (False) != FINE) return;

  if (hop_flag == 0) nextfile (True);
  else quit_mined ();
}

/*
 * Exit editor. If the file has changed, save it.
 */
void
EXMINED ()
{
  if (modified == True)
	if (wrt_text (False) != FINE) return;

  quit_mined ();
}

/*
 * Exit editing current file. Exit editor if multiexit flag set.
 */
void
EXED ()
{
  if (multiexit == True) EXFILE ();
  else EXMINED ();
}

/*
 * Count_chars () count the number of chars that the line would occupy on the
 * screen. Counting starts at the real x-coordinate of the line.
 * Was only called by delete_text () in a buggy part.
 * !!! Would have to be fixed for Chinese/UTF8 if re-activated!
 */
#ifdef UNUSED
int
count_chars (line)
  LINE * line;
{
  register int cnt = get_shift (line->shift_count) * - SHIFT_SIZE;
  register char * textp = line->text;

/* Find begin of line on screen */
  while (cnt < 0) {
	if (is_tab (* textp ++))
		cnt = tab (cnt);
	else
		cnt ++;
  }

/* Count number of chars left */
  cnt = 0;
  while (* textp != '\n') {
	if (is_tab (* textp ++))
		 cnt = tab (cnt);
	else
		cnt ++;
  }
  return cnt;
}
#endif /* UNUSED */


/*  ==================================================================	*
 *				Web marker insertion			*
 *  ==================================================================	*/

void
HTML ()
{
  char marker [maxLINE_LEN];

  if (viewonly == True)
	{viewonlyerr (); return;}
  if (get_string ("HTML marker:", marker, True, " ") != FINE)
	return;

  set_up (DELETE, False);
  S ('<');
  S ('/');
  S ('>');
  MLF ();
  if (insert (cur_line, cur_text, marker) == ERRORS)
	return;
  MLF ();
  MLF ();
  PT ();
  S ('<');
  S ('>');
  MLF ();
  if ((marker [0] == 'A' || marker [0] == 'a') && marker [1] == '\0') {
	S (marker [0]);
	S (' ');
	S ('h');
	S ('r');
	S ('e');
	S ('f');
	S ('=');
  } else
	(void) insert (cur_line, cur_text, marker);
  RD ();
}


/*  ==================================================================	*
 *				Line wrap around			*
 *  ==================================================================	*/

/*
 * determine if the next line contains visible text
 */
int
nonblank_line_follows ()
{
  LINE * next_line = cur_line->next;
  char * next_char;

  if (next_line == NIL_LINE) return 0;
  next_char = next_line->text;
  if (next_char == NIL_PTR) return 0;
  while (white_space (* next_char)) next_char ++;
  return * next_char != '\n';
}

/*
 * JUS justifies the current line according to the current margins
 */
void
justi (left_margin, jushop)
	int left_margin;
	int jushop;
{
  char * poi;
  char * last_blank;
  int column;

  poi = cur_line->text;
  column = 0;
  while (column < left_margin && (poi < cur_text || white_space (* poi))) {
	advance_char_col (& poi, & column);
  }
  if (column < left_margin) {
	move_address (poi, y);
	while (column < left_margin) {
		if (tab (column) <= left_margin) {
			S0 ('\t');
			column = tab (column);
		} else {
			S0 (' ');
			column ++;
		}
	}
	poi = cur_line->text;	/* old text pointer may be invalid */
	column = 0;		/* so start again */
  }
  last_blank = NIL_PTR;
  while (column < right_margin && * poi != '\n') {
	if (column >= left_margin &&
	    (white_space (* poi) /* || * poi == '-' */)) last_blank = poi;
	advance_char_col (& poi, & column);
  }
  if (* poi != '\n') {
	if (last_blank != NIL_PTR) {
		poi = last_blank;
		poi ++;
		move_address (poi, y);
		S0 ('\n');
		justi (next_left_margin, jushop);
	}
	else {	/* no wrapping point found */
		while (! white_space (* poi) && * poi != '\n')
			advance_char_col (& poi, & column);	/* to handle Chin. */
		if (* poi == '\n') {
			move_address (poi, y);
			MRT ();
		}
		else {
			poi ++;
			move_address (poi, y);
			if (* poi != '\n') S0 ('\n');
			else MRT ();
			justi (next_left_margin, jushop);
		}
	}
  }
  else if (poi - last_blank == 1 ||
	   (jushop > 0 && nonblank_line_follows ())) {
  /* continue with next line */
	move_address (poi, y);
	if (poi - last_blank != 1) S0 (' ');
	DCC0 ();				/* concatenate lines */
	while (white_space (* cur_text)) DCC0 ();  /* remove leading space */
	if (* cur_text != '\n') justi (next_left_margin, jushop);
	else {
		poi = cur_text;
		precede_char (& poi, cur_line->text);
		while (white_space (* poi)) {
			DPC0 ();
			poi = cur_text;
			precede_char (& poi, cur_line->text);
		}
		MRT ();
	}
  }
  else {
	move_address (poi, y);
	MRT ();
  }
}
void
JUS ()
{
  if (hop_flag > 0) {
	hop_flag = 0;
	justi (first_left_margin, 1 - JUSmode);
  } else
	justi (first_left_margin, JUSmode);
}
void
JUSandreturn ()
{
  LINE * prevline = cur_line->prev;
  int curcol = get_cur_col ();	/* current position within line */
  int i;
  int not_at_lineend = * cur_text != '\n';

  char * poi;
  int column;	/* effective visual column of current position */

  if (JUSlevel < 2) {
	poi = cur_line->text;
	column = 0;
	while (poi < cur_text) advance_char_col (& poi, & column);
	if (column <= right_margin) return;
  }

	justi (first_left_margin, JUSmode);
	move_address (prevline->next->text, find_y (prevline->next));
	for (i = 0; i < curcol; i ++) {
		if (* cur_text == '\n') {
			MRT ();
			while (white_space (* cur_text)) {
				MRT ();
			}
		}
		MRT ();
	}
#ifdef garble
	if (not_at_lineend && * cur_text == '\n') {
printf (" 1 more \n"); sleep (1);
		MRT ();
		while (white_space (* cur_text)) {
printf (" 1 more \n"); sleep (1);
			MRT ();
		}
	}
#endif
}

void
modify_int (name, var, min, max)
  char * name;
  int * var;
  int min, max;
{
  int number;

  build_string (text_buffer, "%s (%d), new value:", name, * var);
  if (get_number (text_buffer, '\0', & number) == ERRORS) return;
  if (number < min) {
	error ("Value too small", NIL_PTR);
	return;
  }
  if (number > max) {
	error ("Value too large", NIL_PTR);
	return;
  }
  * var = number;
}

void
modify_col (name, var, min, max)
  char * name;
  int * var;
  int min, max;
{
  int number;

  build_string (text_buffer, "%s (%d), new value (Enter for current column):", name, * var);
  if (get_number (text_buffer, '0', & number) == ERRORS) return;
  if (number == 0) number = x + 1;
  if (number < min) {
	error ("Value too small", NIL_PTR);
	return;
  }
  if (number > max) {
	error ("Value too large", NIL_PTR);
	return;
  }
  * var = number;
}

void
ADJLM ()
{	next_left_margin ++;
	modify_col ("both left margins", & next_left_margin, 1, right_margin - 2);
	next_left_margin --;
	first_left_margin = next_left_margin;
}

void
ADJFLM ()
{	first_left_margin ++;
	modify_col ("first left margin", & first_left_margin, 1, right_margin - 2);
	first_left_margin --;
}

void
ADJNLM ()
{	next_left_margin ++;
	modify_col ("next left margin", & next_left_margin, 1, right_margin - 2);
	next_left_margin --;
}

void
ADJRM ()
{
  int left = next_left_margin;
  if (first_left_margin > left) left = first_left_margin;

	right_margin --;
	modify_col ("right margin", & right_margin, left + 2, 999);
	right_margin ++;
}

void
ADJPAGELEN ()
{
	modify_int ("assumed lines per page", & lines_per_page, 0, 999);
}


/*  ==================================================================	*
 *				Miscellaneous				*
 *  ==================================================================	*/

#ifdef unix
/*
 * Set window headline and icon text
 */
void
build_header (ws, fn, modf)
	char * ws;
	char * fn;
	char * modf;
{
  build_string (ws, window_string_code, fn, modf, fn, modf);
}
void
RDheader ()
{
  char window_string [MAX_CHARS];

  if (loading == False) {
	build_header (window_string, file_name [0] == '\0' ? "[buffer]" : file_name,
					modified == True ? mined_modf : "");
	write (output_fd, window_string, strlen (window_string));
  }
}
#endif

/*
 * Set the modified flag
 */
void
set_modified ()
{
  if (modified == False) {
	modified = True;
#ifdef unix
	RDheader ();
#endif
  }
}

/*
 * Redraw the screen
 */
void
RD_nobot ()
{
  reverse_off ();
  clearscreen ();

/* display page */
  display (0, top_line, last_y, y);

/* redraw scroll bar */
  display_scrollbar (False);

/* clear/redraw last line */
  set_cursor (0, YMAX);
  clear_lastline ();
  move_y (y);

#ifdef unix
  RDheader ();
#endif
}

void
RD ()
{
  RD_nobot ();
  if (stat_visible == True) rd_bottom_line ();
}

void
RD_y (y_pos)
  int y_pos;
{
  reverse_off ();
  clearscreen ();

/* display page */
  display (0, top_line, last_y, y_pos);

/* clear/redraw last line */
  set_cursor (0, YMAX);
  clear_lastline ();
  if (stat_visible == True) rd_bottom_line ();
}

/*
 * Adjust current window size after WINCH signal
 */
void
RDwin ()
{
  register LINE * current_line;

  winchg = False;
  getwinsize ();

  current_line = cur_line;
  reset (top_line, y);
/*  move_y (find_y_w_o_RD (current_line)); */
  move_address (cur_text, find_y_w_o_RD (current_line));
  RD_nobot ();
  if (MENU) {
	displaymenuline ();
	set_cursor_xy ();
  }
  if (stat_visible == True) rd_bottom_line ();
  flush ();
}

void
change_screen_size (sb, keep_columns)
  FLAG sb, keep_columns;
{
  int index, mode1, mode2;

/* Experimental area: */
/*	set_screen_mode (mode1);	any available mode number */
/*	set_video_lines (mode1);	0/1/2: 200/350/400 lines */
	/* does not seem to have any effect */
/*	set_textmode_height (mode1);	0/1/2: font height 8/14/16 */
/*	set_grafmode_height (mode1, mode2);
		0/1/2: font height 8/14/16 1/2/3/n: 14/25/43/n lines */
/*	set_fontbank (f);		0..7 */
/**/
  if (hop_flag > 0) {
#ifdef msdos
    if (keep_columns == True) {
      if (sb == BIGGER) {
	index = get_number ("Switch to font bank (0..7) ", '\0', & mode1);
	if (index == ERRORS) return;
	set_fontbank (mode1);
      } else {
	index = get_number ("Set character height (<= 32 pixels) ", '\0', & mode1);
	if (index == ERRORS) return;
	set_font_height (mode1);
      }
    } else {
      if (sb == BIGGER) {
#endif
	index = get_number ("Select video mode ", '\0', & mode1);
	if (index == ERRORS) return;
	set_screen_mode (mode1);
#ifdef msdos
      } else {
	index = get_number ("Select graf font (0/1/2: font height 8/14/16) ", '\0', & mode1);
	if (index == ERRORS) return;
	index = get_number ("Select line number (1/2/3/n: 14/25/43/n) ", '\0', & mode2);
	if (index == ERRORS) return;
	set_grafmode_height (mode1, mode2);	/* 0/1/2: font height 8/14/16 */
					/* 1/2/3/n: 14/25/43/n lines */
      }
    }
#endif
  }
  else {
	resize_screen (sb, keep_columns);
  }
  RDwin ();
}
void
screenmorelines ()
{
  change_screen_size (BIGGER, True);
}
void
screenlesslines ()
{
  change_screen_size (SMALLER, True);
}
void
screenbigger ()
{
  change_screen_size (BIGGER, False);
}
void
screensmaller ()
{
  change_screen_size (SMALLER, False);
}

void
LNCI ()
{
  switch_textmode_height (True);
  RDwin ();
}

void
LNSW ()
{
  switch_textmode_height (False);
  RDwin ();
}

/*
 * Ignore this keystroke.
 */
void
I ()
{
}

/*
 * Fortifying 'HOP' key.
 */
void
HOP ()
{
  hop_flag = 2;
  if (MENU) {
	displayflags ();
	set_cursor_xy ();
	flush ();
  }
  if (! char_ready_within (500))
	status_msg ("Continue HOP command (next command fortified) ...");
}

/*
 * Cancel prefix function.
 */
void
CANCEL ()
{
  hop_flag = 0;
  clear_status ();
}

/*
 * Call proc associated with function key.
 */
void
FUNKEY ()
{
  (* keyproc) ('\0');
  keyproc = I;
}

/*
 * Toggle insert/overwrite mode.
 */
void
TOGINS ()
{
  if (insert_mode == True) insert_mode = False;
  else insert_mode = True;
}

/*
 * Toggle input character translation mode.
 */
void
TOGGO ()
{
  if (transdeutsch == True) transdeutsch = False;
  else transdeutsch = True;
}

#define cmd_char(c)	(c < '\040' ? c + '\100' : (c >= '\140' ? c - '\040' : c))

/*
 * Interpret control-Q commands. Most can be implemented with the Hop function.
 */
void
ctrlQ ()
{
  character c;
  void (* func) ();

  if (! char_ready_within (500))
	status_msg ("^Q: blockBegin Find replAce goto<n>mark HOP...");
  if (quit == True) return;
  c = readchar ();
  if (quit == True) return;
  clear_status ();
  if ('0' <= c && c <= '9') {GOMAn ((int) c - (int) '0'); return;}
  if (c == '\033' || c == QUITCHAR) {CANCEL (); return;}
  switch (cmd_char (c)) {
	case 'B' : {GOMA () ; return;}
	case 'K' : { ; return;}		/* not exactly WS function */
	case 'P' : { ; return;}		/* not exactly WS function */
	case 'V' : { ; return;}		/* not exactly WS function */
	case 'W' :			/* not exactly WS function */
	case 'Z' :			/* not exactly WS function */
	case 'Y' :
	case '\177' : {
			func = key_map [c];
			hop_flag = 1;
			(* func) (c);
			return;
		      }
	case 'F' : {if (hop_flag > 0)
			SRV ();
		    else
			SFW ();
		    return;
		   }
	case 'A' : {if (hop_flag > 0)
			REPL ();
		    else
			GR ();
		    return;
		   }
	case 'Q' : {REPT (' '); return;}	/* not exactly WS function */
	case 'L' :			/* not exactly WS function */
/*
^Q: B/K top/bottom block
    P last position
    W/Z continuous scroll
    V last find or block
    Y/DEL delete line right/left
    0-9 marker
    F find
    A replace
    Q repeat next key/command
    L find misspelling
*/
	default : {
		   func = key_map [c];
		   if ((c < ' ') || ((voidfunc) func == (voidfunc) FUNKEY)) {
	/* (voidfunc) is an identity cast here. It seems to be required for
	   the sake of the apparently totally rotten microvax C compiler */
			hop_flag = 1;
			(* func) (c);
		   }
		   else
			BAD (c);
		   return;
		  }
  }
}

/*
 * Interpret control-K commands.
 */
void
ctrlK ()
{
  character c;

  if (! char_ready_within (500))
	status_msg ("^K: Save Done eXit Quit Read Log <n>mark / block: B/K mark Cop Ydel moV Wr...");
  if (quit == True) return;
  c = readchar ();
  if (quit == True) return;
  clear_status ();
  if ('0' <= c && c <= '9') {MARKn ((int) c - (int) '0'); return;}
  if (c == '\033' || c == QUITCHAR) {CANCEL (); return;}
  switch (cmd_char (c)) {
	case 'S' : {WTU (); return;}
	case 'D' : {EXFILE (); return;}
	case 'X' : {EXMINED (); return;}
	case 'Q' : {QUED (); return;}
	case 'B' : {MARK () ; return;}
	case 'K' : {YA () ; return;}	/* not exactly WS function */
	case 'H' : { ; return;}		/* not exactly WS function */
	case 'C' : {PT () ; return;}	/* not exactly WS function */
	case 'Y' : {DT () ; return;}	/* not exactly WS function */
	case 'V' : {PT (); return;}	/* not exactly WS function */
	case 'W' : {WB (); return;}	/* not exactly WS function */
	case 'N' : { ; return;}		/* not exactly WS function */
	case 'R' : {INSFILE (); return;}
	case 'L' : {CHDI (); return;}
/*
^K  0-9 set/hide marker
    B/K block begin/end
    H block hide
    C/Y/V/W block copy/delete/move/write
    N column block
*/
	default : {
		   BAD (c);
		   return;
		  }
  }
}

/*
 * Interpret control-O commands.
 */
void
ctrlO ()
{
  character c;

  if (! char_ready_within (500))
	status_msg ("^O: L/R left/right margins...");
  if (quit == True) return;
  c = readchar ();
  if (quit == True) return;
  clear_status ();
  if ('0' <= c && c <= '9') {return;}
  if (c == '\033' || c == QUITCHAR) {CANCEL (); return;}
  switch (cmd_char (c)) {
	case 'L' : {ADJLM (); return;}
	case 'R' : {ADJRM (); return;}
	case 'G' : {ADJFLM () /* actually paragraph tab */; return;}
/*
^O  L/R/M set left/right margin /release
    I/N set/clear tab
    F ruler from line
    C center line
    S set line spacing
    W toggle word wrap
    T toggle ruler line
    J toggle justify
    V     vari-tabs
    H     hyph-help
    E     soft hyph
    D     print display
    P     page break
*/
	default : {
		   BAD (c);
		   return;
		  }
  }
}

/*
 * Set marker / go to marker.
 */
void
MARKER ()
{
  character c;

/*  if (! char_ready_within (500))*/
  	status_msg ("0..9: set marker / , or blank: default marker");
/*  if (quit == True) return;*/

  c = readchar ();
  if (quit == True) return;
  clear_status ();
  if (c == '\033' || c == QUITCHAR) {CANCEL (); return;}
  if ('0' <= c && c <= '9') {MARKn ((int) c - (int) '0'); return;}
  if (c == ',' || c == '\'' || c == ' ' || c == ']' || c == '\035') {MARK (); return;}
  BAD (c);
  return;
}
void
GOMARKER ()
{
  character c;

/*  if (! char_ready_within (500))*/
  	status_msg ("0..9: go marker / blank: default marker");
/*  if (quit == True) return;*/

  c = readchar ();
  if (quit == True) return;
  clear_status ();
  if (c == '\033' || c == QUITCHAR) {CANCEL (); return;}
  if ('0' <= c && c <= '9') {GOMAn ((int) c - (int) '0'); return;}
  if (c == ',' || c == '.' || c == 'g' || c == 'G' || c == '\'' || 
      c == ' ' || c == ']' || c == '\035') {GOMA (); return;}
  BAD (c);
  return;
}

/*
 * Interpret Escape commands.
 */
void
ESCAPE ()
{
  character c;
  void (* func) ();

  if (! char_ready_within (500))
	status_msg ("ESC(exit) q(uit w(rite e(dit /\\(search) r(eplace g(oto h(elp ...");
  if (quit == True) return;
  c = readchar ();
  if (quit == True) return;
  clear_status ();
  if ('0' <= c && c <= '9') {REPT (c); return;}
  switch (c) {
	case '\033' : {EXED (); return;}
	case 'q' : {QUED (); return;}
	case '/' : {SFW (); return;}
	case '\\' : {SRV (); return;}
	case 's' : {GR (); return;}
	case 'R' : {LR (); return;}
	case 'r' : {REPL (); return;}
	case 'w' : {WT (); return;}
	case 'W' : {WTU (); return;}
	case 'e' : {EDIT (); return;}
	case 'v' : {VIEW (); return;}
	case 'E' : {EDITmode (); return;}
	case 'V' : {VIEWmode (); return;}
	case 'g' : {GOTO (); return;}
	case 'h' : {HELP (); return;}
	case '?' : {FS (); return;}
	case '.' : {RDwin (); return;}
	case ',' : {MARKER (); return;}
	case '\'' : {GOMARKER (); return;}
	case 'i' : {INSFILE (); return;}
	case 'b' : {WB (); return;}
	case '=' : {REPT (' '); return;}
	case 'z' : {SUSP (); return;}
	case 'd' : {CHDI (); return;}
	case '!' : {SH (); return;}
	case '^' : {MARK (); return;}
	case ']' : {GOMA (); return;}
	case 'n' : {NN (); return;}
	case 'p' : {PBUF (); return;}
	case 'c' : {CMD (); return;}
	case ' ' : return;
	case 'u' : {display_code (); return;}
	case 'U' : {changeuni (); return;}
	case 'X' : {changehex (); return;}
	case 'O' : {changeoct (); return;}
	case 'D' : {changedec (); return;}
	case '+' : {NXTFILE (); return;}
	case '-' : {PRVFILE (); return;}
	case '#' : {NTHFILE (); return;}
	case 'm' : {screensmaller (); return;}
	case 'M' : {screenbigger (); return;}
	case 'l' : {screenlesslines (); return;}
	case 'L' : {screenmorelines (); return;}
	case 'j' : {JUS (); return;}
	case '<' : {ADJLM (); return;}
	case ';' : {ADJFLM (); return;}
	case ':' : {ADJNLM (); return;}
	case '>' : {ADJRM (); return;}
	case 'P' : {ADJPAGELEN (); return;}
	case 'H' : {HTML (); return;}
	case '_' : {UML (); return;}
	case 'f' : {LOWCAP (); return;}
	case '(' : {SCORR (); return;}
	case ')' : {SCORR (); return;}
	case '{' : {SCORR (); return;}
	case '}' : {SCORR (); return;}
	case 'a' : {toggleappend (); return;}
	case 'o' : {TOGINS (); return;}
	case QUITCHAR : {CANCEL (); return;}
	default : {
		   func = key_map [c];
		   if ((c < ' ') || ((voidfunc) func == (voidfunc) FUNKEY)) {
	/* (voidfunc) is an identity cast here. It seems to be required for
	   the sake of the apparently totally rotten microvax C compiler */
			hop_flag = 1;
			(* func) (c);
		   }
		   else
			BAD (c);
		   return;
		  }
  }
}

/*
 * MOUSEescape () is invoked after a mouse escape sequence with 
 * coordinates and button info already stored in 
 * mouse_xpos, mouse_ypos, and mouse_button.
 * Then it performs a menu or other mouse controlled function.
 */
void
MOUSEescape ()
{
  if (mouse_ypos == -1) {	/* menu stuff */
	MINMEN (mouse_xpos);
  } else if (mouse_ypos == YMAX) {
	if (mouse_button == leftbutton) PD ();
	else if (mouse_button == middlebutton) FS ();
	else if (mouse_button == rightbutton) PU ();
  } else if (scrollbar == True && mouse_xpos == XMAX) {
	if (mouse_button == leftbutton) PD ();
	else if (mouse_button == rightbutton) PU ();
	else if (mouse_button == middlebutton
		|| (mouse_button == releasebutton && mouse_lastbutton == middlebutton))
		goproz (mouse_ypos * 100 / YMAX);
  } else if (quickmenu == True) {
	if (mouse_ypos > last_y) mouse_ypos = last_y;
	if (mouse_button == leftbutton) {
		move_to (mouse_xpos, mouse_ypos);
		report_release = True;
	} else if (mouse_button == middlebutton) {
		FS ();
	} else if (mouse_button == rightbutton) {
		move_to (mouse_xpos, mouse_ypos);
		QUICKMEN (mouse_xpos, mouse_ypos);
	} else if (mouse_button == releasebutton 
		&& mouse_lastbutton == leftbutton 
		&& (mouse_xpos != mouse_lastxpos || mouse_ypos != mouse_lastypos)) {
		MARK ();
		move_to (mouse_xpos, mouse_ypos);
		YA ();
	}
  } else {
	if (mouse_ypos > last_y) mouse_ypos = last_y;
	move_to (mouse_xpos, mouse_ypos);
	if (mouse_button == leftbutton) MARK ();
	else if (mouse_button == middlebutton) PT ();
	else if (mouse_button == rightbutton) YA ();
  }
}

/*
 * REPT () prompts for a count and wants a command after that. It repeats the
 * command count times. If a ^\ is given during repeating, stop looping and
 * return to main loop.
 */
void
REPT (firstdigit)
  char firstdigit;
{
  register int count;
  register void (* func) ();
  int index, number;

  hop_flag = 0;
  if (firstdigit >= '0' && firstdigit <= '9')
     index = get_number ("Please continue repeat count...", firstdigit, & number);
  else
     index = get_number ("Please enter repeat count...", '\0', & number);
  if (index == ERRORS) return;

  func = key_map [index];
  if ((voidfunc) func == (voidfunc) I) {	/* Function assigned? */
	/* (voidfunc) is an identity cast here. It seems to be required for
	   the sake of the apparently totally rotten microvax C compiler */
	clear_status ();
	return;
  }
  if ((voidfunc) func == (voidfunc) FUNKEY) {
	/* (voidfunc) is an identity cast here. It seems to be required for
	   the sake of the apparently totally rotten microvax C compiler */
/*	func = * keyproc;	*/
	func = keyproc;
	keyproc = I;
	index = '\0';
  }

  count = number;
  while (count -- > 0 && quit == False) {
	if (stat_visible == True)
		clear_status ();
	(* func) (index);
	flush ();
  }

  if (quit == True)		/* Abort has been given */
	error ("Repeat aborted", NIL_PTR);
  else
	clear_status ();
}

/*
 * Complains to illegal commands and eats up illegal escape sequences.
 */
void
BAD (c)
  character c;
{
  static char message2 [] = "'**' - type a blank";

  if (c < ' ')	{ message2 [1] = '^'; message2 [2] = c + '@'; }
  else		{ message2 [1] = ' '; message2 [2] = c; }
  error ("Unknown command ", message2);
  while (readchar () != ' ' && quit == False) {
	ring_bell ();
	flush ();
  }
  clear_status ();
}

/*
 * Change working directory.
 */
void
CHDI ()
{
  char new_dir [maxLINE_LEN];	/* Buffer to hold new dir. name */

  if (restricted == True)
	{restrictederr (); return;}

#ifdef pc
  build_string (text_buffer, "Drive/Directory: %s, change to:", unnull (getcwd (new_dir, maxLINE_LEN)));
#else
  build_string (text_buffer, "Directory: %s, change to:", unnull (getcwd (new_dir, maxLINE_LEN)));
#endif

  if (get_file (text_buffer, new_dir) != FINE)
	return;
#ifdef msdos
  if (new_dir [0] != '\0' && new_dir [1] == ':')
	if (new_dir [2] == '\0') {
		new_dir [2] = '.';	/* change to current dir. of drive */
		new_dir [3] = '\0';
	}
#endif
  if (chdir (new_dir) == 0) {
#ifdef msdos
	if (new_dir [0] != '\0' && new_dir [1] == ':')
		setdisk (((int) new_dir [0] & (int) '\137') - (int) 'A');
	RD ();	/* disk error dialog may be on screen after chdir */
#endif
	clear_status ();
	overwriteOK = False;	/* Same file base name ... */
	writable = True;
/*	if (viewmode == False)	*/
	    set_modified ();	/* would mean different file now */
  }
  else	{
#ifdef msdos
	RD ();	/* disk error dialog may be on screen */
#endif
	error ("Could not change work dir: ", serror ());
  }
}

/*
 * Print file status.
 */
void
FSTAT ()
{
  fstatus (file_name [0] ? "" : "[buffer]", -1L, -1L);
}
void
FS ()
{
  if (hop_flag > 0)	if (fstat_always == False) fstat_always = True;
			else fstat_always = False;
  else FSTAT ();
}

/*
 * Show Help information on screen
 */
void
HELP ()
{
  if (hop_flag > 0) {
	if (getenv ("MINEDHELPFILE"))
		build_string (text_buffer, (char *) getenv ("MINEDHELPFILE"), mined_dir);
	else	build_string (text_buffer, helpfilename, mined_dir);
	edit_this_file (text_buffer, True);
	return;
  }

  if (getenv ("MINEDHELP"))
	build_string (text_buffer, (char *) getenv ("MINEDHELP"), mined_dir);
  else	build_string (text_buffer, helpcommand, mined_dir);
  clearscreen ();
  status_msg ("Wait for help...");
  flush ();
  raw_mode (OFF);
  system (text_buffer);
  sleep (1);
  raw_mode (ON);
#ifdef pc
  status_msg ("help finished, press a key...");
  flush ();
  (void) readchar ();
#endif
  clear_status ();
  RDwin ();
}

/*
 * Print buffer
 */
void
PBUF ()
{
  int fd;
  char cmd [maxLINE_LEN];	/* Buffer for print command */

  if ((fd = scratch_file (READ, False)) == ERRORS) {
	error ("Buffer is empty.", NIL_PTR);
	return;
  }
  close (fd);
  build_string (cmd, getenv ("MINEDPRINT") ?
		(char *) getenv ("MINEDPRINT") : printcommand, yank_file);
/* Turbo-C wants the cast here since getenv must be declared as far * */
  clear_status ();
  set_cursor (0, YMAX);
  flush ();
  system (cmd);
  sleep (1);
  RDwin ();
}

/*
 * Pipe buffer
 */
void
CMD ()
{
  int fd;
  char cmd [maxLINE_LEN];	/* Buffer for command */
  char command [maxLINE_LEN];	/* Buffer for full command */

  if (restricted == True)
	{restrictederr (); return;}

  if ((fd = scratch_file (READ, False)) == ERRORS) {
	error ("Buffer is empty.", NIL_PTR);
	return;
  }
  close (fd);
  if (get_string ("Command with buffer as input:", cmd, True, "") != FINE)
	return;
  build_string (command, "%s < %s", cmd, yank_file);
  clear_status ();
  set_cursor (0, YMAX);
  flush ();
  raw_mode (OFF);
  system (command);
  sleep (1);
  raw_mode (ON);
  RDwin ();
}

/*
 * Called if an operation is not implemented on this system
 */
void
notavailable ()
{
  error ("Command not available", NIL_PTR);
}

/*
 * Suspend editor after writing back the file.
 */
void
SUSP ()
{
  if (restricted == True)
	{restrictederr (); return;}

  if (cansuspendmyself == True) {
	if (hop_flag == 0 && modified == True)
		if (wrt_text (False) == ERRORS) return;
	set_cursor (0, YMAX);
	raw_mode (OFF);
	suspendmyself ();
	raw_mode (ON);
	clear_status ();
	RDwin ();
  }
  else notavailable ();
}

/*
 * Call an interactive shell.
 */
void
SH ()
{
#ifdef unix
  register int w;
  int pid, status, waiterr;

  if (restricted == True)
	{restrictederr (); return;}

#ifdef FORK
  switch (pid = fork ()) {
#else
  switch (pid = vfork ()) {
#endif
	case -1:			/* Error */
		error ("Cannot fork: ", serror ());
		return;
	case 0:				/* This is the child */
		set_cursor (0, YMAX);
		putchar ('\n');
		raw_mode (OFF);
		if (rpipe == True) {	/* Fix stdin */
			close (STD_IN);
			if (open ("/dev/tty", O_RDONLY, 0) < 0)
				  exit (126);
		}
		execl (getenv ("SHELL"), getenv ("SHELL"), 0);
		_exit (127);	/* Exit with 127 */
		/* NOTREACHED */
	default:			/* This is the parent */
		do {
			w = wait (& status);
		} while (w != -1 && w != pid);
		waiterr = geterrno ();
  }

  raw_mode (ON);
  RDwin ();

  if (w == -1) {
	error ("Wait error: ", serrorof (waiterr));
	if (((status >> 8) == 127) || ((status >> 8) == 126)) sleep (2);
  }
  if ((status >> 8) == 127)		/* Child died with 127 */
	error (getenv ("SHELL"), ": cannot exec this ${SHELL} (not found / not enough memory ?)");
  else if ((status >> 8) == 126)
	error ("Cannot open /dev/tty as fd #0", NIL_PTR);
#else
# ifdef msdos
  char old_dir [maxLINE_LEN];	/* Buffer to hold dir. name */

  if (restricted == True)
	{restrictederr (); return;}

  (void) getcwd (old_dir, maxLINE_LEN);

  set_cursor (0, YMAX);
  raw_mode (OFF);
  system ("COMMAND.COM");
  raw_mode (ON);
  clear_status ();
  RDwin ();

  if (chdir (old_dir) == 0) {
	if (old_dir [0] != '\0' && old_dir [1] == ':')
		setdisk (((int) old_dir [0] & (int) '\137') - (int) 'A');
	RD ();	/* disk error dialog may be on screen after chdir */
  } else {
	overwriteOK = False;	/* Same file base name ... */
	writable = True;
/*	if (viewmode == False)	*/
	    set_modified ();	/* would mean different file now */
	RD ();	/* disk error dialog may be on screen */
	error ("Could not reset previous work dir: ", serror ());
  }

# else
  if (restricted == True)
	{restrictederr (); return;}
#  ifdef vms
/* Who can tell me why this hangs the process after return from the CLI ?
  set_cursor (0, YMAX);
  raw_mode (OFF);
  system ("SPAWN");
  raw_mode (ON);
  clear_status ();
  RDwin ();
*/
  notavailable ();
#  else
  notavailable ();
#  endif
# endif
#endif
}


/*  ==================================================================	*
 *				Main					*
 *  ==================================================================	*/

char * minedopt;

void
WordStar_keys ()
{
  int i;

  for (i = 0; i < 32; i ++) key_map [i] = ws_key_map [i];
  control_prefix = '\020';
}

FLAG
eval_option ()
{
  switch (* minedopt) {
	case '+': plus_opt = True; break;
	case 'M': MENU = 1 - MENU; break;
	case 'v': viewonly = True; break;
	case 'm': multiexit = True; break;
	case 'P': proportional = True; break;
	case 'p': paradisp = True; break;
	case 'r': RET_opt = 'r'; break;
	case 'R': RET_opt = 'R'; break;
	case '-': restricted = True; break;
	case 'C': auto_UTF_detect = False;
		  utf8_text = False;
		  Chinese = True;
		  break;
	case 'U': if (utf8_input == True) {
			utf8_screen = False;
			utf8_input = False;
		  } else {
			utf8_screen = True;
			utf8_input = True;
		  }
		  break;
	case 'u': auto_UTF_detect = False;
		  if (plus_opt == True)
			utf8_text = False;
		  else
			utf8_text = True;
		  break;
	case 'w': if (wordnonblank == True)
			wordnonblank = False;
		  else
			wordnonblank = True;
		  break;
	case 'a': append_flag = True; break;
	case 'j': JUSlevel = 1; break;
	case 'J': JUSlevel = 2; break;
	case 'B': /*
		  key_map ['\010'] = DPC;
		  key_map ['\177'] = DCC;
		  */
		  key_map ['\010'] = MLF;
		  key_map ['\177'] = DPC;
		  break;
	case 'T': tab_left = False; break;
	case 'D': if (transdeutsch == True) auto_transdeutsch = True;
		  transdeutsch = True;
		  break;
	case 'W': WordStar_keys (); break;
	case 's': page_stay = True; break;
	case 'S': page_scroll = True; break;
	case 'x': xprot = exeprot; break;
	case 't': minedopt ++;
		  if (* minedopt == '\0') {minedopt --; TABchar = TABdefault;}
		  else TABchar = * minedopt;
		  break;
	case 'L': minedopt ++;
		  if (* minedopt == '\0') minedopt --;
		  else linesplitmarker = * minedopt;
		  break;
	case 'd': minedopt ++;
		  if (* minedopt == '-') display_delay = -1;
		  else if (* minedopt >= '0' && * minedopt <= '9')
			display_delay = (int) * minedopt - (int) '0';
		  else minedopt --;
		  break;
	case '/': minedopt ++;
		  inisearch = minedopt;
		  break;
	default:  return False;
  }
  return True;
}

int
main (argc, argv)
  int argc;
  char * argv [];
{
  register int index;	/* index in key table */
  int initlinenum;
  int initlini = 0;
  LINE * initline;
  char * Mark;
  FLAG goon;

/* fprot = umask (0); */

  build_string (mined_dir, argv [0]);
  index = 0;
  while (mined_dir [index] != '\0') index ++;
  while (index >= 0 && mined_dir [index] != '/'
#ifdef msdos
		&& mined_dir [index] != '\\' && mined_dir [index] != ':'
#endif
#ifdef vms
		&& mined_dir [index] != ']' && mined_dir [index] != ':'
#endif
	) index --;
  index ++;
  if (index == 0) {
	mined_dir [index] = '.';
	index ++;
  }
  mined_dir [index] = '\0';

  if (getenv ("NoCtrlSQ") || getenv ("NoControlSQ")) {
	/* ^S and ^Q may come arbitrarily from terminal, so don't use them */
	controlQS = True;
	key_map ['\021'] = I;
	key_map ['\023'] = I;
  }
/*  if (getenv ("MINEDMULT")) multiexit = True; */
  if (getenv ("MINEDPROP")) proportional = True;
  if (getenv ("utf8_screen")) utf8_screen = True;
  if (getenv ("utf8_input")) utf8_input = True;
  if (getenv ("utf8_term")) {
	utf8_screen = True;
	utf8_input = True;
  }
  if (getenv ("MINEDCHIN")) Chinese = True;
  if (getenv ("MINEDMAC")) RET_opt = 'R';
  if (getenv ("MINEDWS")) WordStar_keys ();
  transout = (char *) getenv ("MINEDOUT");
  if (transout != NIL_PTR && * transout != '\0') {
	translate_output = True;
	translen = strlen (transout);
  }
#ifdef unix
  if (getenv ("MINEDMODF")) mined_modf = getenv ("MINEDMODF");
#endif
#ifdef pc
  if (getenv ("windir")) display_delay = 0;
#endif

  Mark = (char *) getenv ("MINEDSHIFT");
/* Turbo-C wants the cast here since getenv must be declared as far * */
  if (Mark != NIL_PTR) {
	SHIFT_MARK = Mark [0];
	if (Mark [0] != '\0') SHIFT_BEG = Mark [1];
  }

  Mark = (char *) getenv ("MINEDTAB");
  if (Mark != NIL_PTR) {
/*	TABchar = (Mark [0] == '\0' ? TABdefault : Mark [0]);	*/
	if (Mark [0] == '\0') TABchar = TABdefault;
	else {
		TABchar = Mark [0];
		TABchar0 = Mark [1];
		if (TABchar0 != '\0') TABchar2 = Mark [2];
		if (TABchar2 != '\0') {
			TABcharmid = Mark [3];
			TABchar0 = '\0';
		}
	}
  } else
	TABchar = TABdefault;
  UTF_TAB = (char *) getenv ("MINEDUTFTAB");
  if (UTF_TAB != NIL_PTR) {
	UTF_TAB0 = UTF_TAB;
	if (* UTF_TAB0 != '\0') advance_utf8 (& UTF_TAB0);
	UTF_TAB2 = UTF_TAB0;
	if (* UTF_TAB2 != '\0') advance_utf8 (& UTF_TAB2);
	UTF_TABmid = UTF_TAB2;
	if (* UTF_TABmid != '\0') {
		advance_utf8 (& UTF_TABmid);
		if (* UTF_TABmid != '\0') UTF_TAB0 = "";
	}
  }

  Mark = (char *) getenv ("MINEDRET");
  if (Mark != NIL_PTR) {
	RET_MARK = Mark [0];
	if (RET_MARK != '\0') RET_BLANK = Mark [1];
	if (RET_BLANK != '\0') RET_BLANK2 = Mark [2];
  } else
	RET_MARK = RETdefault;
  Mark = (char *) getenv ("MINEDPARA");
  if (Mark != NIL_PTR) {
	PARA_MARK = Mark [0];
  } else
	PARA_MARK = PARAdefault;
  UTF_RET = (char *) getenv ("MINEDUTFRET");
  if (UTF_RET != NIL_PTR) {
	UTF_RETblank = UTF_RET;
	if (* UTF_RETblank != '\0') advance_utf8 (& UTF_RETblank);
	UTF_RETblank2 = UTF_RETblank;
	if (* UTF_RETblank2 != '\0') advance_utf8 (& UTF_RETblank2);
  }
  UTF_PARA = (char *) getenv ("MINEDUTFPARA");

  dimansi = (char *) getenv ("MINEDDIM");
  if (dimansi == NIL_PTR) dimansi = "31";
  uniansi = (char *) getenv ("MINEDUNI");
  if (uniansi == NIL_PTR) uniansi = "46";
  else if ((character) * uniansi > '9') {
	UNI_MARK = * uniansi;
	do uniansi ++;
	while (* uniansi == ' ');
  }
  ctrlansi = (char *) getenv ("MINEDCTRL");
  if (ctrlansi == NIL_PTR) ctrlansi = "";
  menuansi = (char *) getenv ("MINEDMENU");
  if (menuansi == NIL_PTR) menuansi = "";
  diagansi = (char *) getenv ("MINEDDIAG");
  if (diagansi == NIL_PTR) diagansi = "";

  if ((minedopt = (char *) getenv ("MINED")) != NIL_PTR)
     while (* minedopt != '\0') {
	(void) eval_option ();
	minedopt ++;
  }

  fnami = 1;
  goon = True;
  do {
    if (fnami < argc) {
      if (* argv [fnami] == '+' && argv [fnami] [1] >= '0' && argv [fnami] [1] <= '9') {
	initlini = fnami;
	fnami += 1;
      }
      else if (* argv [fnami] == '-'
	      || * argv [fnami] == '+'
#ifdef msdos
	      || * argv [fnami] == '/'
#endif
	      ) {
	minedopt = argv [fnami];
	if (* minedopt == '+')
		plus_opt = True;
	else
		plus_opt = False;
	minedopt ++;
	goon = eval_option ();
	fnami += 1;
      }
      else goon = False;
    } else goon = False;
  } while (goon == True);

  get_term ();

  fnami_min = fnami;
  fnami_max = argc - 1;
  fnami_cnt = argc - fnami_min;
  fnamv = argv;	/* Why did this produce a warning? C is such a stupid language! */
  if (! (fnami < argc))
     fnami = 0;

  if (! isatty (STD_IN)) {	/* Reading from pipe */
	if (fnami != 0) {
		panic ("Cannot read both pipe and file", NIL_PTR);
	}
	rpipe = True;
	set_modified ();	/* Set modified flag not to loose buffer */
#ifdef msdos
	panic ("Cannot edit after input from pipe", "MSDOS C incompatibility");
#else
	if ((input_fd = open ("/dev/tty", O_RDONLY, 0)) < 0)
	   panic ("Cannot open /dev/tty for read", serror ());
#endif
  }
  if (! isatty (STD_OUT)) {
	wpipe = True;
	set_modified (); /* Set modified flag not to ignore buffer on exit */
    /*	if ((output_fd = open ("/dev/tty", O_WRONLY, 0)) < 0)
	   panic ("Cannot open /dev/tty for write", serror ()); */
  }

  raw_mode (ON);	/* Set tty to appropriate mode */
  key_map [erase_char] = DPC;
  clearscreen ();

/*
 * Generate names of paste files and of panic-file
 */
#ifdef unix
  temp_dir = getenv ("TMPDIR");
  if (temp_dir == NIL_PTR || temp_dir [0] == '\0') temp_dir = "/tmp";
  if (getenv ("USER")) {
	build_string (yankie_file, "%s/minedbuf.%s.", temp_dir, getenv ("USER"));
	build_string (panic_file, "%s/minedpanic.%s.%d", temp_dir, getenv ("USER"), getpid ());
  }
  else {
	build_string (yankie_file, "%s/minedbuf.%d.", temp_dir, geteuid ());
	build_string (panic_file, "%s/minedpanic.%d.%d", temp_dir, geteuid (), getpid ());
  }
#endif
#ifdef vms
  if (getenv ("SYS$SCRATCH"))
	temp_dir = "SYS$SCRATCH";
  else	temp_dir = "SYS$LOGIN";
  if (getenv ("USER")) {
	build_string (yankie_file, "%s:$MINEDBUF$%s.", temp_dir, getenv ("USER"));
	build_string (panic_file, "%s:$MINEDPANIC$%s.%d", temp_dir, getenv ("USER"), getpid ());
  }
  else {
	build_string (yankie_file, "%s:$MINEDBUF$%d.", temp_dir, geteuid ());
	build_string (panic_file, "%s:$MINEDPANIC$%d.%d", temp_dir, geteuid (), getpid ());
  }
#endif
#ifdef msdos
  temp_dir = (char *) getenv ("TEMP");
  if (temp_dir == NIL_PTR || temp_dir [0] == '\0') temp_dir = (char *) getenv ("TMP");
  if (temp_dir == NIL_PTR || temp_dir [0] == '\0') temp_dir = "\\";
  build_string (yankie_file, "%s\\minedbuf.", temp_dir);
  build_string (panic_file, "%s\\mined-pa.nic", temp_dir);
#endif

  header = tail = alloc_header (); /* Make header of list */
  if (header == NIL_LINE) panic ("Cannot allocate memory", NIL_PTR);
  header->text = NIL_PTR;
  header->next = tail->prev = header;

/* Load the file (if any) */
  if (fnami == 0)
	load_file (NIL_PTR, False);
  else {
	/* This should be applied to all file names, or better, not at all:
	if (length_of (argv [fnami]) > maxLINE_LEN) {
		argv [fnami] [maxLINE_LEN] = '\0';
	}
	*/
	load_wild_file (argv [fnami], False);
  }
  loading = True;	/* keep loading flag True until entering main loop */

  if (initlini != 0) {
     make_number (& initlinenum, argv [initlini] + 1);
     if (initlinenum > 0) {
	if (initlinenum <= 0 || (initline = proceed (header->next, initlinenum - 1)) == tail)
	   error ("Illegal line number: ", num_out ((long) initlinenum));
	else {
	   move_to (x, find_y_w_o_RD (initline));
	   fstatus ("Read", -1L, -1L);
	}
     }
  }

  if (inisearch != NIL_PTR) {
	search_for (inisearch, FORWARD);
	fstatus ("Read", -1L, -1L);
  }

  if (wpipe == True) {
	file_name [0] = '\0'; /* don't let user believe he's editing a file */
	fstatus ("Editing for standard output", -1L, -1L);
  }
  RD ();
  flush ();
  catch_signals (catch_interrupt);
  loading = False;

/* Main loop of the editor */
  for (;;) {
	if (fstat_always == True && stat_visible == False)
		FSTAT ();
	else if (always_disp_code == True && stat_visible == False)
		display_the_code ();

	if (MENU && top_line_scrolled == True) {
		displaymenuline ();
		set_cursor_xy ();
		flush ();
	}
	index = readchar ();
	if (stat_visible == True)
		clear_status ();
	if (quit == False) {	/* Call the function for the typed key */
		(* key_map [index]) (index);
		if (hop_flag > 0) hop_flag --;
		flush ();	/* Flush output (if any) */
	}
	if (quit == True) {
		CANCEL ();
		quit = False;
	}
  }
  /* NOTREACHED */
}


/*  ==================================================================	*
 *				End					*
 *  ==================================================================	*/
