/*  ==================================================================	*
 *				Editor mined				*
 *				Part 2					*
 *  ==================================================================	*/

#include "mined.h"


/*  ==================================================================	*
 *			Definitions specific for mined2.c		*
 *  ==================================================================	*/

#ifndef copycommand
# ifdef vms
# define copycommand "copy %s %s"
# endif
# ifdef msdos
# define copycommand "copy %s %s > nul:"
# endif
#endif

/*
 * viewonlyerr () outputs an error message with a beep
 */
void
viewonlyerr ()
{
  ring_bell ();
  error ("View only mode", NIL_PTR);
}

/*
 * restrictederr () outputs an error message with a beep
 */
void
restrictederr ()
{
  ring_bell ();
  error ("Restricted mode", NIL_PTR);
}

FLAG yank_status = NOT_VALID;	/* Status of yank_file */
FLAG redraw_pending = False;	/* was a redraw suppressed in find_y ? */


/*  ==================================================================	*
 *				Forward declarations			*
 *  ==================================================================	*/

int delete_text ();
void file_insert ();
void yank ();
int compile ();
int reverse_scroll ();
int forward_scroll ();
int insert ();
int legal ();
FLAG checkmark ();


/*  ==================================================================	*
 *				Move Commands				*
 *  ==================================================================	*/

/*
 * Move one line up.
 */
void
MUP ()
{
  if (hop_flag > 0) HIGH ();
  else if (y == 0) {		/* Top line of screen. Scroll one line */
	if (reverse_scroll (True) != ERRORS) {
		move_y (y);
	}
  }
  else			/* Move to previous line */
	move_y (y - 1);
}

/*
 * Move one line down.
 */
void
MDN ()
{
  if (hop_flag > 0) LOW ();
  else if (y == last_y) {	/* Last line of screen. Scroll one line */
	if (bot_line->next == tail && bot_line->text [0] != '\n') {
		return;
	}
	else {
		(void) forward_scroll (True);
		move_y (y);
	}
  }
  else			/* Move to next line */
	move_y (y + 1);
}

/*
 * Move left one position.
 */
void
MLF ()
{
  if (hop_flag > 0) BLINE ();
  else if (x == 0 && get_shift (cur_line->shift_count) == 0) {/* Begin of line */
	if (cur_line->prev != header) {
		MUP ();					/* Move one line up */
		move_to (LINE_END, y);
	}
  }
  else
	move_to (x - 1, y);
}

/*
 * Move right one position.
 */
void
MRT ()
{
  if (hop_flag > 0) ELINE ();
  else if (* cur_text == '\n') {
	if (cur_line->next != tail) {		/* Last char of file */
		MDN ();				/* Move one line down */
		move_to (LINE_START, y);
	}
  }
  else
	move_to (x + 1, y);
}

/*
 * Move to top of screen
 */
void
HIGH ()
{
  move_y (0);
}

/*
 * Move to bottom of screen
 */
void
LOW ()
{
  move_y (last_y);
}

/*
 * Move to begin of line.
 */
void
BLINE ()
{
  move_to (LINE_START, y);
}

/*
 * Move to end of line.
 */
void
ELINE ()
{
  move_to (LINE_END, y);
}

/*
 * GOTO () prompts for a linenumber and moves to that line.
 */
void
goline (number)
  int number;
{
  LINE * line;
  if (number <= 0 || (line = proceed (header->next, number - 1)) == tail)
	error ("Illegal line number: ", num_out ((long) number));
  else	{
	  clear_status ();
	  move_y (find_y (line));
	}
}

void
goproz (number)
  int number;
{
  goline ((total_lines - 1) * number / 100 + 1);
}

void
GOTO ()
{
  character c;
  char end;
  int number;

  if (MENU) {
	hop_flag = 1;
	displayflags ();
	set_cursor_xy ();
	flush ();
	hop_flag = 0;
  }
  if (! char_ready_within (500)) status_msg ("HOP: type command (fortified) or number...");
  if (quit == True) return;
  c = readchar ();
  if (quit == True) return;
  if ('0' <= c && c <= '9') {
	if (lines_per_page > 0)
		end = get_number ("...number, then go line / %(of lines) / p(age / m(ark / g(o marker / f(ile #", c, & number);
	else	end = get_number ("...number, then go line / %(of lines) / m(ark / g(o marker / f(ile #", c, & number);
	if (end == '%')
		goproz (number);
	else if (end == 'm' || end == 'M' || end == ',')
		MARKn (number);
	else if (end == '\'' || end == '.' || end == 'g' || end == 'G')
		GOMAn (number);
	else if (end == 'f' || end == 'F' || end == '#')
		edit_nth_file (number);
	else if (lines_per_page > 0 && (end == 'p' || end == 'P') && number > 0)
		goline (number * lines_per_page - lines_per_page + 1);
	else if (end != ERRORS)
		goline (number);
	return;
  }
  else {
	clear_status ();
	hop_flag = 1;
	(* key_map [c]) (c);
	return;
  }
}

/*
 * Scroll forward one page or to eof, whatever comes first. (Bot_line becomes
 * top_line of display.) Try to leave the cursor on the same line. If this is
 * not possible, leave cursor on the line halfway the page.
 */
void
PD ()
{
  register int i;
  int new_y;

  if (hop_flag > 0) {hop_flag = 0; EFILE (); return;}

  for (i = 0; i < SCREENMAX; i ++)
	if (forward_scroll (page_scroll) == ERRORS)
		break;			/* EOF reached */

  if (y - i < 0)			/* Line no longer on screen */
	new_y = (page_stay == True) ? 0 : SCREENMAX >> 1;
  else	new_y = y - i;

  if (page_scroll == False) display (0, top_line, last_y, new_y);

  move_y (new_y);
}

/*
 * Scroll backwards one page or to top of file, whatever comes first.
 * (Top_line becomes bot_line of display).
 * The very bottom line (YMAX) is always blank.
 * Try to leave the cursor on the same line.
 * If this is not possible, leave cursor on the line halfway the page.
 */
void
PU ()
{
  register int i;
  int new_y;

  if (hop_flag > 0) {hop_flag = 0; BFILE (); return;}

  for (i = 0; i < SCREENMAX; i ++) {
	if (reverse_scroll (page_scroll) == ERRORS)
		/* should also flag reverse_scroll that clearing of 
		   bottom line is not desired */
		break;			/* Top of file reached */
  }

  if (y + i > SCREENMAX)		/* line no longer on screen */
	new_y = (page_stay == True) ? last_y : SCREENMAX >> 1;
  else
	new_y = y + i;

  if (can_scroll_reverse == True && page_scroll == True) {
	set_cursor (0, YMAX);	/* Erase very bottom line */
	clear_lastline ();
  }
  else display (0, top_line, last_y, new_y);

  move_y (new_y);
}

/*
 * Go to top of file, scrolling if possible, else redrawing screen.
 */
void
BFILE ()
{
  if (proceed (top_line, - SCREENMAX) == header)
	PU ();			/* It fits. Let PU do it */
  else {
	reset (header->next, 0);/* Reset top_line, etc. */
	RD_y (0);		/* Display full page */
  }
  move_to (LINE_START, 0);
}

/*
 * Go to last position of text, scrolling if possible, else redrawing screen
 */
void
EFILE ()
{
  if (proceed (bot_line, SCREENMAX) == tail)
	PD ();			/* It fits. Let PD do it */
  else {
	reset (proceed (tail->prev, - SCREENMAX), SCREENMAX);
	RD_y (last_y);		/* Display full page */
  }
  move_to (LINE_END, last_y);
}

/*
 * Scroll one line up. Leave the cursor on the same line (if possible).
 */
void
SU ()
{
  register int i;

  if (hop_flag > 0) {
	hop_flag = 0;
	for (i = 0; i < (SCREENMAX >> 1); i ++) SU ();
	return;
  }

  if (reverse_scroll (True) != ERRORS) {	/* else we are at top of file */
	move_y ((y == SCREENMAX) ? SCREENMAX : y + 1);
  }
}

/*
 * Scroll one line down. Leave the cursor on the same line (if possible).
 */
void
SD ()
{
  register int i;

  if (hop_flag > 0) {
	hop_flag = 0;
	for (i = 0; i < (SCREENMAX >> 1); i ++) SD ();
	return;
  }

  if (forward_scroll (True) != ERRORS)
	move_y ((y == 0) ? 0 : y - 1);
}

/*
 * Perform a forward scroll. It returns ERRORS if we're at the last line of
 * the file.
 */
int
forward_scroll (update)
  FLAG update;
{
  if (bot_line->next == tail)		/* Last line of file. No dice */
	return ERRORS;
  top_line = top_line->next;
  bot_line = bot_line->next;
  cur_line = cur_line->next;

/* Perform the scroll on screen */
  if (update == True) {
	clear_menus ();
	scroll_forward ();
	set_cursor (0, SCREENMAX);
	line_print (bot_line);
  }

  return FINE;
}

/*
 * Perform a backwards scroll. It returns ERRORS if we're at the first line 
 * of the file. It updates the display completely if update is True. 
 * Otherwise it leaves that to the caller (page up function).
 */
int
reverse_scroll (update)
  FLAG update;
{
  if (top_line->prev == header)
	return ERRORS;		/* Top of file. Can't scroll */

  if (last_y != SCREENMAX)	/* Reset last_y if necessary */
	last_y ++;
  else
	bot_line = bot_line->prev;	/* Else adjust bot_line */
  top_line = top_line->prev;
  cur_line = cur_line->prev;

/* Perform the scroll on screen */
  if (update == True)
    if (can_scroll_reverse == True) {
	clear_menus ();
	set_cursor (0, 0);
	scroll_reverse ();
	set_cursor (0, YMAX);	/* Erase very bottom line */
	clear_lastline ();
	set_cursor (0, 0);
	line_print (top_line);
    }
    else display (0, top_line, last_y, y);

  return FINE;
}

/*----------------------*
 *	Word moves	*
 *----------------------*/

/*
 * A word was previously defined as a number of non-blank characters
 * separated by tabs, spaces or linefeeds.
 * By consulting idfchar (), sequences of real letters only or digits
 * or underlines are recognized as words.
 */
extern int idfchar ();

/*
 * BSEN () and ESEN () look for the beginning or end of the current sentence.
 */
void
BSEN ()
{
  search_for ("[;.]", REVERSE);
}

void
ESEN ()
{
  search_for ("[;.]", FORWARD);
}

/*
 * SCORR () looks for a corresponding bracket ([<->] etc.)
 */
void
SCORR ()
{
  switch (* cur_text) {
	case '(':	search_corresponding ("[()]", FORWARD, ')'); return;
	case ')':	search_corresponding ("[()]", REVERSE, '('); return;
	case '[':	search_corresponding ("[\\[\\]]", FORWARD, ']'); return;
	case ']':	search_corresponding ("[\\[\\]]", REVERSE, '['); return;
	case '{':	search_corresponding ("[{}]", FORWARD, '}'); return;
	case '}':	search_corresponding ("[{}]", REVERSE, '{'); return;
	case '<':	search_corresponding ("[<>]", FORWARD, '>'); return;
	case '>':	search_corresponding ("[<>]", REVERSE, '<'); return;
	case '':	search_corresponding ("[]", FORWARD, ''); return;
	case '':	search_corresponding ("[]", REVERSE, ''); return;
	default:	error ("No bracket", NIL_PTR); return;
  }
}

/*
 * SIDF () searches for the identifier at the current position
 */
void
SIDF (method)
  FLAG method;
{
  char idf_buf [MAX_CHARS];	/* identifier to search for */
  char * idf_buf_poi = idf_buf;
  char * idf_poi;

  if (! idfchar (* cur_text)) {
	error ("No identifier", NIL_PTR);
	return;
  } else {
	idf_poi = cur_text;
	while (idfchar (* idf_poi) && idf_poi != cur_line->text) idf_poi --;
	if (! idfchar (* idf_poi)) idf_poi ++;
	while (idfchar (* idf_poi)) * idf_buf_poi ++ = * idf_poi ++;
	* idf_buf_poi = '\0';
	search_for (idf_buf, method);
  }
}

/*
 * MPW () moves to the start of the previous word. A word is defined as a
 * number of non-blank characters separated by tabs spaces or linefeeds.
 */
void
move_previous_word (remove)
  FLAG remove;
{
  register char * begin_line;
  char * textp;
  char start_char = * cur_text;
  char * start_pos = cur_text;
  FLAG idfsearch;

  if (remove == DELETE && viewonly == True)
	{viewonlyerr (); return;}

/* First check if we're at the beginning of line. */
  if (cur_text == cur_line->text) {
	if (cur_line->prev == header)
		return;
	start_char = '\0';
  }

  MLF ();

  begin_line = cur_line->text;
  textp = cur_text;

/* Check if we're in the middle of a word. */
  if (! alpha (* textp) || ! alpha (start_char)) {
	while (textp != begin_line && (white_space (* textp) || * textp == '\n'))
		precede_char (& textp, begin_line);
  }

/* Now we're at the end of previous word. Skip non-blanks until a blank comes */
  if (wordnonblank == True) {
	while (textp != begin_line && alpha (* textp))
		precede_char (& textp, begin_line);
  } else {
	if (idfchar (* textp)) {
		idfsearch = True;
		while (textp != begin_line && idfchar (* textp))
			precede_char (& textp, begin_line);
	}
	else {
		idfsearch = False;
		while (textp != begin_line && alpha (* textp) && ! idfchar (* textp))
			precede_char (& textp, begin_line);
	}
  }

/* Go to the next char if we're not at the beginning of the line */
/* At the beginning of the line, check whether to stay or to go to the word */
  if (textp != begin_line && * textp != '\n')
	advance_char (& textp);
  else if (textp == begin_line && * textp != '\n' &&
	   ((wordnonblank == True) ? * textp == ' '
		:   ((idfsearch == True) ? ! idfchar (* textp)
			: (! alpha (* textp) || idfchar (* textp))))) {
	advance_char (& textp);
	if (white_space (* textp) || textp == start_pos)
		/* no word there or not moved, so go back */
		precede_char (& textp, begin_line);
  }

/* Find the x-coordinate of this address, and move to it */
  move_address (textp, y);
  if (remove == DELETE)
	(void) delete_text (cur_line, textp, cur_line, start_pos);
}

void
MPW ()
{
  if (hop_flag > 0) BSEN ();
  else move_previous_word (NO_DELETE);
}

/*
 * MNW () moves to the start of the next word. A word is defined as a number of
 * non-blank characters separated by tabs spaces or linefeeds. Always keep in
 * mind that the pointer shouldn't pass the '\n'.
 */
void
move_next_word (remove)
  FLAG remove;
{
  char * textp = cur_text;

  if (remove == DELETE && viewonly == True)
	{viewonlyerr (); return;}

/* Move to the end of the current word. */
  if (wordnonblank == True) {
	if (* textp != '\n')
		advance_char (& textp);
	while (alpha (* textp))
		advance_char (& textp);
  } else {
	if (idfchar (* textp))
		while (* textp != '\n' && idfchar (* textp))
			advance_char (& textp);
	else
		while (alpha (* textp) && ! idfchar (* textp))
			advance_char (& textp);
  }

/* Skip all white spaces */
  while (* textp != '\n' && white_space (* textp))
	textp ++;
/* If we're deleting, delete the text in between */
  if (remove == DELETE) {
	(void) delete_text (cur_line, cur_text, cur_line, textp);
	return;
  }

/* If we're at end of line, move to the beginning of (first word on) the next line */
  if (* textp == '\n' && cur_line->next != tail) {
	MDN ();
	move_to (LINE_START, y);
	textp = cur_text;
/*	while (* textp != '\n' && white_space (* textp))	*/
/*		textp ++;					*/
  }
  move_address (textp, y);
}

void
MNW ()
{
  if (hop_flag > 0) ESEN ();
  else move_next_word (NO_DELETE);
}

/*
 * find_y () checks if the matched line is on the current page. If it is, it
 * returns the new y coordinate, else it displays the correct page with the
 * matched line in the middle (unless redrawflag is set to False) 
 * and returns the new y value.
 */
int
find_y_RD (match_line, redrawflag)
  LINE * match_line;
  FLAG redrawflag;
{
  register LINE * line;
  register int count = 0;

/* Check if match_line is on the same page as currently displayed. */
  for (line = top_line; line != match_line && line != bot_line->next;
						      line = line->next)
	count ++;
  if (line != bot_line->next)
	return count;

/* Display new page, with match_line in center. */
  if ((line = proceed (match_line, - (SCREENMAX >> 1))) == header) {
  /* Can't display in the middle. Make first line of file top_line */
	count = 0;
	for (line = header->next; line != match_line; line = line->next)
		count ++;
	line = header->next;
  }
  else	/* New page is displayed. Set cursor to middle of page */
	count = SCREENMAX >> 1;

/* Reset pointers and redraw the screen */
  reset (line, 0);
  if (redrawflag == True) {
	RD_y (count);
	redraw_pending = False;
  }
  else	redraw_pending = True;

  return count;
}

int
find_y (match_line)
  LINE * match_line;
{
  return find_y_RD (match_line, True);
}

int
find_y_w_o_RD (match_line)
  LINE * match_line;
{
  return find_y_RD (match_line, False);
}


/*  ==================================================================	*
 *				Modify Commands				*
 *  ==================================================================	*/

/*
 * DCC deletes the character under the cursor. If this character is a '\n' the
 * current line is joined with the next one.
 * If this character is the only character of the line, the current line will
 * be deleted.
 * DCC0 deletes without justification.
 */
void
DCC ()
{
  char * after_char;

  if (* cur_text == '\n')
	if (cur_line->next == tail)
	   return;
	else
	   (void) delete_text (cur_line, cur_text, cur_line->next, cur_line->next->text);
  else {
	after_char = cur_text;
	advance_char (& after_char);
	(void) delete_text (cur_line, cur_text, cur_line, after_char);
  }
}
void
DCC0 ()
{
  DCC ();
}

/*
 * DPC deletes the character on the left side of the cursor.  If the cursor
 * is at the beginning of the line, the last character if the previous line
 * is deleted. With hop flag, delete left part of line from current point.
 * DPC0 deletes without justification.
 */
void
DPC ()
{
  char * delete_pos;

  if (x == 0 && cur_line->prev == header)
	return;			/* Top of file */

  if (viewonly == True)
	{viewonlyerr (); return;}

  if (hop_flag > 0) {
	hop_flag = 0;
	if (cur_text != cur_line->text) {
	  delete_pos = cur_text;
	  BLINE ();
	  (void) delete_text (cur_line, cur_line->text, cur_line, delete_pos);
	}
  }
  else {
	MLF ();			/* Move one left */
	DCC ();			/* Delete character under cursor */
  }
}
void
DPC0 ()
{
  DPC ();
}

/*
 * DLINE delete the whole current line.
 */
void
DLINE ()
{
  if (viewonly == True)
	{viewonlyerr (); return;}

  if (hop_flag > 0) {
    hop_flag = 0;
    if (* cur_text != '\n')
	(void) delete_text (cur_line, cur_text, cur_line, cur_text + length_of (cur_text) - 1);
  }
  else {
/*
    if (cur_line->next == tail)
	(void) delete_text (cur_line, cur_line->text, cur_line, cur_text + length_of (cur_text) - 1);
    else
	(void) delete_text (cur_line, cur_line->text, cur_line->next, cur_line->next->text);
    BLINE ();
*/
    BLINE ();
    if (* cur_text != '\n')
	DLN ();
    DCC ();
  }
}

/*
 * DLN deletes all characters until the end of the line. If the current
 * character is a '\n', then delete that char.
 */
void
DLN ()
{
  if (hop_flag > 0) {
	hop_flag = 0;
	DLINE ();
  }
  else if (* cur_text == '\n')
	DCC ();
  else	(void) delete_text (cur_line, cur_text, cur_line, cur_text + length_of (cur_text) - 1);
}

/*
 * DNW () deletes the next word (as defined in MNW ())
 */
void
DNW ()
{
  if (* cur_text == '\n')
	DCC ();
  else
	move_next_word (DELETE);
}

/*
 * DPW () deletes the previous word (as defined in MPW ())
 */
void
DPW ()
{
  if (cur_text == cur_line->text)
	DPC ();
  else
	move_previous_word (DELETE);
}

/*
 * Insert character `character' at current location.
 * S0 inserts without justification.
 */
void
SNL ()
{
  S ('\n');
}

/*
 * enter a character or a byte of a multi-byte character 
 * (with intermediate storage)
 */
void
S1 (newchar, JUSlvl, utf8_transform)
  register character newchar;
  int JUSlvl;
  FLAG utf8_transform;
{
  static character buffer [7];
  static character * utfpoi = buffer;
  static int utfcount = 1;
  static character firstbyte;
  static int width = 1;
  static unsigned long unichar;

  if (utf8_text == True) {
	if (utf8_transform == True && last_char_latin1 == False) {
	/* UTF-8 input for UTF-8 text */
		if (newchar < 0x80) {
			* utfpoi = newchar;
			utfpoi ++;
			* utfpoi = '\0';
			utfpoi = buffer;
		} else if ((newchar & 0xC0) == 0x80) {
		/* UTF-8 sequence byte */
			* utfpoi = newchar;
			utfpoi ++;
			utfcount --;
			if (utfcount == 0) {
				* utfpoi = '\0';
				utfpoi = buffer;
			}
			else
				return;
		} else { /* first UTF-8 byte */
			utfpoi = buffer;
			* utfpoi = newchar;
			if ((newchar & 0xE0) == 0xC0)
				utfcount = 2;
			else if ((newchar & 0xF0) == 0xE0)
				utfcount = 3;
			else if ((newchar & 0xF8) == 0xF0)
				utfcount = 4;
			else if ((newchar & 0xFC) == 0xF8)
				utfcount = 5;
			else if ((newchar & 0xFE) == 0xFC)
				utfcount = 6;
			else /* ignore illegal UTF-8 code */
				return;
			utfpoi ++;
			utfcount --;
			return;
		}
	} else {
	/* 8-bit input for UTF-8 text */
		if (newchar < 0x80) {
			buffer [0] = newchar;
			buffer [1] = '\0';
		} else {
			buffer [0] = (newchar >> 6) | 0xC0;
			buffer [1] = newchar & 0x3F | 0x80;
			buffer [2] = '\0';
		}
	}
  } else if (Chinese == True) {
	/* 8/16-bit (Chinese) input for Chinese text */
	if (firstbyte != '\0') {
		buffer [0] = firstbyte;
		buffer [1] = newchar;
		width = 2;
	} else if (multichar (newchar)) {
		firstbyte = newchar;
		return;
	} else {
		buffer [0] = newchar;
		buffer [1] = '\0';
		width = 1;
	}
	firstbyte = '\0';
  } else if (utf8_transform == True && last_char_latin1 == False) {
	/* UTF-8 input for 8-bit text */
	if (newchar < 0x80) {
		buffer [0] = newchar;
	} else if ((newchar & 0xC0) == 0x80) {
	/* UTF-8 sequence byte */
		unichar = (unichar << 6) | (newchar & 0x3F);
		utfcount --;
		if (utfcount == 0) {
			if ((unichar & 0xFF) == unichar)
				buffer [0] = unichar & 0xFF;
			else
				buffer [0] = '';
		}
		else
			return;
	} else { /* first UTF-8 byte */
		if ((newchar & 0xE0) == 0xC0) {
			utfcount = 2;
			unichar = newchar & 0x1F;
		} else if ((newchar & 0xF0) == 0xE0) {
			utfcount = 3;
			unichar = newchar & 0x0F;
		} else if ((newchar & 0xF8) == 0xF0) {
			utfcount = 4;
			unichar = newchar & 0x07;
		} else if ((newchar & 0xFC) == 0xF8) {
			utfcount = 5;
			unichar = newchar & 0x03;
		} else if ((newchar & 0xFE) == 0xFC) {
			utfcount = 6;
			unichar = newchar & 0x01;
		} else /* ignore illegal UTF-8 code */
			return;
		utfcount --;
		return;
	}
  } else
	/* 8-bit input for 8-bit text */
	buffer [0] = newchar;

/* Insert the new character */
  if (insert (cur_line, cur_text, buffer) == ERRORS)
	return;

/* Fix screen */
  if (newchar== '\n') {
	set_cursor (0, y);
	if (y == SCREENMAX) {		/* Can't use display () */
		line_print (cur_line);
		(void) forward_scroll (True);
		move_to (0, y);
	}
	else {
		reset (top_line, y);	/* Reset pointers */
		if (can_add_line == True) {
			clear_menus ();
			add_line (y + 1);
			clear_status ();
			display (y, cur_line, 1, y + 1);
		}
		else	display (y, cur_line, last_y - y, y + 1);
		move_to (0, y + 1);
	}
  }
  else if (x + width == XBREAK) /* If line must be shifted, just call move_to */
	move_to (x + width, y);
  else {			/* else display rest of line */
	put_line (cur_line, x, False, False);
	move_to (x + width, y);
  }
  if (JUSlvl > 0)
	JUSandreturn ();
}
void
S (newchar)
  register character newchar;
{
  S1 (newchar, JUSlevel, utf8_input);
}
void
Suni1 (newchar)
  register character newchar;
{
  S1 (newchar, JUSlevel, False);
}
void
Sunicode (newchar)
  unsigned long newchar;
{
  if (newchar < 0x80) {
	S1 (newchar, JUSlevel, True);
  } else if (newchar < 0x800) {
	S1 (0xC0 | (newchar >> 6), JUSlevel, True);
	S1 (0x80 | (newchar & 0x3F), JUSlevel, True);
  } else if (newchar < 0x10000) {
	S1 (0xE0 | (newchar >> 12), JUSlevel, True);
	S1 (0x80 | ((newchar >> 6) & 0x3F), JUSlevel, True);
	S1 (0x80 | (newchar & 0x3F), JUSlevel, True);
  } else if (newchar < 0x200000) {
	S1 (0xF0 | (newchar >> 18), JUSlevel, True);
	S1 (0x80 | ((newchar >> 12) & 0x3F), JUSlevel, True);
	S1 (0x80 | ((newchar >> 6) & 0x3F), JUSlevel, True);
	S1 (0x80 | (newchar & 0x3F), JUSlevel, True);
  } else if (newchar < 0x4000000) {
	S1 (0xF8 | (newchar >> 24), JUSlevel, True);
	S1 (0x80 | ((newchar >> 18) & 0x3F), JUSlevel, True);
	S1 (0x80 | ((newchar >> 12) & 0x3F), JUSlevel, True);
	S1 (0x80 | ((newchar >> 6) & 0x3F), JUSlevel, True);
	S1 (0x80 | (newchar & 0x3F), JUSlevel, True);
  } else if (newchar < 0x80000000) {
	S1 (0xFC | (newchar >> 30), JUSlevel, True);
	S1 (0x80 | ((newchar >> 24) & 0x3F), JUSlevel, True);
	S1 (0x80 | ((newchar >> 18) & 0x3F), JUSlevel, True);
	S1 (0x80 | ((newchar >> 12) & 0x3F), JUSlevel, True);
	S1 (0x80 | ((newchar >> 6) & 0x3F), JUSlevel, True);
	S1 (0x80 | (newchar & 0x3F), JUSlevel, True);
  } else
	error ("Invalid Unicode value", NIL_PTR);
}
void
S0 (newchar)
  register character newchar;
{
  S1 (newchar, 0, utf8_input);
}

/*
 * Replace current character with its hex/octal/decimal representation.
 */
character hexdig (c)
  character c;
{
  if (c < 10) return c + '0';
	else  return c - 10 + 'A';
}
void
insertcode (c, radix)
  character c;
  int radix;
{
  int radix2;

  if (radix == 8) {
	S (hexdig ((c >> 6) & 007));
	S (hexdig ((c >> 3) & 007));
	S (hexdig ((c) & 007));
  } else if (radix == 16) {
	S (hexdig ((c >> 4) & 017));
	S (hexdig ((c) & 017));
  } else {	/* assume radix = 10 or, at least, three digits suffice */
	radix2 = radix * radix;
	S (hexdig (c / radix2));
	S (hexdig ((c % radix2) / radix));
	S (hexdig (c % radix));
  }
}
void
changetocode (radix, unicode)
  int radix;
  FLAG unicode;
{
  character c = * cur_text;
  int utfcount = 1;
  unsigned long unichar;
  char buffer [7];
  char * textpoi;
  char * utfpoi;

  if (c == '\n') {
#ifdef msdos
	insertcode ('\r', radix);
#endif
	insertcode ('\n', radix);
  } else if (utf8_text == True) {
	if ((c & 0x80) == 0x00) {
		utfcount = 1;
		unichar = c;
	} else if ((c & 0xE0) == 0xC0) {
		utfcount = 2;
		unichar = c & 0x1F;
	} else if ((c & 0xF0) == 0xE0) {
		utfcount = 3;
		unichar = c & 0x0F;
	} else if ((c & 0xF8) == 0xF0) {
		utfcount = 4;
		unichar = c & 0x07;
	} else if ((c & 0xFC) == 0xF8) {
		utfcount = 5;
		unichar = c & 0x03;
	} else if ((c & 0xFE) == 0xFC) {
		utfcount = 6;
		unichar = c & 0x01;
	} else /* illegal UTF-8 code */ {
		DCC ();
		insertcode (c, radix);
		error ("Invalid UTF-8 sequence", NIL_PTR);
		return;
	}
	utfcount --;
	utfpoi = buffer;
	* utfpoi ++ = c;
	textpoi = cur_text;
	textpoi ++;
	while (utfcount > 0 && (* textpoi & 0xC0) == 0x80) {
		c = * textpoi ++;
		* utfpoi ++ = c;
		unichar = (unichar << 6) | (c & 0x3F);
		utfcount --;
	}
	* utfpoi = '\0';
	DCC ();
	if (unicode == True) {
		if (unichar > 0xFFFF) {
			insertcode (unichar >> 24, radix);
			insertcode (unichar >> 16, radix);
		}
		insertcode (unichar >> 8, radix);
		insertcode (unichar, radix);
	} else {
		utfpoi = buffer;
		while (* utfpoi != '\0')
			insertcode (* utfpoi ++, radix);
	}
	if (utfcount > 0)
		error ("Invalid UTF-8 sequence", NIL_PTR);
  } else if (Chinese == True && multichar (c)) {
	buffer [0] = c;
	textpoi = cur_text;
	textpoi ++;
	c = * textpoi;
	DCC ();
	insertcode (buffer [0], radix);
	insertcode (c, radix);
  } else {
	DCC ();
	insertcode (c, radix);
  }
}
void
changeuni ()
{
  changetocode (16, True);
}
void
changehex ()
{
  changetocode (16, False);
}
void
changeoct ()
{
  changetocode (8, False);
}
void
changedec ()
{
  changetocode (10, False);
}

/*
 * display_code shows UTF-8 code sequence and Unicode value on the status line
 */
char hexbuf [20];
char * hexbufpoi = hexbuf;

void
appendcode (c)
  character c;
{
	* hexbufpoi ++ = hexdig ((c >> 4) & 0x0F);
	* hexbufpoi ++ = hexdig ((c) & 0x0F);
}
void
display_the_code ()
{
  character c = * cur_text;
  int utfcount = 1;
  unsigned long unichar;
  char * textpoi;
  FLAG invalid = False;

  hexbufpoi = hexbuf;

  if (c == '\n') {
#ifdef msdos
	appendcode ('\r');
#endif
	appendcode ('\n');
  } else if (utf8_text == True) {
	if ((c & 0x80) == 0x00) {
		utfcount = 1;
		unichar = c;
	} else if ((c & 0xE0) == 0xC0) {
		utfcount = 2;
		unichar = c & 0x1F;
	} else if ((c & 0xF0) == 0xE0) {
		utfcount = 3;
		unichar = c & 0x0F;
	} else if ((c & 0xF8) == 0xF0) {
		utfcount = 4;
		unichar = c & 0x07;
	} else if ((c & 0xFC) == 0xF8) {
		utfcount = 5;
		unichar = c & 0x03;
	} else if ((c & 0xFE) == 0xFC) {
		utfcount = 6;
		unichar = c & 0x01;
	} else /* illegal UTF-8 code */ {
		appendcode (c);
		invalid = True;
	}
	if (invalid == False) {
		utfcount --;
		appendcode (c);
		textpoi = cur_text;
		textpoi ++;
		while (utfcount > 0 && (* textpoi & 0xC0) == 0x80) {
			c = * textpoi ++;
			appendcode (c);
			unichar = (unichar << 6) | (c & 0x3F);
			utfcount --;
		}
		if (utfcount > 0)
			invalid = True;
	}
	* hexbufpoi = '\0';
	if (invalid == False) {
	    if (unichar > 0xFFFF) {
		build_string (text_buffer, "UTF-8 sequence: %s, UCS-4: %08X", hexbuf, unichar);
	    } else {
		build_string (text_buffer, "UTF-8 sequence: %s, UCS-2 (Unicode): %04X", hexbuf, unichar);
	    }
	} else
		build_string (text_buffer, "Invalid UTF-8 sequence: %s", hexbuf);
	status_msg (text_buffer);
	return;
  } else if (Chinese == True && multichar (c)) {
	appendcode (c);
	textpoi = cur_text;
	textpoi ++;
	c = * textpoi;
	appendcode (c);
  } else {
	appendcode (c);
  }
  * hexbufpoi = '\0';
  build_string (text_buffer, "Character code: %s", hexbuf);
  status_msg (text_buffer);
}
void
display_code ()
{
  if (hop_flag > 0) {
	hop_flag = 0;
	if (always_disp_code == True)
		always_disp_code = False;
	else	always_disp_code = True;
  }
  else
	display_the_code ();
}

/*
 * Replace current diacritic transcription with real character
 */
void
UML ()
{
  character c1 = * cur_text;
  char * c2poi = cur_text;
  character c2;

  c2poi ++;
  c2 = * c2poi;

  if (c1 == 'a' && c2 == 'e') {DCC (); DCC (); S ('');}
  else if (c1 == 'o' && c2 == 'e') {DCC (); DCC (); S ('');}
  else if (c1 == 'u' && c2 == 'e') {DCC (); DCC (); S ('');}
  else if (c1 == 'A' && c2 == 'e') {DCC (); DCC (); S ('');}
  else if (c1 == 'O' && c2 == 'e') {DCC (); DCC (); S ('');}
  else if (c1 == 'U' && c2 == 'e') {DCC (); DCC (); S ('');}
  else if (c1 == 'A' && c2 == 'E') {DCC (); DCC (); S ('');}
  else if (c1 == 'O' && c2 == 'E') {DCC (); DCC (); S ('');}
  else if (c1 == 'U' && c2 == 'E') {DCC (); DCC (); S ('');}
  else if (c1 == 's' && c2 == 's') {DCC (); DCC (); S ('');}
}

/*
 * Flip lower and upper case letters
 */
void
LOWCAP ()
{
  character c1;

  do {
	c1 = * cur_text;
	if ((c1 >= 'a' && c1 <= 'z')
	 || (c1 >= (character) '' && c1 <= (character) '')
	   ) {
		DCC ();
		S ((character) ((unsigned int) c1 - 0x20));
	}
	else if ((c1 >= 'A' && c1 <= 'Z')
	      || (c1 >= (character) '' && c1 <= (character) '')
		) {
		DCC ();
		S ((character) ((unsigned int) c1 + 0x20));
	}
	else if (c1 == (character) '') {
		DCC ();
		S ('S'); S ('S');
	}
	else move_to (x + 1, y);
  } while (hop_flag > 0 && idfchar (* cur_text));
}

/*
 * insert_accent inserts accented character
 */
void
insert_accent (name, routine)
  char * name;
  character (* routine) ();
{
  register character letter;

  build_string (text_buffer, "Enter character to compose with %s...", name);
  status_msg (text_buffer);
  letter = (* routine) (readchar ());
  clear_status ();
  Suni1 (letter);
}

/*
 * CTRl inserts a control-char at the current location. A message that this
 * function is called is displayed at the status line.
 */
void
CTRl ()
{
  register character ctrl;
  unsigned long unichar;
  char mnemonic [maxLINE_LEN];

  status_msg ("Enter control character / # or compose character...");
  ctrl = readchar ();
  if (ctrl == ring || ctrl == ',')
	{insert_accent ("angstrom/cedilla", angstrom); return;}
  else switch (ctrl) {
	case '"':
	case ':':	{insert_accent ("diaeresis", diaeresis); return;}
	case '\'':	{insert_accent ("acute (d'aigu)", acute); return;}
	case '`':	{insert_accent ("grave", grave); return;}
	case '^':	{insert_accent ("circumflex", circumflex); return;}
	case '~':	{insert_accent ("tilde", tilde); return;}
  }
  if (utf8_text == True) {
    if (ctrl == '#') {
	if (get_hex_number ("Enter hex digits for Unicode value to compose character...", & unichar) 
		!= ERRORS) {
		Sunicode (unichar);
	}
	return;
    } else if (ctrl == ' ') {
	if (get_string ("Enter character mnemonic: ", mnemonic, False, " ") == ERRORS)
		return;
	unichar = compose_mnemo (mnemonic);
	clear_status ();
	Sunicode (unichar);
	return;
    } else if (
	((ctrl > ' ') && (ctrl < 'Z')) || (ctrl > '^')
	) {
	build_string (text_buffer, "Enter character to compose with %c...", ctrl);
	status_msg (text_buffer);
	unichar = compose (ctrl, readchar ());
	clear_status ();
	Sunicode (unichar);
	return;
    }
  } else if (ctrl == '#') {
	if (get_hex_number ("Enter hex digits for character...", & unichar) 
		!= ERRORS) {
		Suni1 (unichar);
	}
	return;
  } else if (
	((ctrl > ' ') && (ctrl < 'Z')) || (ctrl > '^')
	) {
	build_string (text_buffer, "Enter character to compose with %c...", ctrl);
	status_msg (text_buffer);
	unichar = compose (ctrl, readchar ());
	clear_status ();
	Suni1 (unichar);
	return;
  }
  clear_status ();
  if (ctrl == '\177') {S ('\177'); return;}
  ctrl = ctrl & '\237';
  if (ctrl == '\0') error ("Can't handle NULL char - not inserted", NIL_PTR);
  else S (ctrl);
}

/*
 * LIB insert a line at the current position and moves back to the end of
 * the previous line.
 */
void
LIB ()
{
  hop_flag = 0;

  if (viewonly == True)
	{viewonlyerr (); return;}

  S ('\n');			/* Insert the line */
  MUP ();			/* Move one line up */
  move_to (LINE_END, y);	/* Move to end of this line */
}


/*  ==================================================================	*
 *				Yank Commands				*
 *  ==================================================================	*/

LINE * mark_line = NIL_LINE;		/* For marking position. */
char * mark_text = NIL_PTR;
#define maxmarkers 10
LINE * mark_n_line [maxmarkers] = {
	NIL_LINE, NIL_LINE, NIL_LINE, NIL_LINE, NIL_LINE, 
	NIL_LINE, NIL_LINE, NIL_LINE, NIL_LINE, NIL_LINE};
char * mark_n_text [maxmarkers] = {
	NIL_PTR, NIL_PTR, NIL_PTR, NIL_PTR, NIL_PTR, 
	NIL_PTR, NIL_PTR, NIL_PTR, NIL_PTR, NIL_PTR};
int lines_saved;			/* Nr of lines in buffer */

/*
 * PT () inserts the buffer at the current location.
 */
void
PT ()
{
  register int fd;		/* File descriptor for buffer */

  if (viewonly == True)
	{viewonlyerr (); return;}

  if (hop_flag > 0) {
	if ((fd = open (yankie_file, O_RDONLY | O_BINARY, 0)) < 0) {
		error ("No inter window buffer present", NIL_PTR);
		return;
	}
  }
  else if ((fd = scratch_file (READ, False)) == ERRORS) {
	error ("Buffer is empty", NIL_PTR);
	return;
  }
  /* Insert the buffer */
/*	file_insert (fd, False); => positioning error when TAB in last line */
/*		also: UTF-8 positioning correct? (char_count vs. length_of) */
	file_insert (fd, True);
}

/*
 * INSFILE () prompts for a filename and inserts the file at the current
 * location in the file.
 */
void
INSFILE ()
{
  register int fd;		/* File descriptor of file */
  char name [maxLINE_LEN];	/* Buffer for file name */

  if (restricted == True)
	{restrictederr (); return;}

  if (viewonly == True)
	{viewonlyerr (); return;}

/* Get the file name */
  if (get_file ("Get and insert file:", name) != FINE)
	return;
  clear_status ();

  if ((fd = open (name, O_RDONLY | O_BINARY, 0)) < 0)
	error ("Cannot open file: " /*, name */, serror ());
  else {	/* Insert the file */
	file_insert (fd, True);	/* leave cursor at begin of insertion */
  }
}

/*
 * File_insert () inserts the contents of an opened file (as given by
 * filedescriptor fd) at the current location.
 * After the insertion, if old_pos is True, the cursor remains at the
 * start of the inserted text, if old_pos is False, it is placed to
 * its end. If old_pos is False, this works erroneously if the last line
 * inserted contains a TAB character!!
 */
void
file_insert (fd, old_pos)
  int fd;
  FLAG old_pos;
{
  char line_buffer [MAX_CHARS];		/* Buffer for next line */
  register LINE * line = cur_line;
  register int line_count = total_lines;	/* Nr of lines inserted */
  LINE * page = cur_line;
  int ret = ERRORS;

  reset_get_line ();

/* Get the first piece of text (might be ended with a '\n') from fd */
  if (get_line (fd, line_buffer) == ERRORS)
	return;				/* Empty file */

/* Insert this text at the current location */
  if (insert (line, cur_text, line_buffer) == ERRORS)
	return;

/* Repeat getting lines (and inserting lines) until EOF is reached */
  while (line != NIL_LINE
	 && (ret = get_line (fd, line_buffer)) != ERRORS && ret != NO_LINE)
	line = line_insert (line, line_buffer, ret);

  if (line == NIL_LINE) sleep (2) /* show memory allocation error msg */;
  else if (ret == NO_LINE) {	/* Last line read not ended by a '\n' */
	line = line->next;
	if (insert (line, line->text, line_buffer) == ERRORS)
		sleep (2) /* give time to read error msg */;
  }

  (void) close (fd);

/* If illegal lines were input, report */
  show_get_l_errors ();

/* Calculate nr of lines added */
  line_count = total_lines - line_count;

/* Fix the screen */
  if (line_count == 0) {		/* Only one line changed */
	set_cursor (0, y);
	line_print (line);
	move_to ((old_pos == True) ? x : x + length_of (line_buffer), y);
  }
  else {				/* Several lines changed */
	reset (top_line, y);	/* Reset pointers */
	while (page != line && page != bot_line->next)
		page = page->next;
	if (page != bot_line->next || old_pos == True)
		display (y, cur_line, SCREENMAX - y, y);
		/* screen display style parameter (last) may be inaccurate */
	if (old_pos == True)
		move_to (x, y);
	else {
		if (ret == NO_LINE)
			move_to (col_count (line_buffer), find_y (line));
		else
			move_to (0, find_y (line->next));
	}
  }

/* If nr of added line >= REPORT, print the count */
  if (line_count >= REPORT)
	status_line (num_out ((long) line_count), " lines added");
}

/*
 * WB () writes the buffer (yank_file) into another file, which
 * is prompted for.
 */
void
WB ()
{
  register int new_fd;		/* Filedescriptor to copy file */
  int yank_fd;			/* Filedescriptor to buffer */
  register unsigned int cnt;	/* Count check for read/write */
  int ret = FINE;		/* Error check for write */
  char file_name [maxLINE_LEN];	/* Output file name */
  char * msg_doing; char * msg_done;

  if (restricted == True)
	{restrictederr (); return;}

/* Checkout the buffer */
  if ((yank_fd = scratch_file (READ, False)) == ERRORS) {
	error ("Buffer is empty", NIL_PTR);
	return;
  }

/* Get file name */
  if (get_file ((hop_flag > 0) ? "Append buffer to file:"
			       : "Write buffer to file:", file_name) != FINE)
	return;

/* Create the new file or open previous file for appending */
  if (hop_flag > 0) {
    if ((new_fd = open (file_name, O_WRONLY | O_CREAT | O_APPEND | O_BINARY, fprot)) < 0) {
	error ("Cannot append to file: ", serror ());
	return;
    }
    msg_doing = "Appending "; msg_done = "Appended";
  }
  else {
    if (checkoverwrite (file_name) != True)
	return;
    else if ((new_fd = open (file_name, O_WRONLY | O_CREAT | O_BINARY, fprot)) < 0) {
	error ("Cannot create file: ", serror ());
	return;
    }
    msg_doing = "Writing "; msg_done = "Wrote";
  }

  status_line (msg_doing, file_name);

/* Copy buffer into file */
  while ((cnt = read (yank_fd, text_buffer, sizeof (text_buffer))) > 0)
	if (write (new_fd, text_buffer, cnt) != cnt) {
		bad_write (new_fd);
		ret = ERRORS;
		break;
	}

/* Clean up open files and status_line */
  (void) close (new_fd);
  (void) close (yank_fd);

  if (ret != ERRORS)			/* Bad write */
	file_status (msg_done, chars_saved, file_name, lines_saved, 
			False, True, False, False, 0L);
}

/*
 * MARK sets mark_line / mark_text to the current line / current text pointer.
 */
void
MARK ()
{
  if (hop_flag > 0) GOMA ();
  else {
	Pushmark ();
	mark_line = cur_line;
	mark_text = cur_text;
	status_msg ("Mark set");
  }
}

/*
 * GOMA moves to the marked position
 */
void
GOMA ()
{
  if (checkmark (mark_line, mark_text) == NOT_VALID)
	error ("Mark not set", NIL_PTR);
  else
	move_address (mark_text, find_y (mark_line));
}

/*
 * MARKn sets mark n to the current line / current text pointer.
 * Markn sets it silently.
 * Pushmark saves the currrent marker to marker 0.
 */
void
MARKn (n)
  int n;
{
  if (hop_flag > 0) GOMAn (n);
  else {
	if (n < 0 || n >= maxmarkers) {
		error ("Marker # out of range", NIL_PTR);
		return;
	}
	mark_n_line [n] = cur_line;
	mark_n_text [n] = cur_text;
	status_msg ("Mark set");
  }
}

void
Markn (n)
  int n;
{
	if (n == -1) {	/* initial mark */
		mark_line = cur_line;
		mark_text = cur_text;
	} else if (n < 0 || n >= maxmarkers) {
		error ("Marker # out of range", NIL_PTR);
		return;
	} else {
		mark_n_line [n] = cur_line;
		mark_n_text [n] = cur_text;
	}
}

void
Pushmark ()
{
	mark_n_line [0] = mark_line;
	mark_n_text [0] = mark_text;
}

/*
 * GOMAn moves to the marked position n
 */
void
GOMAn (n)
  int n;
{
  if (n < 0 || n >= maxmarkers) {
	error ("Marker # out of range", NIL_PTR);
	return;
  }

  if (checkmark (mark_n_line [n], mark_n_text [n]) == NOT_VALID)
	error ("Mark not set", NIL_PTR);
  else
	move_address (mark_n_text [n], find_y (mark_n_line [n]));
}

/*
 * Yankie () provides a reference to the last saved buffer to be read
 * by other mined invocations.
 */
void
yankie ()
{
  delete_file (yankie_file);
#ifdef unix
  link (yank_file, yankie_file);
#else
  build_string (text_buffer, copycommand, yank_file, yankie_file);
  system (text_buffer);
#endif
}

/*
 * Set_up is an interface to the actual yank. It calls checkmark () to check
 * if the marked position is still valid. If it is, yank is called with the
 * arguments in the right order.
 */
void
set_up (remove, append)
  FLAG remove;	/* == DELETE if text should be deleted */
  FLAG append;	/* == True if text should only be appended to yank buffer */
{
  switch (checkmark (mark_line, mark_text)) {
	case NOT_VALID :
		error ("Mark not set", NIL_PTR);
		return;
	case SMALLER :
		yank (mark_line, mark_text, cur_line, cur_text, remove, append);
		yankie ();
		break;
	case BIGGER :
		yank (cur_line, cur_text, mark_line, mark_text, remove, append);
		yankie ();
		break;
	case SAME :		/* Ignore stupid behaviour */
	/*	yank_status = EMPTY;	*/
		chars_saved = 0L;
		status_msg ("Nothing to save");
		break;
	default :
		error ("Internal mark error", NIL_PTR);
		return;
  }
}

/*
 * YA () puts the text between the marked position and the current
 * in the buffer.
 */
void
YA ()
{
  if (append_flag == True)
	set_up (NO_DELETE, True);
  else if (hop_flag > 0) {
	if (MENU) {
		append_flag = True;
		displayflags ();
		set_cursor_xy ();
		flush ();
	}
	set_up (NO_DELETE, True);
	if (MENU) {
		append_flag = False;
		displayflags ();
		set_cursor_xy ();
		flush ();
	}
  } else
	set_up (NO_DELETE, False);
}

/*
 * DT () is essentially the same as YA (), but in DT () the text is deleted.
 */
void
DT ()
{
  if (viewonly == True)
	{viewonlyerr (); return;}

  if (append_flag == True)
	set_up (DELETE, True);
  else if (hop_flag > 0)
	set_up (DELETE, True);
  else
	set_up (DELETE, False);
}

/*
 * Check_mark () checks if mark_line and mark_text are still valid pointers.
 * If they are it returns
 * SMALLER if the marked position is before the current,
 * BIGGER if it isn't or SAME if somebody didn't get the point.
 * NOT_VALID is returned when mark_line and/or mark_text are no longer valid.
 * Legal () checks if mark_text is valid on the mark_line.
 */
FLAG
checkmark (mark_line, mark_text)
  register LINE * mark_line;
  register char * mark_text;
{
  register LINE * line;
  FLAG cur_seen = False;

/* Special case: check is mark_line and cur_line are the same. */
  if (mark_line == cur_line) {
	if (mark_text == cur_text)	/* Even same place */
		return SAME;
	if (legal (mark_line, mark_text) == ERRORS) /* mark_text out of range */
		return NOT_VALID;
	if (mark_text < cur_text)
		return SMALLER;
	else	return BIGGER;
  }

/* Start looking for mark_line in the line structure */
  for (line = header->next; line != tail; line = line->next) {
	if (line == cur_line)
		cur_seen = True;
	else if (line == mark_line)
		break;
  }

/* If we found mark_line (line != tail) check for legality of mark_text */
  if (line == tail || legal (mark_line, mark_text) == ERRORS)
	return NOT_VALID;

/* cur_seen is True if cur_line is before mark_line */
  if (cur_seen == True)
	return BIGGER;
  else	return SMALLER;
}

/*
 * Legal () checks if mark_text is still a valid pointer.
 */
int
legal (mark_line, mark_text)
  register LINE * mark_line;
  register char * mark_text;
{
  register char * textp = mark_line->text;

/* Locate mark_text on mark_line */
  while (textp != mark_text && * textp != '\0')
	textp ++;
  return (* textp == '\0') ? ERRORS : FINE;
}

/*
 * Yank puts all the text between start_position and end_position into
 * the buffer.
 * The caller must check that the arguments to yank () are valid (e.g. in
 * the right order).
 */
void
yank (start_line, start_textp, end_line, end_textp, remove, append)
  LINE * start_line, * end_line;
  char * start_textp, * end_textp;
  FLAG remove;	/* == DELETE if text should be deleted */
  FLAG append;	/* == True if text should only be appended to yank buffer */
{
  register LINE * line = start_line;
  register char * textp = start_textp;
  int fd;

/* Create file to hold buffer */
  if ((fd = scratch_file (WRITE, append)) == ERRORS)
	return;

  chars_saved = 0L;
  lines_saved = 0;
  if (append == True)
	status_msg ("Appending text ...");
  else	status_msg ("Saving text ...");

/* Keep writing chars until the end_location is reached. */
  while (textp != end_textp) {
	if (writechar (fd, * textp) == ERRORS) {
		(void) close (fd);
		return;
	}
	if (* textp ++ == '\n') {	/* Move to the next line */
		line = line->next;
		textp = line->text;
		lines_saved ++;
	}
	chars_saved ++;
  }

/* Flush the I/O buffer and close file */
  if (flush_buffer (fd) == ERRORS) {
	(void) close (fd);
	return;
  }
  (void) close (fd);
  yank_status = VALID;

  /*
   * Check if the text should be deleted as well. In case it should,
   * the following hack is used to save a lot of code.
   * First move back to the start_position (this might be the current
   * location) and then delete the text.
   * This might look a bit confusing to the user the first time.
   * Delete () will fix the screen.
   */
  if (remove == DELETE) {
	move_to (find_x (start_line, start_textp), find_y (start_line));
	if (delete_text (start_line, start_textp, end_line, end_textp)
		== ERRORS) {
		sleep (2) /* give time to read allocation error msg */;
	}
	mark_line = cur_line;
	mark_text = cur_text;
  }

  if (append == True)
	status_line (num_out (chars_saved), " bytes appended to buffer");
  else	status_line (num_out (chars_saved), " bytes saved in buffer");
}

/*
 * Scratch_file () tries to create a unique file in a temporary directory.
 * It tries several different filenames until one can be created
 * or MAXTRIALS attempts have been made.
 * After MAXTRIALS times, an error message is given and ERRORS is returned.
 */

#define MAXTRIALS 99

int
scratch_file (mode, append)
  FLAG mode;	/* Can be READ or WRITE permission */
  FLAG append;	/* == True if text should only be appended to yank buffer */
{
  static int trials = 0;	/* Keep track of trials */
  int fd = 0;			/* Filedescriptor to buffer */

/* If yank_status == NOT_VALID, scratch_file is called for the first time */
  if (yank_status == NOT_VALID && mode == WRITE) { /* Create new file */
	/* Generate file name. */
#ifdef msdos
	build_string (yank_file, "%s%d", yankie_file, trials);
#else
	build_string (yank_file, "%s%d-%d", yankie_file, getpid (), trials);
#endif
	/* Check file existence */
	if (access (yank_file, 0 /* F_OK */) == 0
	    || (fd = creat (yank_file, bufprot)) < 0) {
		if (++ trials >= MAXTRIALS) {
		    build_string (text_buffer, "Unable to create scratchfile %s: ", yank_file);
		    if (fd == 0)
			error (text_buffer, "File exists");
		    else
			error (text_buffer, serror ());
		    return ERRORS;
		}
		else
		    return scratch_file (mode, append);	/* try again */
	}
  }
  else if (yank_status == NOT_VALID && mode == READ) {
	return ERRORS;
  }
  else /* yank_status == VALID */
	if (  (mode == READ && (fd = open (yank_file, O_RDONLY | O_BINARY, 0)) < 0)
	   || (mode == WRITE &&
		(fd = open (yank_file, O_WRONLY | O_CREAT |
			((append == True) ? O_APPEND : O_TRUNC), bufprot)) < 0)) {
	yank_status = NOT_VALID;
	return ERRORS;
  }

  clear_buffer ();
  return fd;
}


/*  ==================================================================	*
 *				End					*
 *  ==================================================================	*/
