/*  ==================================================================	*
 *				Editor mined				*
 *				auxiliary part				*
 *  ==================================================================	*/

#include "mined.h"

int panic_level = 0;		/* To adjust error handling to situation */
FLAG pagewrapped = False;	/* Did output on the bottom line wrap and scroll? */
FLAG last_char_latin1 = False;	/* was accented character composed? */
FLAG char_on_status_line = False; /* is output active on status line ? */

/*  ==================================================================	*
 *			Auxiliary routines				*
 *  ==================================================================	*/

/*
 * put_char () puts a character byte to the screen (via buffer)
 * if UTF-8 text is being output through put_char () but the screen 
 * is not in UTF-8 mode, put_char () transforms the byte sequences
 *
 * putchar (c) is __putchar (c)
 * __putchar (c) does (void) writechar (output_fd, (c)) (except with curses)
 */

#define illegal_UTF indicate_UTF
void
indicate_UTF (c)
  char c;
{
  if (char_on_status_line == True)
	reverse_off ();
  unidisp_on ();
  __putchar (c);
  unidisp_off ();
  if (char_on_status_line == True)
	reverse_on ();
}

static int utfcount = 0;
static unsigned long unichar;

void
put_end ()
{
  if (utf8_text && ! utf8_screen && utfcount > 0) {
	illegal_UTF ('');
	utfcount = 0;
  }
}

void
put_char (c)
  character c;
{
  if (utf8_text && ! utf8_screen) {
	if (c < 0x80) {
		if (utfcount > 0) {
			illegal_UTF ('');
			utfcount = 0;
		}
		__putchar (c);
	} else if ((c & 0xC0) == 0x80) {
		if (utfcount == 0) {
			illegal_UTF ('8');
			return;
		}
		unichar = (unichar << 6) | (c & 0x3F);
		utfcount --;
		if (utfcount == 0) {
			if ((unichar & 0xFF) == unichar && unichar >= ' ')
				__putchar (unichar & 0xFF);
			else {
				if (unichar < ' ')
					indicate_UTF (unichar + '@');
				else
					indicate_UTF (UNI_MARK);
			}
		}
		else
			return;
	} else { /* first UTF-8 byte */
		if (utfcount > 0) {
			illegal_UTF ('');
			utfcount = 0;
		}
		if ((c & 0xE0) == 0xC0) {
			utfcount = 2;
			unichar = c & 0x1F;
		} else if ((c & 0xF0) == 0xE0) {
			utfcount = 3;
			unichar = c & 0x0F;
		} else if ((c & 0xF8) == 0xF0) {
			utfcount = 4;
			unichar = c & 0x07;
		} else if ((c & 0xFC) == 0xF8) {
			utfcount = 5;
			unichar = c & 0x03;
		} else if ((c & 0xFE) == 0xFC) {
			utfcount = 6;
			unichar = c & 0x01;
		} else {	/* ignore illegal UTF-8 code */
			illegal_UTF ('4' + (c & 1));
			return;
		}
		utfcount --;
		return;
	}
  } else
	__putchar (c);
}

void
putcharacter (c)
  character c;
{
  if (utf8_screen && ! utf8_text)
	pututf8 (c);
  else
	put_char (c);
}

/*
 * put UTF-8 character
	7 bit	0xxxxxxx
	8..11	110xxxxx 10xxxxxx
	12..16	1110xxxx 10xxxxxx 10xxxxxx
 */
void
pututf8 (c)
  unsigned int c;
{
  if (c < 0x80) {
	put_char (c);
  } else if (c < 0x800) {
	put_char (0xC0 | (c >> 6));
	put_char (0x80 | (c & 0x3F));
  } else {
	put_char (0xE0 | (c >> 12));
	put_char (0x80 | ((c >> 6) & 0x3F));
	put_char (0x80 | (c & 0x3F));
  }
}

/*
 * put 1 UTF-8 character (from string)
 */
void
put_utfchar (s)
  char * s;
{
	int follow = UTF_len (* s) - 1;

	__putchar (* s);
	s ++;
	while (follow > 0 && (* s & 0xC0) == 0x80) {
		__putchar (* s ++);
		follow --;
	}
}

/*
 * put mark character, possibly switching alternate character set
 */
FLAG dim_mode = False;
FLAG mark_mode = False;
FLAG use_extracset = False;

void
putmarkmode (c, utfmark)
  char c;
  char * utfmark;
{
  if (dim_mode == False) {
	dimon ();
	dim_mode = True;
  }

  if (utf8_screen == True) {
	if (utf8_text == True && utfmark != NIL_PTR && * utfmark != '\0')
		put_utfchar (utfmark);
	else	pututf8 (c & 0xFF);
  }
  else if ((c & '\200') != '\0') {
	if (mark_mode == True) {
		end_extracset ();
		mark_mode = False;
	}
	putchar (c);
  }
  else {
	if (mark_mode == False && use_extracset == True) {
		start_extracset ();
		mark_mode = True;
	}
	putchar (c);
  }
}

void
endmarkmode ()
{
  if (dim_mode == True) {
	dimoff ();
	dim_mode = False;
  }
  if (mark_mode == True) {
	end_extracset ();
	mark_mode = False;
  }
}

void
putmark (c, utfmark)
  char c;
  char * utfmark;
{
  putmarkmode (c, utfmark);
  endmarkmode ();
}

/*
 * Delete file.
 */
void
delete_file (file)
  char * file;
{
#ifdef unix
  unlink (file);
#endif
#ifdef msdos
  unlink (file);
#endif
#ifdef vms
  delete (file);
#endif
}

/*
 * Delete yank file if there is one.
 */
void
delete_yank_file ()
{
  if (yank_status == VALID) delete_file (yank_file);
}

/*
 * Panic () is called with a mined error msg and an optional system error msg.
 * It is called when something unrecoverable has happened.
 * It writes the message to the terminal, resets the tty and exits.
 * Ask the user if he wants to save his file.
 */
#define panic_msg(msg)	if (isscreenmode == True) {status_msg (msg); sleep (2);} else (void) printf ("%s\n", msg);
void
panicking (message, err, signum)
  register char * message;
  register char * err;
  register int signum;
{
  int panic_written;
  void QUED ();

  panic_level ++;

  if (panic_level < 2) {
	if (loading == False && modified == True) {
		panic_written = panicwrite ();
		if (panic_written == ERRORS) {
			build_string (text_buffer, "Error writing panic file %s", panic_file);
			sleep (2);
		}
		else
			build_string (text_buffer, "Panic file %s written", panic_file);
		ring_bell ();
		panic_msg (text_buffer);
	}
	if (signum != 0)
		build_string (text_buffer, message, signum);
	else if (err == NIL_PTR)
		build_string (text_buffer, "%s", message);
	else
		build_string (text_buffer, "%s (Error: %s)", message, err);
	panic_msg (text_buffer);

	/* "normal" panic handling: */
	if (loading == False) {
		QUED ();	/* Try to save the file and quit */
		/* QUED returned: something wrong */
		sleep (2);
		panic_msg ("Aborted writing file in panic mode - trying to continue");
		panic_level --;
		return;
	}
  }

  if (panic_level < 3) {
	if (isscreenmode == True) {
		set_cursor (0, YMAX);
		putchar ('\n');
		raw_mode (OFF);
	}
	delete_yank_file ();
  }
  exit (1) /* abort () sends IOT which would again be caught */;
}

void
panic (message, err)
  register char * message;
  register char * err;
{
  panicking (message, err, 0);
}

void
panicio (message, err)
  register char * message;
  register char * err;
{
/* Should panic_level already be increased here ? */
  panic (message, err);
}

void
catch_interrupt (signum)
  int signum;
{
  panicking ("External signal %d caught - terminating", NIL_PTR, signum);
  catch_signals (catch_interrupt);
}

/*-------------------------------------------------------------------------*/

/*
 * Memory allocation
 */
#ifdef msdos
#include <alloc.h>
#define allocate farmalloc
#define freemem farfree
#else
extern void * malloc ();
extern void free ();
#define allocate malloc
#define freemem free
#endif

char *
alloc (bytes)
  int bytes;
{
  return allocate ((unsigned) bytes);
/*
  char * p;

  if ((p = allocate ((unsigned) bytes)) == NIL_PTR)
	panic ("Out of memory", NIL_PTR);
  return p;
*/
}

void
free_space (p)
  char * p;
{
  freemem (p);
}

/*
 * free header list
 */

#define pointersize	sizeof (void *)
#define blocksizeof(typ) ((sizeof (typ) + pointersize - 1) / pointersize * pointersize)

LINE * free_header_list = NIL_LINE;

void
alloc_headerblock (n)
  int n;
{
  LINE * new_header;
  LINE * new_list;
  int i = 0;

  new_list = (LINE *) alloc (n * blocksizeof (LINE));
  if (new_list == NIL_LINE) free_header_list = NIL_LINE;
  else while (i < n) {
	new_header = (LINE *) ((long) new_list + i * blocksizeof (LINE));
	new_header->next = free_header_list;
	free_header_list = new_header;
	i ++;
  }
}

LINE *
alloc_header ()
{
/*  return (LINE *) alloc (sizeof (LINE)); */
  LINE * new_header;

  if (free_header_list == NIL_LINE) {
	alloc_headerblock (64);
	if (free_header_list == NIL_LINE) alloc_headerblock (16);
	if (free_header_list == NIL_LINE) alloc_headerblock (4);
	if (free_header_list == NIL_LINE) alloc_headerblock (1);
	if (free_header_list == NIL_LINE) return NIL_LINE;
  }
  new_header = free_header_list;
  free_header_list = free_header_list->next;
  return new_header;
}

void
free_header (hp)
  LINE * hp;
{
/*  freemem (hp); */
  hp->next = free_header_list;
  free_header_list = hp;
}

/*
 * Basename () finds the absolute name of the file out of a given path_name.
 */
#ifdef UNUSED
char *
basename (path)
  char * path;
{
  register char * ptr = path;
  register char * last = NIL_PTR;

  while (* ptr != '\0') {
	if (* ptr == '/')
		last = ptr;
	ptr ++;
  }
  if (last == NIL_PTR)
	return path;
  if (* (last + 1) == '\0') {	/* E.g. /usr/tmp/pipo/ */
	* last = '\0';
	return basename (path);	/* Try again */
  }
  return last + 1;
}
#endif

/*
 * Unnull () changes a NULL string pointer into an empty string pointer 
 * to allow easy feading of string results into build_string / sprintf
 */
char *
unnull (s)
  char * s;
{
  if (s == NIL_PTR) return "";
  else return s;
}

/*
 * Output an (unsigned) long in a 10 digit field without leading zeros.
 * It returns a pointer to the first digit in the buffer.
 */
char *
num_out (number)
  long number;
{
  static char num_buf [11];		/* Buffer to build number */
  register long digit;			/* Next digit of number */
  register long pow = 1000000000L;	/* Highest ten power of long */
  FLAG digit_seen = False;
  int i;

  for (i = 0; i < 10; i ++) {
	digit = number / pow;		/* Get next digit */
	if (digit == 0L && digit_seen == False && i != 9)
		num_buf [i] = ' ';
	else {
		num_buf [i] = '0' + (char) digit;
		number -= digit * pow;	/* Erase digit */
		digit_seen = True;
	}
	pow /= 10L;			/* Get next digit */
  }
  num_buf [11] = '\0';
  for (i = 0; num_buf [i] == ' '; i ++)	/* Skip leading spaces */
	;
  return & num_buf [i];
}

/*
 * Build_string () prints the arguments as described in fmt, into the buffer.
 * %s indicates a string argument, %d indicates an integer argument.
 */
#ifndef build_string /* otherwise build_string is sprintf */
/* VARARGS */
void
build_string (buf, fmt, args)
  register char * buf, * fmt;
  int args;
{
  int * argptr = & args;
  char * scanp;
  FLAG islong;

  while (* fmt) {
     if (* fmt == '%') {
	fmt ++;
	if (* fmt == 'l') {islong = True; fmt ++;}
		     else islong = False;
	switch (* fmt ++) {
	case 's' :
		scanp = (char *) * argptr;
		break;
	case 'd' :
		if (islong == True) {
			scanp = num_out ((long) * ((long *) argptr));
			if (sizeof (long) > sizeof (int)) argptr ++;
		break;
		}
		else {
			scanp = num_out ((long) * argptr);
			break;
		}
	case 'D' :
		scanp = num_out ((long) * ((long *) argptr));
		if (sizeof (long) > sizeof (int)) argptr ++;
		break;
	default :
		scanp = "";
	}
	while (* buf ++ = * scanp ++)
		;
	buf --;
	argptr ++;
     }
     else
	* buf ++ = * fmt ++;
  }
  * buf = '\0';
}
#endif /* ndef build_string */

/*
 * make_number () converts a string into a natural number
 * returns the character after the last digit
 */
int
make_number (num, str)
  int * num;
  char * str;
{
  register char * chpoi = str;

  * num = 0;
  while (* chpoi >= '0' && * chpoi <= '9' && quit == False) {
	* num *= 10;
	* num += * chpoi - '0';
	chpoi ++;
  }
  return * chpoi;
}

/*
 * length_of () returns the number of bytes in the string `string'
 * excluding the '\0'.
 */
int
length_of (string)
  register char * string;
{
  register int count = 0;

  if (string != NIL_PTR) {
	while (* string ++ != '\0')
		count ++;
  }
  return count;
}

/*
 * char_count () returns the number of characters in the string `string'
 * excluding the '\0'.
 */
/*
 * col_count () returns the number of screen columns in the string `string'
 */
int
char_count (string)
  char * string;
{
  int count = 0;

  if (string != NIL_PTR) {
	while (* string != '\0') {
		advance_char (& string);
		count ++;
	}
  }
  return count;
}
int
col_count (string)
  char * string;
{
  int count = 0;

  if (string != NIL_PTR) {
	while (* string != '\0') {
		advance_char_col (& string, & count);
	}
  }
  return count;
}
int
old_char_count (string)
  char * string;
{
  int count = 0;

  if (string != NIL_PTR) {
	if (utf8_text == True)
		while (* string != '\0') {
			if ((* string) & 0xC0 != 0x80)
				count ++;
			string ++;
		}
	else if (Chinese == True)
		while (* string != '\0') {
			count ++;
			if (((* string) & 0x80) == 0x80)
				string ++;
			if ((* string) != '\0')
				string ++;
		}
	else
		while (* string ++ != '\0')
			count ++;
  }
  return count;
}

/*
 * text_bytes_of () returns the number of bytes in the string `string'
 * up to and excluding the first '\n'.
 */
int
text_bytes_of (string)
  register char * string;
{
  register int count = 0;

  if (string != NIL_PTR) {
	while (* string != '\0' && * string != '\n') {
		string ++;
		count ++;
	}
  }
  return count;
}

/*
 * Copy_string () copies the string `from' into the string `to'. `To' must be
 * long enough to hold `from'.
 */
void
copy_string (to, from)
  register char * to;
  register char * from;
{
  while ((* to ++ = * from ++) != '\0')
	;
}

/*-------------------------------------------------------------------------*/

/*
 * serrorof delivers the error message of the given errno value.
 * serror delivers the error message of the current errno value.
 * geterrno just returns the current errno value.
 */
#ifdef vms
/* #define includeerrno */
#  ifdef includeerrno
#  include <errno.h>
#  else
    extern volatile int noshare errno;
    extern volatile int noshare vaxc$errno; /* VMS error code when errno = EVMSERR */
#  define EVMSERR 65535
    extern volatile int noshare sys_nerr;
    extern volatile char noshare * sys_errlist [];
#  endif
#else
  extern int errno;
  extern int sys_nerr;
  extern char * sys_errlist [];
#endif

char *
serrorof (errnum)
  int errnum;
{
  if ((errnum < 0) || (errnum >= sys_nerr))
	{ static char s [20];
#ifdef vms
	  if (errnum == EVMSERR)
	     build_string (s, "VMS error %d", vaxc$errno);
	  else
#endif
	     build_string (s, "Unknown error %d", errnum);
	  return s;
	}
  else	return sys_errlist [errnum];
}

char *
serror ()
{ return serrorof (errno); }

int
geterrno ()
{ return errno; }


/*  ==================================================================	*
 *				Output					*
 *  ==================================================================	*/

int
iscontrol (c)
  character c;
{
#ifdef msdos
	return ((c) == '\177') 
		|| ((character) c < (character) ' ');
#else
  if (utf8_text == True)
	return ((c) == '\177') 
		|| ((character) c < (character) ' ');
  else
	return ((c) == '\177') 
		|| ((character) ((c) & '\177') < (character) ' ');
#endif
}

#define controlchar(c)	(((c) == '\177') ? '?' : (c) + '@')

/*
 * Bad_write () is called when a write failed. Notify the user.
 */
void
bad_write (fd)
  int fd;
{
  if (fd == output_fd) {	/* Cannot write to terminal? */
	panicio ("Write error on terminal", serror ());
  }

  clear_buffer (); /* out_count = 0; */
  ring_bell ();
  error ("Write aborted (File incomplete): ", serror ());
}

/*
 * Flush the I/O buffer on filedescriptor fd.
flush () is (void) flush_buffer (output_fd)
 */
int
flush_buffer (fd)
  int fd;
{
  if (out_count <= 0)		/* There is nothing to flush */
	return FINE;
#ifdef conio
  if (fd == output_fd) {
	cputs (screen);
  }
#else
#ifdef BorlandC
  if (fd == output_fd) {
	screen [out_count] = '\0';
/* don't ask me why that crazy compiler doesn't work with write () below */
	printf ("%s", screen);
  }
#endif
#endif
  else {
	char * writepoi = screen;
	int written = 0;
	int none_count = 0;
/*	int less_count = 0;	*/

	while (out_count > 0) {
		written = write (fd, writepoi, out_count);
		if (written == -1) {
			bad_write (fd);
			return ERRORS;
		}
/*
		if (written != out_count) less_count ++;
		if (less_count > 0) {
fprintf (stderr, "   written %d/%d     \n", written, out_count); sleep (2);
		}
*/
		if (written == 0) {
			none_count ++;
			if (none_count > 20) {
				bad_write (fd);
				return ERRORS;
			}
		}
		out_count -= written;
		writepoi += written;
	}
  }
  clear_buffer (); /* Empty buffer: out_count = 0; */
  return FINE;
}

/*
 * Write_char does a buffered output.
 */
int
writechar (fd, c)
  int fd;
  char c;
{
  char clow;

  if (c == '\n') if (fd == output_fd) {
	if (writechar (fd, '\015') == ERRORS) return ERRORS;
  }
  if (translate_output == True && fd == output_fd) {
	if ((c & '\200') != '\0') {
		altcset_on ();
		clow = (c & '\177');
		if ((int) clow < translen)
			writechar (output_fd, transout [clow]);
		else	writechar (output_fd, clow);
		altcset_off ();
		return FINE;
	}
  }
  screen [out_count ++] = c;
  if (out_count == screen_BUFL)	/* Flush on screen_BUFL chars */
	return flush_buffer (fd);
#ifdef DEBUG
  if (fd == output_fd) flush ();
#endif
  return FINE;
}

/*
 * Writestring writes the given string on the given filedescriptor.
 * (buffered via writechar via screen !)
putstring (str) is (void) writestring (output_fd, (str))
 */
int
writestring (fd, text)
  register int fd;
  register char * text;
{
  while (* text)
	 if (writechar (fd, * text ++) == ERRORS)
		return ERRORS;
  return FINE;
}

/*
 * Print string on terminal, printing controls with ^.
 */
int lpos = 0;

void
print_char (c)
  register character c;
{
  char_on_status_line = True;
  if (utf8_text && (c & 0xC0) == 0x80) {
	put_char (c);
  } else {
	lpos ++;
	if (iscontrol (c)) {
		put_char ('^');
		lpos ++;
		putcharacter (controlchar (c));
	}
	else putcharacter (c);
  }
  char_on_status_line = False;
}

int
printlim_char (c, limit)
  register character c;
  register int limit;
{
  char_on_status_line = True;
  if (utf8_text && (c & 0xC0) == 0x80) {
	put_char (c);
  } else {
	if (lpos == limit) {
		putmark (SHIFT_MARK, NIL_PTR);
		char_on_status_line = False;
		return ERRORS;
	}
	lpos ++;
	if (iscontrol (c)) {
		put_char ('^');
		if (lpos == limit) {
			putmark (SHIFT_MARK, NIL_PTR);
			char_on_status_line = False;
			return ERRORS;
		}
		lpos ++;
		putcharacter (controlchar (c));
	}
	else putcharacter (c);
  }
  char_on_status_line = False;
  return FINE;
}

void
printlim_string (text, limit)
  register char * text;
  register int limit;
{
  lpos = 0;
  while (* text != '\0')
	if (printlim_char (* text ++, limit) == ERRORS)
		return;
}

void
print_string (text)
  register char * text;
{
  lpos = 0;
  while (* text != '\0')
	print_char (* text ++);
}

void
put_blanks (endpos)
  int endpos;
{
  int startpos = 0;
  while (startpos ++ <= endpos) putchar (' ');
}

void
clear_wholeline ()
{
  if (can_clear_eol == True) clear_eol ();
  else put_blanks (XMAX);
}

void
clear_lastline ()
{
  if (can_clear_eol == True) clear_eol ();
  else put_blanks (XMAX - 1);
}


/*  ==================================================================	*
 *			Buffer oriented output				*
 *  ==================================================================	*/

/*
 * Put_line prints the given line on the standard output.
 * If offset is not zero, printing will start at that x-coordinate.
 * If the FLAG clear_line is True, then the screen line will be cleared
 * when the end of the line has been reached.
line_print (line) is put_line (line, 0, True, False)
 * put_line is directly called only by S () and delete_text ()
 */
void
put_line (line, offset, clear_line, positioning)
  LINE * line;		/* Line to print */
  int offset;		/* Offset to start if positioning == False */
			/* position if positioning == True */
  FLAG clear_line;	/* Clear to eoln if True */
  FLAG positioning;	/* positioning inside line if True (for prop. fonts) */
{
  char * textp = line->text;
  char * parap;
  int count = get_shift (line->shift_count) * - SHIFT_SIZE;
  int count_ini = count;
  int tab_count;			/* Used in tab expansion */
  int tab_mid;
  int offset_start;
  int offset_stop;
  char * utfpoi;
  char c;
  int follow;

  if (positioning == True) {
	offset_start = 0;
	offset_stop = offset;
  } else {
	offset_start = offset;
	offset_stop = XBREAK;
  }

/* Skip all chars as indicated by the offset_start and the shift_count field */
  while (count < offset_start) {
	advance_char_col (& textp, & count);
  }

  if (count == 0 && count_ini < 0 && SHIFT_BEG != '\0') {
	putmark (SHIFT_BEG, NIL_PTR);
	if (! is_tab (* textp))
		advance_char_col (& textp, & count);
	else	count ++;
  }

  while (* textp != '\n' && count < offset_stop) {
	if (is_tab (* textp)) {		/* Expand tabs to spaces */
		tab_count = tab (count);
		tab_mid = count + (tab_count - count) / 2 + 1;
		if (TABchar0 != '\0') {
			if (TABchar0 != TABchar) {
				count ++;
				putmarkmode (TABchar0, UTF_TAB0);
			}
			while (count < offset_stop && count < tab_count - 1) {
				count ++;
				putmarkmode (TABchar, UTF_TAB);
			}
			if (count < offset_stop && count < tab_count) {
				count ++;
				if (TABchar2 != '\0')
					putmarkmode (TABchar2, UTF_TAB2);
				else	putmarkmode (TABchar, UTF_TAB);
			}
		}
		else while (count < offset_stop && count < tab_count) {
			count ++;
			if (count == tab_mid && TABcharmid != '\0')
				putmarkmode (TABcharmid, UTF_TABmid);
			else	putmarkmode (TABchar, UTF_TAB);
		}
		endmarkmode ();

		textp ++;
	}
	else {
		if (iscontrol (* textp)) {
			put_end ();
			ctrldisp_on ();
			putcharacter (controlchar (* textp));
			ctrldisp_off ();
			textp ++;
		}
		else {
			c = * textp ++;
			putcharacter (c);
			if (utf8_text == True) {
				follow = UTF_len (c) - 1;
				while (follow > 0 && (* textp & 0xC0) == 0x80) {
					put_char (* textp ++);
					follow --;
				}
			} else if (Chinese == True && multichar (c)) {
				put_char (* textp ++);
			}
		}
		count ++;
	}
  }
  put_end ();

  if (positioning == True) {
	/* self-made cursor for terminals (such as xterm)
	   which have display problems with proportional screen fonts
	   and their cursor */
	reverse_on ();
	if (* textp != '\n') {
		if (utf8_text == True) {
			put_utfchar (textp);
			put_end ();
		}
		else
			putcharacter (* textp);
	} else if (RET_MARK != '\0') {
		if (PARA_MARK != '\0' && paradisp == True) {
			parap = textp;
			parap --;
			if (* parap == ' ' || * parap == '\t')
				putmark (RET_MARK, UTF_RET);
			else	putmark (PARA_MARK, UTF_PARA);
		} else
			putmark (RET_MARK, UTF_RET);
	}
	else putchar (' ');
	reverse_off ();
	set_cursor (0, YMAX);
  }
  else /* (positioning == False) */ {
	/* If line is longer than XBREAK chars, print the shift_mark */
	if (count == XBREAK && * textp != '\n') {
		putmark (SHIFT_MARK, NIL_PTR);
		count ++;
	}

	/* Mark end of line if so desired */
	if (* textp == '\n' && RET_MARK != '\0') {
		if (PARA_MARK != '\0' && paradisp == True) {
			parap = textp;
			parap --;
			if (* parap == ' ')
				putmark (RET_MARK, UTF_RET);
			else	putmark (PARA_MARK, UTF_PARA);
		} else
			putmark (RET_MARK, UTF_RET);
		count ++;
		if (RET_BLANK) {
			while (count < XBREAK) {
				putmark (RET_BLANK, UTF_RETblank);
				count ++;
			}
			if (RET_BLANK2 && count <= XBREAK) {
				putmark (RET_BLANK2, UTF_RETblank2);
				count ++;
			}
		}
	}

	/* Clear the rest of the line if clear_line is True */
	if (clear_line == True) {
		if (can_clear_eol == True) {
			if (count <= XBREAK) clear_eol ();
		}
		else {
			while (count ++ <= XBREAK)	/* clear up to XMAX */
				putchar (' ');
		}
	}
  }
}

/*
 * set_cursor_xy sets the cursor by either directly calling set_cursor
 * or, in the case of proportional font support, reprinting the line
 * up to the x position
 */
void
set_cursor_xy ()
{
  if (proportional == True) {
	set_cursor (0, y);
	if (x != 0) put_line (cur_line, x, False, True);
	/* cur_line may still be undefined if x == 0 */
  }
	else set_cursor (x, y);
}

/*
 * Proceed returns the count'th line after `line'. When count is negative
 * it returns the count'th line before `line'. When the next (previous)
 * line is the tail (header) indicating EOF (tof) it stops.
 */
LINE *
proceed (line, count)
  register LINE * line;
  register int count;
{
  if (count < 0)
	while (count ++ < 0 && line != header)
		line = line->prev;
  else
	while (count -- > 0 && line != tail)
		line = line->next;
  return line;
}

/*
 * Reset assigns bot_line, top_line and cur_line according to `head_line'
 * which must be the first line of the screen, and a y-coordinate,
 * which will be the current y-coordinate (if it isn't larger than last_y)
 */
void
reset (head_line, screen_y)
  LINE * head_line;
  int screen_y;
{
  register LINE * line;

  top_line = line = head_line;

/* Search for bot_line (might be last line in file) */
  for (last_y = 0; last_y < total_lines - 1 && last_y < SCREENMAX
					&& line->next != tail; last_y ++)
	line = line->next;

  bot_line = line;
  y = (screen_y > last_y) ? last_y : screen_y;

/* Set cur_line according to the new y value */
  cur_line = proceed (top_line, y);
}

/*
 * Display line at screen line y_pos if it lies between y_min and y_max.
 * If it is no text line (end of file), clear screen line.
 */
void
display_line_at (y_pos, line, y_min, y_max, first)
  int y_pos, y_min, y_max;
  register LINE * line;
  FLAG first;
{
  line = proceed (line, y_pos - y_min);
  if (y_pos >= y_min && y_pos <= y_max) {
	set_cursor (0, y_pos);
	if (line == tail) clear_wholeline ();
	else {
		if (first == False) {
			if (display_delay >= 0) flush ();
			if (display_delay > 0)
#ifdef msdos
				delay (display_delay);
#else
#   ifdef sysV
				/* ??? */;
#   else
				(void) usleep (1000 * display_delay);
#   endif
#endif
		}
		line_print (line);
	}
  }
}

/*
 * Display () shows count + 1 lines on the terminal starting at the given 
 * coordinates. At end of file, the rest of the screen is blanked.
 * When count is negative, a backwards print from `line' will be done.
 */
void
display (y_coord, line, count, new_pos)
  int y_coord, new_pos;
  register LINE * line;
  register int count;
{
  int y_max = y_coord + count;
  int y_off;

/* Find new startline if count is negative */
  if (count < 0) {
	line = proceed (line, count);
	count = - count;
  }

  clear_menus ();

  display_line_at (new_pos, line, y_coord, y_max, True);
  y_off = 0;
  while (y_off < count) {
	y_off ++;
	display_line_at (new_pos - y_off, line, y_coord, y_max, False);
	display_line_at (new_pos + y_off, line, y_coord, y_max, False);
  }
#ifdef UNUSED
/* old code, building the display from top to bottom (how boring): */
/* with this code, XBREAK must be set to XMAX - 1 */
/* Print the lines */
  set_cursor (0, y_coord);
  while (line != tail && count -- >= 0) {
	line_print (line);
	line = line->next;
  }

/* Print the blank lines (if any) */
  if (loading == False) {
	while (count -- >= 0) {
		clear_eol ();
		putchar ('\n');
	}
  }
#endif
}


/*  ==================================================================	*
 *			Mined Terminal Dialog				*
 *  ==================================================================	*/

/*
 * promptyn reads in a 'y' or 'n' character.
 */
character
promptyn ()
{
  register character c;
  while ((c = readchar ()) != 'y' && c != 'n' && c != '\033' && quit == False) {
	ring_bell ();
	flush ();
  }
  if (c == '\033') quit = True;
  return c;
}

/*
 * In case of a QUIT signal, swallow the dummy char generated by catchquit ()
 * called by re_search () and change ()
 */
void
swallow_dummy_quit_char ()
{
#ifdef UNUSED
  (void) readchar (); /* Swallow away a quit character delivered by QUIT */
/* Not needed because this character is ignored by being the CANCEL command */
#endif
}

/*
 * Readchar () reads one character from the terminal.
 * There are problems due to interruption of the read operation by signals
 * (QUIT, WINCH). The waitingforinput flag is only a partial solution.
 * Unix doesn't provide sufficient facilities to handle these situations
 * neatly and properly. Moreover, different Unix versions yield different
 * surprising effects. However, the use of select () could still be
 * an improvement.
 */
int
readchar ()
{
  register character c;

#ifdef msdos
  FLAG waiting;

  if (winchg == True && waitingforinput == False) RDwin ();
	/* In the Unix version, this is now done in __readchar () */
  waiting = waitingforinput;
	/* must be saved since in the MSDOS version, readchar can 
	   be called recursively */
#endif
  waitingforinput = True;

  c = _readchar ();
  while (c == (character) FUNcmd && (* keyproc) == MOUSEescape 
	&& mouse_button == releasebutton && report_release == False)
	c = _readchar ();
  report_release = False;

#ifdef msdos
  waitingforinput = waiting;
#else
  waitingforinput = False;
#endif

  /* the modification   if (quit == True) c = QUITCHAR;   (now in __readchar)
     must not be placed after resetting the flag  waitingforinput = False;  .
     Otherwise a QUIT signal coming in just between these two would
     discard the last valid character just taken up. */

  return c;
}

/*-------------------------------------------------------------------------*/

/*
 * DIRECT...getxy () reads in the information part of a cursor escape sequence
 */
void
DIRECTxtermgetxy ()
{
  character button;
  int xpos, ypos;

  mouse_lastbutton = mouse_button;
  mouse_lastxpos = mouse_xpos;
  mouse_lastypos = mouse_ypos;

  button = _readchar ();
  if (button == QUITCHAR) quit = True;
  mouse_shift = button & 0x1C;
  button &= 0x03;
  if (button == 0x00) mouse_button = leftbutton;
  else if (button == 0x01) mouse_button = middlebutton;
  else if (button == 0x02) mouse_button = rightbutton;
  else mouse_button = releasebutton;

  xpos = _readchar ();
  if (xpos == QUITCHAR) quit = True;
  mouse_xpos = xpos - 33;
  ypos = _readchar ();
  if (ypos == QUITCHAR) quit = True;
  mouse_ypos = ypos - 33 - MENU;
}

void
DIRECTcrttoolgetxy ()
{
  character c;
  int xpos, ypos;

  mouse_lastbutton = mouse_button;
  mouse_lastxpos = mouse_xpos;
  mouse_lastypos = mouse_ypos;

  c = get_digits (& ypos); /* c should be ';' */
  c = get_digits (& xpos);
  if (c == 'l') mouse_button = leftbutton;
  else if (c == 'm') mouse_button = middlebutton;
  else if (c == 'r') mouse_button = rightbutton;
  else mouse_button = releasebutton;

  mouse_ypos = ypos - 1 - MENU;
  mouse_xpos = xpos - 1;
}

/*
 * DIRECT... () reads in a direct cursor movement input sequence.
 * Then it either performs a direct function (move/copy/paste) or 
 * invokes the menu functions.
 * Is no longer called as this is already handled in _readchar ().
 */
void
DIRECTxterm ()
{
  DIRECTxtermgetxy ();
  MOUSEescape ();
}

void
DIRECTcrttool ()
{
  DIRECTcrttoolgetxy ();
  MOUSEescape ();
}


/*-------------------------------------------------------------------------*/

#ifndef msdos

extern struct {
	char * fk;
	void (* fp) ();
} keycode [];

#define MAXCODELEN 7 /* max. length of function key sequence to be detected,
			depending on the keycode table */

/*
 * queue collects the keys of an Escape sequence typed in until the 
 * sequence can be detected or rejected.
 * If the queue is not empty, queue [0] contains the character next 
 * to be delivered by _readchar () (it's not a ring buffer).
 * The queue contents is always terminated by a '\0', so queue can also 
 * be taken as a character string.
 */
static character queue [MAXCODELEN + 1], * endp = queue;
static character ctrl_queue [MAXCODELEN + 1], * endcp = ctrl_queue;

int
q_empty ()
{
  return (endp == queue ? 1 : 0);
}

int
q_notfull ()
{
  return (endp - queue == MAXCODELEN ? 0 : 1);
}

void
q_clear ()
{
  endp = queue;
  endcp = ctrl_queue;
}

int
q_len ()
{
  return endp - queue;
}

void
q_put (c)
  character c;
/* queue must not be full prior to this call! */
{
  * endp = c;
  * ++ endp = '\0';
  if (c == '\0')
	* endcp = '@';
  else	* endcp = c;
  * ++ endcp = '\0';
}

character
q_get ()
{
  character c;
  register character * pd, * ps;

  c = * queue; pd = queue; ps = pd + 1;
  while (ps <= endp) * pd ++ = * ps ++;
  if (endp > queue) {
	endp --;
	endcp --;
  }
  return c;
}

/*
 * Look up key sequence in keycode table.
 * findkey (str) >=  0: str == keycode [findkey (str)].fk
 *		 == -1: str is prefix of some entry in keycode
 *		 == -2: str is not contained in keycode
 */
int
findkey (str)
  char * str;
{
  static int lastmatch = 0;	/* last index with string matching prefix */
  register int i;

  if (keycode [0].fk == NIL_PTR) return -2;
  i = lastmatch;
  do {
	if (strncmp (str, keycode [i].fk, strlen (str)) == 0) {
		lastmatch = i;
		return (strlen (str) == strlen (keycode [i].fk) ? i : -1);
	}
	++ i;
	if (keycode [i].fk == NIL_PTR) i = 0;
  } while (i != lastmatch);
  return -2;
}

/*
 * Is a character available within a specified number of milliseconds ?
 */
int
char_ready_within (msec)
  int msec;
{
  return (q_len () > 0) || inputreadyafter (input_fd, msec);
}

#else /* ndef msdos */

int
char_ready_within (msec)
  int msec;
{
  return inputreadyafter (input_fd, msec);
}

#endif /* ndef msdos */

void (* keyproc) () = I;
character (* accentproc) ();

/*
 * Read a character from terminal, considering function keys and 
 * composing special character of an 8 bit character set.
 * _readchar () takes the following actions:
 *  -	function key sequences according to the table 'keycode' are 
 *	transformed into a special controlling character which is 
 *	assigned the function FUNKEY. Also the intended editor function, 
 *	as taken from the table 'keycode', is saved in a variable for 
 *	use by FUNKEY.
 *  -	the prefix keys for diacritic and special characters are 
 *	combined with the following key to make up the character.
 */
int
_readchar ()
{
  register character ch;
#ifndef msdos
  int res;
#endif

#ifndef msdos
  if (q_len () > 0) return q_get ();
#endif

  ch = __readchar ();
  last_char_latin1 = False;

  if (ch == '\000') {
	ch = __readchar ();
	keyproc = pc_key_map [ch];
#ifdef DEBUG
	if ((voidfunc) keyproc == (voidfunc) I) return ch;
	/* (voidfunc) is an identity cast here. It seems to be required 
	   for the sake of the apparently totally rotten microvax compiler */
#endif
	accentproc = (charfunc) keyproc;
	if ((accentproc == grave) || (accentproc == circumflex)
		 || (accentproc == acute) || (accentproc == diaeresis)
		 || (accentproc == tilde) || (accentproc == angstrom))
		{ch = (* accentproc) (readchar ());
		 keyproc = I;
		 last_char_latin1 = True;
		 return ch;
		}
	else return FUNcmd /* index of FUNKEY */;
  }
#ifndef msdos
  else if (ch == '\033') {
	/* q_clear (); */
	q_put (ch);
	while  ((res = findkey (ctrl_queue)) == -1 /* prefix of table entry */
		&& q_notfull ()
		&& inputreadyafter (input_fd, escape_delay)
	       )
	    q_put (__readchar ());
	if (quit == True) return '\0';
	else if (res < 0) /* key pattern not detected in keycode table */
	     /* {if (q_len () > 1) return 0;
		 else return ch;} */
	     return q_get () /* just deliver the typed characters */;
	else {
	     q_clear ();

	     keyproc = keycode [res].fp;
	     if (keyproc == DIRECTxterm) {
		DIRECTxtermgetxy ();
		keyproc = MOUSEescape;
	     } else if (keyproc == DIRECTcrttool) {
		DIRECTcrttoolgetxy ();
		keyproc = MOUSEescape;
	     }

	     accentproc = (charfunc) keyproc;
	     if ((accentproc == grave) || (accentproc == circumflex)
		 || (accentproc == acute) || (accentproc == diaeresis)
		 || (accentproc == tilde) || (accentproc == angstrom))
		{ch = (* accentproc) (readchar ());
		 keyproc = I;
		 last_char_latin1 = True;
		 return ch;
		}
	     else return FUNcmd /* index of FUNKEY */;
	}
  }
#endif
  else if (transdeutsch == True) {
	switch (ch) {
		case '{': return '\344';
		case '|': return '\366';
		case '}': return '\374';
		case '~': return '\337';
		case '[': return '\304';
		case '\\': return '\326';
		case ']': return '\334';
		case (unsigned char) '\344': return '{';
		case (unsigned char) '\366': return '|';
		case (unsigned char) '\374': return '}';
		case (unsigned char) '\337': return '~';
		case (unsigned char) '\304': return '[';
		case (unsigned char) '\326': return '\\';
		case (unsigned char) '\334': return ']';
		default: return ch;
	}
  } else
	return ch;
}


/*  ==================================================================	*
 *			Scroll Bar Display				*
			- under construction -
 *  ==================================================================	*/

/* mined displays last_y + 1 lines in a display area of SCREENMAX + 1 lines */
/* the file has (approx.?) total_lines lines */
/* the file position was in line # line_number */

static int prev_range_start = 0;
static int prev_range_size = 0;

void
display_scrollbar (update)
  FLAG update;
{
  int range_start = (line_number - y) * (SCREENMAX + 1) / total_lines;
  int range_size = (last_y + 1) * (SCREENMAX + 1) / total_lines;
  int i;
  int disp_start;
  int disp_end;

  if (range_size == 0) range_size = 1;

#define scrollbarx
#ifdef scrollbar
  if (update == False ||
      (update == True && 
	(range_start != prev_range_start || range_size != prev_range_size))
     ) {
	disp_start = range_start - 1;
	disp_end = range_start - 2 + range_size;

	for (i = 0; i <= SCREENMAX; i ++) {
		set_cursor (XMAX, i );
		if (i >= disp_start && i <= disp_end) {
			putstring ("[41m");
			putchar (' ');
			putstring ("[0m");
		} else {
			putstring ("[45m");
			putchar (' ');
			putstring ("[0m");
		}
	}
  }
#endif

  prev_range_start = range_start;
  prev_range_size = range_size;
}


/*  ==================================================================	*
 *			Status Line Dialog				*
 *  ==================================================================	*/

/*
 * Display a line telling how many chars and lines the file contains. Also tell
 * whether the file is readonly and/or modified.
fstatus (mess, bytes, chars) is
	file_status ((mess), (bytes), file_name, \
		     total_lines, True, writable, modified, viewonly, (chars))
 */
/* directly called only from WB: file_status 
   (msg_done, chars_saved, file_name, lines_saved, False, True, False, False); */
void
file_status (message, count, file, lines, textstat, writefl, changed, viewing, charcount)
  char * message;
  register long count;		/* Contains number of bytes in file */
  register long charcount;	/* Contains number of characters in file */
  char * file;
  int lines;
  FLAG textstat, writefl, changed, viewing;
{
  register LINE * line;
  register int line_num = 0;
  static char msg [maxLINE_LEN + 40];	/* Buffer to hold line */
  char yank_msg [maxLINE_LEN];	/* Buffer for msg of yank_file */

  if (count < 0)	/* # chars not given; count them */
	for (line = header->next; line != tail; line = line->next) {
		count += length_of (line->text);
		charcount += char_count (line->text);
		line_num ++;
		if (line == cur_line) line_number = line_num;
	}
  else			/* check for current line # only */
	for (line = header->next; line != cur_line->next; line = line->next) {
		line_num ++;
		if (line == cur_line) line_number = line_num;
	}

  if (yank_status == VALID && textstat == True)	/* Append buffer info */
	/* build_string (yank_msg, " Buffer: %ld char%s.", chars_saved,
					(chars_saved == 1L) ? "" : "s");
	*/
	build_string (yank_msg, "");
	/* Empty paste buffer is only an initial condition and thus this 
	   would be no significant information; since buffer contents 
	   may be appended, the exact count is not available anyway. */
  else
	yank_msg [0] = '\0';

  build_string (msg,
		(textstat == True) 
			? "%s %s%s%s%s, %d line%s, %ld char%s (%ld bytes). Line %d (page %d).%s"
			: "%s %s%s%s%s, %d line%s, %ld char%s.",
		message,
		(rpipe == True && * message != '[') ?
			"standard input" : file,
			/* previously only basename (file) was printed */
		(viewing == True) ? " (View only)" : "",
		(changed == True) ? " (modified)" : "",
		(writefl == False) ? " (Readonly)" : "",
		lines, (lines == 1) ? "" : "s",
		charcount, (charcount == 1L) ? "" : "s",
		count,
		line_number,
		(lines_per_page != 0)
			? (line_number - 1) / lines_per_page + 1
			: 1,
		yank_msg);

  status_msg (msg);		/* Print the information */
}

/*
 * Input () reads a string from the terminal.
 * Return values:
 *	when QUIT character typed => ERRORS
 *	when empty input and clearfl == True: NO_INPUT
 *	else: FINE
 */
character *
input_byte (inbuf, ptr, c)
  character * inbuf;
  character * ptr;
  character c;
{
	if (c > '\0' && (ptr - inbuf) < maxLINE_LEN) {
		* ptr ++ = c;
		* ptr = '\0';
		print_char (c);
		putstring (" \b");
	} else {
		ring_bell ();
	}
	return ptr;
}
character *
input_unicode (inbuf, ptr, unichar)
  character * inbuf;
  character * ptr;
  unsigned long unichar;
{
	character c;

	if (unichar < 0x80) {
	    if ((ptr - inbuf) >= maxLINE_LEN)
		ring_bell ();
	    else {
		* ptr ++ = unichar;
		print_char (unichar);
		* ptr = '\0';
	    }
	} else if (unichar < 0x800) {
	    if ((ptr - inbuf) + 2 > maxLINE_LEN)
		ring_bell ();
	    else {
		c = 0xC0 | (unichar >> 6);
		* ptr ++ = c;
		print_char (c);
		c = 0x80 | (unichar & 0x3F);
		* ptr ++ = c;
		print_char (c);
		* ptr = '\0';
	    }
	} else {
	    if ((ptr - inbuf) + 3 > maxLINE_LEN)
		ring_bell ();
	    else {
		c = 0xE0 | (unichar >> 12);
		* ptr ++ = c;
		print_char (c);
		c = 0x80 | ((unichar >> 6) & 0x3F);
		* ptr ++ = c;
		print_char (c);
		c = 0x80 | (unichar & 0x3F);
		* ptr ++ = c;
		print_char (c);
		* ptr = '\0';
	    }
	}
	putstring (" \b");
	return ptr;
}
character *
input_character (inbuf, ptr, c)
  character * inbuf;
  character * ptr;
  character c;
{
	if (utf8_text == True)
		input_unicode (inbuf, ptr, c);
	else
		input_byte (inbuf, ptr, c);
}

int
input (inbuf, clearfl, term_input)
  character * inbuf;
  FLAG clearfl;
  char * term_input;
{
  character * ptr;
  register character c;
  char * term_chars;

  unsigned long unichar;
  char mnemonic [maxLINE_LEN];
  char * mnemo;
  character c1;
  int utfcount;
  int i;

  ptr = inbuf;
  * ptr = '\0';
  while (quit == False) {
     if (lpos >= XBREAK) pagewrapped = True;

     flush ();
     c = readchar ();
     for (term_chars = term_input; * term_chars != '\0'; term_chars ++)
	if (c == * term_chars)
		c = '\n';

     switch (c) {
	case '\b' :		/* Erase previous char */
	case '\177' /* DEL */ :
	    if (ptr > inbuf) {
		ptr --;
		diagdisp_off ();
		if (iscontrol (* ptr)) {
			putstring (" \b\b\b  \b\b");
			lpos = lpos - 2;
		} else if (Chinese == True && inChinese (inbuf, ptr)) {
			ptr --;
			putstring (" \b\b\b  \b\b");
			lpos = lpos - 2;
		} else {
			ptr ++;
			precede_char (& ptr, inbuf);
			putstring (" \b\b \b");
			lpos = lpos - 1;
		}
		diagdisp_on ();
		putstring (" \b");
		* ptr = '\0';
	    } else
		ring_bell ();
	    break;
	case QUITCHAR :
	case '\033' :
		quit = True;
		break;
	case '\n' :		/* End of input */
	case '\015' :
		/* If inbuf is empty clear status_line */
		return (ptr == inbuf && clearfl == True) ?
			NO_INPUT : FINE;
	case (character) FUNcmd:
		return ERRORS;

	default :

		if (c == control_prefix /* ^V/^P */) {
			c = readchar ();
			if (c == ring || c == ',')
				{c = angstrom (readchar ());
				 ptr = input_character (inbuf, ptr, c);
				}
			else switch (c) {
			case '"':
			case ':':	{c = diaeresis (readchar ());
					 ptr = input_character (inbuf, ptr, c);
					 break;
					}
			case '\'':	{c = acute (readchar ());
					 ptr = input_character (inbuf, ptr, c);
					 break;
					}
			case '`':	{c = grave (readchar ());
					 ptr = input_character (inbuf, ptr, c);
					 break;
					}
			case '^':	{c = circumflex (readchar ());
					 ptr = input_character (inbuf, ptr, c);
					 break;
					}
			case '~':	{c = tilde (readchar ());
					 ptr = input_character (inbuf, ptr, c);
					 break;
					}
			default:
			    if (utf8_text == True) {
				if (c == '#') {
				  i = 0;
				  unichar = 0;
				  while (i < 4 && quit == False) {
					c = readchar ();
					if (c == QUITCHAR) quit = True;
					if (c == '\033') quit = True;

					if (c >= '0' && c <= '9') {
						i ++;
						unichar *= 16;
						unichar += c - '0';
					} else if (c >= 'A' && c <= 'F') {
						i ++;
						unichar *= 16;
						unichar += c - 'A' + 10;
					} else if (c >= 'a' && c <= 'f') {
						i ++;
						unichar *= 16;
						unichar += c - 'a' + 10;
					} else
						quit = True;
				  }
				  if (quit == True) {
					ring_bell ();
					break;
				  }
				} else if (c == ' ') {
					/* read mnemonic ... */
					mnemo = mnemonic;
					while (c != '\n' && quit == False) {
						c = readchar ();
						if (c == QUITCHAR) quit = True;
						if (c == '\033') quit = True;

						if (c == ' ') c = '\n';
						if (c != '\n')
							* mnemo ++ = c;
					}
					* mnemo = '\0';
					if (quit == True) {
						ring_bell ();
						break;
					}

					unichar = compose_mnemo (mnemonic);
				} else if ( ((c > ' ') && (c < 'Z'))
				            || (c > '^') ) {
					unichar = compose (c, readchar ());
				} else if (c > '\0' 
				           && (ptr - inbuf) < maxLINE_LEN) {
					* ptr ++ = c;
					* ptr = '\0';
					print_char (c);
					putstring (" \b");
					break;
				} else {
					ring_bell ();
					break;
				}

				/* enter unichar as utf-8 ... */
				if (unichar < 0x80) {
				    if ((ptr - inbuf) >= maxLINE_LEN)
					ring_bell ();
				    else {
					* ptr ++ = unichar;
					print_char (unichar);
					* ptr = '\0';
				    }
				} else if (unichar < 0x800) {
				    if ((ptr - inbuf) + 2 > maxLINE_LEN)
					ring_bell ();
				    else {
					c = 0xC0 | (unichar >> 6);
					* ptr ++ = c;
					print_char (c);
					c = 0x80 | (unichar & 0x3F);
					* ptr ++ = c;
					print_char (c);
					* ptr = '\0';
				    }
				} else {
				    if ((ptr - inbuf) + 3 > maxLINE_LEN)
					ring_bell ();
				    else {
					c = 0xE0 | (unichar >> 12);
					* ptr ++ = c;
					print_char (c);
					c = 0x80 | ((unichar >> 6) & 0x3F);
					* ptr ++ = c;
					print_char (c);
					c = 0x80 | (unichar & 0x3F);
					* ptr ++ = c;
					print_char (c);
					* ptr = '\0';
				    }
				}
				putstring (" \b");
			    } else {
				if ( ((c > ' ') && (c < 'Z'))
				            || (c > '^') ) {
					c = compose (c, readchar ());
				} else {
					if (c != '\177') c = c & '\237';
				}
				if (c > '\0' 
				    && (ptr - inbuf) < maxLINE_LEN) {
					* ptr ++ = c;
					* ptr = '\0';
					print_char (c);
					putstring (" \b");
				} else
					ring_bell ();
			    }
			}

		} else if (utf8_text == True && multichar (c)) {
		   if (utf8_input == True) {
		/* UTF-8 input for UTF-8 text */
			if ((c & 0xE0) == 0xC0)
				utfcount = 2;
			else if ((c & 0xF0) == 0xE0)
				utfcount = 3;
			else if ((c & 0xF8) == 0xF0)
				utfcount = 4;
			else if ((c & 0xFC) == 0xF8)
				utfcount = 5;
			else if ((c & 0xFE) == 0xFC)
				utfcount = 6;
			else /* ignore illegal UTF-8 code */
				utfcount = 0;
			if (utfcount > 0
			    && (ptr - inbuf) + utfcount <= maxLINE_LEN) {
				* ptr ++ = c;
				print_char (c);
				utfcount --;
				while (utfcount > 0) {
					c = readchar ();
					* ptr ++ = c;
					print_char (c);
					utfcount --;
				}
				* ptr = '\0';
				putstring (" \b");
			} else
				ring_bell ();
		   } else {
		/* 8-bit input for UTF-8 text */
		      if ((ptr - inbuf) + 1 < maxLINE_LEN) {
			c1 = 0xC0 | (c >> 6);
			* ptr ++ = c1;
			print_char (c1);
			c1 = 0x80 | (c & 0x3F);
			* ptr ++ = c1;
			print_char (c1);
			* ptr = '\0';
			putstring (" \b");
		      } else
			ring_bell ();
		   }

		} else if (utf8_input == True && multichar (c)) {
		/* UTF-8 input for 8-bit text */
			if ((c & 0xE0) == 0xC0) {
				utfcount = 2;
				unichar = c & 0x1F;
			} else if ((c & 0xF0) == 0xE0) {
				utfcount = 3;
				unichar = c & 0x0F;
			} else if ((c & 0xF8) == 0xF0) {
				utfcount = 4;
				unichar = c & 0x07;
			} else if ((c & 0xFC) == 0xF8) {
				utfcount = 5;
				unichar = c & 0x03;
			} else if ((c & 0xFE) == 0xFC) {
				utfcount = 6;
				unichar = c & 0x01;
			} else /* ignore illegal UTF-8 code */
				utfcount = 0;
			if (utfcount > 0
			    && (ptr - inbuf) < maxLINE_LEN) {
				utfcount --;
				while (utfcount > 0) {
					c = readchar ();
					unichar = (unichar << 6) | (c & 0x3F);
					utfcount --;
				}
				c = unichar & 0xFF;
				if (c == unichar) {
					* ptr ++ = c;
					print_char (c);
					* ptr = '\0';
					putstring (" \b");
				} else
					ring_bell ();
			} else
				ring_bell ();

		} else if (Chinese == True && multichar (c)) {
		/* 8/16-bit (Chinese) input for Chinese text */
		   if ((ptr - inbuf) + 1 < maxLINE_LEN) {
			* ptr ++ = c;
			print_char (c);
			c = readchar ();
			* ptr ++ = c;
			print_char (c);
			* ptr = '\0';
			putstring (" \b");
		   }
		   else
			ring_bell ();

		} else if ((ptr - inbuf) < maxLINE_LEN) {
		/* 8-bit input for 8-bit text */
			if ((c > '\0')) {
				* ptr ++ = c;
				* ptr = '\0';
				print_char (c);
				putstring (" \b");
			}
			else
				ring_bell ();

		} else
			ring_bell ();
     }
  }
  quit = False;
  return ERRORS;
}

/*
 * Show concatenation of s1 and s2 on the status line (bottom of screen)
 * If revfl is True, turn on reverse video on both strings. Set stat_visible
 * only if bottom_line is visible.
 * The return value is FINE except for get_string, where it is taken
 * from the call to input ().
status_line (str1, str2) is (void) bottom_line (ON, (str1), (str2), NIL_PTR, False, "")
status_msg (str)	 is status_line (str, NIL_PTR)
status_beg (str)	 is (void) bottom_line (ON, (str), NIL_PTR, NIL_PTR, True, "")
error (str1, str2)	 is (void) bottom_line (ON, (str1), (str2), NIL_PTR, False, "")
clear_status ()		 is (void) bottom_line (OFF, NIL_PTR, NIL_PTR, NIL_PTR, False, "")
get_string (str1, str2, fl, term_chars)
			 is bottom_line (ON, (str1), NIL_PTR, (str2), fl, term_chars)
 */
FLAG lastrevfl;
char * lastinbuf;
FLAG input_active = False;
char status_buf [maxLINE_LEN];

void
rd_bottom_line ()
{
  set_cursor (0, YMAX);
  diagdisp_on ();
  if (lastinbuf == NIL_PTR)
	printlim_string (status_buf, XBREAK);
  else {
	print_string (status_buf);
	print_string (lastinbuf);
  }
  if (input_active == False) {
	diagdisp_off ();
	set_cursor_xy ();	/* Set cursor back to old position */
  }
  flush ();	/* Perform the actual screen output */
}

int
bottom_line (revfl, s1, s2, inbuf, statfl, term_input)
  FLAG revfl;
  char * s1, * s2;
  char * inbuf;
  FLAG statfl;
  char * term_input;
{
  int ret = FINE;

  if (inbuf != NIL_PTR) * inbuf = '\0';
  lastrevfl = revfl;
  lastinbuf = inbuf;

  if (pagewrapped == True) {
	status_buf [0] = '\0';
	RD ();
	pagewrapped = False;
  }

  build_string (status_buf, " %s%s ", unnull (s1), unnull (s2));
		/* (s1 == NIL_PTR) ? "" : s1, (s2 == NIL_PTR) ? "" : s2); */

  if (revfl == ON && stat_visible == True) {
	set_cursor (0, YMAX);
	clear_lastline ();
  }
  set_cursor (0, YMAX);
  if (revfl == ON) {		/* Print rev. start sequence */
	diagdisp_on ();
	stat_visible = True;
  }
  else {			/* Used as clear_status () */
	diagdisp_off ();
	stat_visible = False;
  }

  if (inbuf == NIL_PTR)
	printlim_string (status_buf, XBREAK);
  else {
	print_string (status_buf);
	input_active = True;
	ret = input (inbuf, statfl, term_input);
	input_active = False;
  }

  /* Print normal video */
  diagdisp_off ();
  if (can_clear_eol == True) clear_eol ();
  else {
	put_blanks (XMAX - 1 - lpos);
	set_cursor (lpos, YMAX);
  }

  if (inbuf != NIL_PTR) {
	set_cursor (0, YMAX);
  }
  else if (statfl == True)
	diagdisp_on ();
  else
	set_cursor_xy ();	/* Set cursor back to old position */

  flush ();	/* Perform the actual screen output */
  if (ret != FINE) clear_status ();
  return ret;
}

/*
 * get_number () reads a number from the terminal.
 * get_hex_number () reads a hexadecimal number from the terminal.
 * If the parameter firstdigit is a valid digit, empty input (return) 
 * will yield a valid number; if it is '\0', it won't.
 * The last character typed in is returned.
 * ERRORS is returned on a bad number or on interrupted input.
 * The resulting number is put into the integer the argument points to.
 */
int
get_number (message, firstdigit, result)
  char * message;
  char firstdigit;
  int * result;
{
  register int index;
  register int count;

  status_beg (message);

  if (firstdigit > '\0')
	index = firstdigit;
  else
	index = readchar ();

  if (index == QUITCHAR) quit = True;
  if (index == '\033') quit = True;
  if (quit == False && (index < '0' || index > '9')) {
	error ("Bad number", NIL_PTR);
	return ERRORS;
  }

/* Convert input to a decimal number */
  count = 0;
  while (index >= '0' && index <= '9' && quit == False) {
	print_char (index); flush ();
	if (lpos >= XBREAK) pagewrapped = True;
	count *= 10;
	count += index - '0';
	index = readchar ();
	if (index == QUITCHAR) quit = True;
	if (index == '\033') quit = True;
  }

  clear_status ();
  if (quit == True) {
	clear_status ();
	return ERRORS;
  }
  * result = count;
  return index;
}

int
get_hex_number (message, result)
  char * message;
  unsigned long * result;
{
  register int index;
  register int count = 0;
  int i;
  FLAG ok = False;

  status_beg (message);

  i = 0;
  while (i < 9 && ok == False) {
	index = readchar ();

	if (index >= '0' && index <= '9') {
		i ++;
		count *= 16;
		count += index - '0';
		print_char (index); flush ();
	} else if (index >= 'A' && index <= 'F') {
		i ++;
		count *= 16;
		count += index - 'A' + 10;
		print_char (index); flush ();
	} else if (index >= 'a' && index <= 'f') {
		i ++;
		count *= 16;
		count += index - 'a' + 10;
		index = index - 'a' + 'A';
		print_char (index); flush ();
	} else if (index == '\b' || index == '\177') {
		if (i > 0) {
			i --;
			count /= 16;
			putstring ("\b \b"); flush ();
			if (lpos >= XBREAK) pagewrapped = True;
			lpos --;
		}
	} else if (index == ' ' || index == '\n') {
		ok = True;
	} else {
		quit = True;
		ok = True;
	}

	if (lpos >= XBREAK) pagewrapped = True;
  }

  clear_status ();
  if (quit == True) {
	error ("Bad hex value", NIL_PTR);
	quit = False;
	return ERRORS;
  }
  * result = count;
  return index;
}

/*
 * get_digits () reads in a number. In contrast to get_number, it does no
 * echoing, no messaging, and it does not require any digits at all.
 * The last character typed in is returned.
 * The resulting number is put into the integer the arguments points to.
 * It is for use within I/O routines (e.g. reading mouse escape sequences 
 * or cursor position report) and thus calls _readchar ().
 */
int
get_digits (result)
  int * result;
{
  register int index;
  register int count;

  index = _readchar ();
  if (index == QUITCHAR) quit = True;
  * result = -1;
/* Convert input to a decimal number */
  count = 0;
  while (index >= '0' && index <= '9' && quit == False) {
	count *= 10;
	count += index - '0';
	* result = count;
	index = _readchar ();
	if (index == QUITCHAR) quit = True;
  }

  if (quit == True) {
	return QUITCHAR;
  }
  return index;
}

/*
 * Get_file () reads a filename from the terminal.
 */
int
get_file (message, file)
  char * message, * file;
{
  int ret = get_string (message, file, True, "");
#ifndef msdos
  char * filei;
  char file1 [maxLINE_LEN];

  if (file [0] == '~' && file [1] == '/') {
	filei = file; filei ++;
	build_string (file1, "%s%s", unnull (getenv ("HOME")), filei);
	build_string (file, file1);
  }
#endif
  return ret;
}


/*  ==================================================================	*
 *			text modification routines			*
 *  ==================================================================	*/

/*
 * make_line installs the buffer into a LINE structure.
 * It returns a pointer to the allocated structure.
 */
LINE *
make_line (buffer, length)
  char * buffer;
  int length;
{
  register LINE * new_line = alloc_header ();

  if (new_line == NIL_LINE) {
	ring_bell ();
	error ("Cannot allocate more memory for new line header", NIL_PTR);
	return NIL_LINE;
  } else {
    new_line->text = alloc (length + 1);
    if (new_line->text == NIL_PTR) {
	ring_bell ();
	error ("Cannot allocate more memory for new line", NIL_PTR);
	return NIL_LINE;
    } else {
	new_line->shift_count = 0;
	copy_string (new_line->text, buffer);
	return new_line;
    }
  }
}

/*
 * Line_insert () inserts a new line with text pointed to by `string'.
 * It returns the address of the new line.
 */
LINE *
line_insert (line, string, len)
  register LINE * line;
  char * string;
  int len;
{
  register LINE * new_line;

/* Allocate space for LINE structure and text */
  new_line = make_line (string, len);

  if (new_line != NIL_LINE) {
/* Install the line into the double linked list */
	new_line->prev = line;
	new_line->next = line->next;
	line->next = new_line;
	new_line->next->prev = new_line;
/* Increment total_lines */
	total_lines ++;
  }
  return new_line;
}

/*
 * Insert () insert the string `string' at the given line and location.
 */
int
insert (line, location, string)
  register LINE * line;
  char * location, * string;
{
  register char * bufp = text_buffer;	/* Buffer for building line */
  register char * textp = line->text;
  char * newtext;

  if (viewonly == True)
	{viewonlyerr (); return ERRORS;}

  if (length_of (textp) + text_bytes_of (string) >= MAX_CHARS) {
	error ("Line too long", NIL_PTR);
	return ERRORS;
  }

/* Copy part of line until `location' has been reached */
  while (textp != location)
	* bufp ++ = * textp ++;

/* Insert string at this location */
  while (* string != '\0')
	* bufp ++ = * string ++;
  * bufp = '\0';

/* First, allocate memory for next line contents to make sure the */
/* operation succeeds or fails as a whole */
  newtext = alloc (length_of (text_buffer) + length_of (location) + 1);
  if (newtext == NIL_PTR) {
	ring_bell ();
	error ("Cannot allocate memory for insertion", NIL_PTR);
	return ERRORS;
  }
  else { /* Install the new text in this line */
	if (* (string - 1) == '\n') {		/* Insert a new line */
		if (line_insert (line, location, length_of (location)) == NIL_LINE)
			return ERRORS;
		set_modified ();
	}
	else		/* Append last part of line to text_buffer */
		copy_string (bufp, location);

	free_space (line->text);
	set_modified ();
	line->text = newtext;
	copy_string (line->text, text_buffer);
	return FINE;
  }
}

/*
 * Line_delete () deletes the argument line out of the line list. The pointer
 * to the next line is returned.
 */
LINE *
line_delete (line)
  register LINE * line;
{
  register LINE * next_line = line->next;

/* Delete the line */
  line->prev->next = line->next;
  line->next->prev = line->prev;

/* Free allocated space */
  free_space (line->text);
  free_header (line);

/* Decrement total_lines */
  total_lines --;

  return next_line;
}

/*
 * Delete_text () deletes all the characters (including newlines) between the 
 * startposition and endposition and fixes the screen accordingly. It 
 * displays the number of lines deleted.
 */
int
delete_text (start_line, start_textp, end_line, end_textp)
  register LINE * start_line;
  LINE * end_line;
  char * start_textp, * end_textp;
{
  register char * textp = start_line->text;
  register char * bufp = text_buffer;	/* Storage for new line->text */
  LINE * line;
  LINE * after_end = end_line->next;
  int line_cnt = 0;			/* Nr of lines deleted */
  int count = 0;
  int shift = 0;			/* Used in shift calculation */
  int nx = x;
  int ret = FINE;
  char * newtext;

  if (viewonly == True)
	{viewonlyerr (); return ret;}

  set_modified ();			/* File has been modified */

/* Set up new line. Copy first part of start line until start_position. */
  while (textp < start_textp) {
	* bufp ++ = * textp ++;
	count ++;
  }

/* Check if line doesn't exceed MAX_CHARS */
  if (count + length_of (end_textp) >= MAX_CHARS) {
	error ("Line too long", NIL_PTR);
	return ret;
  }

/* Copy last part of end_line if end_line is not tail */
  copy_string (bufp, (end_textp != NIL_PTR) ? end_textp : "\n");

/* Delete all lines between start and end_position (including end_line) */
  line = start_line->next;
  while (line != after_end && line != tail) {
	/* Here, the original mined compared with end_line->next which has 
	   already been discarded when the comparison should become true.
	   This severe error remained undetected until I ported to MSDOS */
	line = line_delete (line);
	line_cnt ++;
  }

/* Check if last line of file should be deleted */
  if (end_textp == NIL_PTR && length_of (start_line->text) == 1 && total_lines > 1) {
	start_line = start_line->prev;
	(void) line_delete (start_line->next);
	line_cnt ++;
  }
  else {	/* Install new text */
	newtext = alloc (length_of (text_buffer) + 1);
	if (newtext == NIL_PTR) {
		ring_bell ();
		error ("No more memory after deletion", NIL_PTR);
		ret = ERRORS;
	} else {
		free_space (start_line->text);
		start_line->text = newtext;
		copy_string (start_line->text, text_buffer);
	}
  }

/* Fix screen. First check if line is shifted. Perhaps we should shift it back */
#ifdef UNUSED
/* !!! This resulted in a positioning error when a line containing TABs */
/* !!! was shifted back. So better leave it.				*/
  if (get_shift (start_line->shift_count)) {
	shift = (XBREAK - count_chars (start_line)) / SHIFT_SIZE;
	if (shift > 0) {		/* Shift line `shift' back */
		if (shift >= get_shift (start_line->shift_count))
			start_line->shift_count = 0;
		else
			start_line->shift_count -= shift;
		nx += shift * SHIFT_SIZE; /* Reset x value */
	}
  }
#endif

  if (line_cnt == 0) {		/* Check if only one line changed */
	if (shift > 0) {	/* Reprint whole line */
		set_cursor (0, y);
		line_print (start_line);
	}
	else {			/* Just display last part of line */
		set_cursor_xy ();
		put_line (start_line, x, True, False);
	}
	move_to (nx, y);	   /* Reset cur_text */
	return ret;
  }

  shift = last_y;	   /* Save value */
  reset (top_line, y);
  if ((line_cnt <= SCREENMAX - y) && can_delete_line == True) {
	clear_status ();
	display (y, start_line, 0, y);
	line = proceed (start_line, SCREENMAX - y - line_cnt + 1);
	while (line_cnt -- > 0) {
		delete_line (y + 1);
		if (line != tail) {
			set_cursor (0, SCREENMAX);
			line_print (line);
			line = line->next;
		}
	}
  }
  else
	display (y, start_line, shift - y, y);
/*  move_to ((line_cnt == 1) ? nx : 0, y); */
  move_to (nx, y);
  return ret;
}


/*  ==================================================================	*
 *				End					*
 *  ==================================================================	*/
