/***********************************************************************\
	Editor mined
	menu handling part
\***********************************************************************/

#include "syntax.h"

#include "mined.h"
#define gotoxy(x, y)	move_cursor (x, y - 1)

#define arrlen(arr)	(sizeof (arr) / sizeof (* arr))


/***********************************************************************\
	Menu tables
\***********************************************************************/

typedef struct {char * itemname;
		void (* itemfu) ();
		char * hopitemname;
		} itemtype;

typedef struct {char * menuname; int menulen;
		itemtype * menuitems;
		} menutype;

local itemtype Filemenu [] =
{
	{"Open", EDIT, ""},
	{"View", VIEW, ""},
	{"Save", WT, ""},
	{"New Name", NN, ""},
	{"Exit", EXED, ""},
	{"Next file", NXTFILE, "First file"},
	{"Prev file", PRVFILE, "Last file"},
	{"Nth file", NTHFILE, ""},
	{"insert file", INSFILE, ""},
	{"write buffer", WB, "append buffer"},
	{"save position", SAVPOS, ""},
	{"Quit", QUED, ""},
};

local itemtype Editmenu [] =
{
	{"Mark", MARK, "Go to Mark"},
	{"Go to Mark", GOMA, ""},
	{"Cut", DT, "Cut-Append"},
	{"Copy", YA, "Append"},
	{"Paste", PT, "Paste external"},
	{"Set marker ...", MARKER, ""},
	{"Go marker ...", GOMARKER, ""},
};

local itemtype Searchmenu [] =
{
	{"Find", SFW, "FindIdf"},
	{"FindReverse", SRV, "FindIdfRev"},
	{"FindNext", RS, "Previous Find"},
	{"FindIdf", SIDFW, ""},
	{"FindIdfRev", SIDRV, ""},
	{"Substitute", GR, ""},
	{"Replace", REPL, ""},
	{"LineReplace", LR, ""},
	{"Go to Mark", GOMA, ""},
	{"matching paren", SCORR, ""},
	{"Go to ...", GOTO, ""},
};

local itemtype Paragraphmenu [] =
{
	{"justify", JUS, "justify other mode"},
	{"set left margin", ADJLM, ""},
	{"set first left margin", ADJFLM, ""},
	{"set next left margin", ADJNLM, ""},
	{"set right margin", ADJRM, ""},
};

local itemtype Specialeditmenu [] =
{
	{"HTML command", HTML, ""},
	{"case switch", LOWCAP, "word case switch"},
	{"umlaut replacement", UML, ""},
	{"-> unicode", changehex, ""},
	{"-> hex", changehex, ""},
	{"-> octal", changeoct, ""},
	{"-> decimal", changedec, ""},
	{"insert control char", CTRl, ""},
};

local itemtype Screensizemenu [] =
{
#ifdef msdos
	{"bigger", screenbigger, "video mode"},
	{"smaller", screensmaller, "graphic mode"},
	{"more lines", screenmorelines, "font bank"},
	{"less lines", screenlesslines, "char height"},
#else
	{"bigger", screenbigger, ""},
	{"smaller", screensmaller, ""},
	{"more lines", screenmorelines, ""},
	{"less lines", screenlesslines, ""},
#endif
	{"switch line #", LNSW, ""},
	{"circul line #", LNCI, ""},
};

local menutype menus [] =
{
	{"File", arrlen (Filemenu), Filemenu},
	{"Edit", arrlen (Editmenu), Editmenu},
	{"Search", arrlen (Searchmenu), Searchmenu},
	{"Special", arrlen (Specialeditmenu), Specialeditmenu},
	{"Paragraph", arrlen (Paragraphmenu), Paragraphmenu},
#ifdef msdos
	{"Screen size", arrlen (Screensizemenu), Screensizemenu},
#endif
};

local itemtype Popupmenu [] =
{
	{"Mark", MARK, "Go to Mark"},
	{"Cut", DT, "Cut-Append"},
	{"Copy", YA, "Append"},
	{"Paste", PT, "Paste external"},
	{"HTML command", HTML, ""},
	{"Find", SFW, "FindIdf"},
	{"FindReverse", SRV, "FindIdfRev"},
	{"FindNext", RS, "Previous Find"},
	{"FindIdf", SIDFW, ""},
	{"FindIdfRev", SIDRV, ""},
	{"Go to Mark", GOMA, ""},
	{"matching paren", SCORR, ""},
};


/***********************************************************************\
	Flag operations
\***********************************************************************/

local void
	togglenothing ()
Begin
End

local char *
	dispnothing ()
Begin
	return " ";
End


local void
	toggleHOP ()
Begin
	If hop_flag > 0
	Then	hop_flag = 0;
	Else	HOP ();
	Fi
End

local char *
	dispHOP ()
Begin
	return hop_flag > 0 ? "#H" : "h";
End


local void
	toggleVIEW ()
Begin
	If viewonly == True
	Then	EDITmode ();
	Else	VIEWmode ();
	Fi
End

local char *
	dispVIEW ()
Begin
	return viewonly == True ? "#V" : "#E";
End


local void
	toggleJUSmode ()
Begin
	JUSmode = 1 - JUSmode;
End

local char *
	dispJUSmode ()
Begin
	return JUSmode == 1 ? "#<" : "#>";
End


local void
	toggleJUSlevel ()
Begin
	JUSlevel ++;
	If JUSlevel > 2
	Then	JUSlevel = 0;
	Fi
End

local char *
	dispJUSlevel ()
Begin
	If JUSlevel == 0
	Then	return "j";
	Elsif JUSlevel == 1
	Then	return "#j";
	Else	return "#J";
	Fi
End


global void
	toggleappend ()
Begin
	If append_flag == True
	Then	append_flag = False;
	Else	append_flag = True;
	Fi
End

local char *
	dispappend ()
Begin
	If append_flag == True
	Then	return "#A";
	Else	return ":";
	Fi
End


global void
	toggleUTF ()
Begin
	If utf8_text == True
	Then	utf8_text = False;
	Else	utf8_text = True;
	Fi
	RD ();
End

local char *
	dispUTF ()
Begin
	If utf8_text == True
	Then	return "#U";
	Else	return "L";
	Fi
End


/***********************************************************************\
	Flag table
\***********************************************************************/

typedef struct {void (* toggle) (); char * (* dispflag) ();} flagitemtype;

local flagitemtype Flagmenu [] =
{
	{toggleUTF, dispUTF},
	{toggleappend, dispappend},
	{toggleHOP, dispHOP},
	{togglenothing, dispnothing},
	{toggleVIEW, dispVIEW},
	{togglenothing, dispnothing},
	{toggleJUSlevel, dispJUSlevel},
	{toggleJUSmode, dispJUSmode},
};


/***********************************************************************\
	Global variables
\***********************************************************************/

local int menu_width = 12;
local int popupmenu_width = 16;
local int first_dirty_line = 0;
local int last_dirty_line = 0;
local int menu_source_line;
local int menuborder = 1;
local int popupmenuborder = 1;
local FLAG pulldownmenu_displayed = False;
local FLAG popupmenu_displayed = False;
local int flags_pos = 62;
local int flags_displayed = 9;


/***********************************************************************\
	putnstr () writes a string to screen at adjusted length
\***********************************************************************/
local void
	putnstr (s, l) char * s; int l;
Begin
	char buf [99];
	int i;

	strcpy (buf, "");
	strncat (buf, s, l);
	For i = strlen (s) While i < l Step i ++
	Do
		strcat (buf, " ");
	Done
	putstring (buf);
End




/***********************************************************************\
	displaymenuheader () displays a menu header
\***********************************************************************/
local void
	displaymenuheader (meni) int meni;
Begin
	gotoxy (meni * menu_width, 0);
	putchar (' ');
	menudisp_on ();
	putnstr ("", menuborder);
	putnstr (menus [meni].menuname, menu_width - 2 - 2 * menuborder);
	putnstr ("", menuborder);
	menudisp_off ();
	putchar (' ');
End


/***********************************************************************\
	calcmenuvalues () determines menu screen positions
\***********************************************************************/
global void
	calcmenuvalues ()
Begin
	/* determine positions for menus and flags display */
	menu_width = XMAX / (arrlen (menus) + 2);
	If menu_width < 10
	Then	menuborder = 0;
	Else	menuborder = 1;
	Fi
	popupmenu_width = 18;
	If popupmenu_width > XMAX
	Then	popupmenu_width = XMAX;
	Fi
	If popupmenu_width < 15
	Then	popupmenuborder = 0;
	Else	popupmenuborder = 1;
	Fi
	flags_pos = arrlen (menus) * menu_width + 
			(XMAX - arrlen (menus) * menu_width) / 3;
	flags_displayed = XMAX - flags_pos - 2;
	If flags_displayed > arrlen (Flagmenu)
	Then	flags_displayed = arrlen (Flagmenu);
	Fi
End


/***********************************************************************\
	displaymenuheaders () displays the menu headers
\***********************************************************************/
global void
	displaymenuheaders ()
Begin
	int i;

	calcmenuvalues ();

	If menu_width < 3
	Then	return;
	Fi

	For i = 0 While i < arrlen (menus) Step i ++
	Do
		displaymenuheader (i);
	Done
End


/***********************************************************************\
	displayflags () displays the menu flags
\***********************************************************************/
global void
	displayflags ()
Begin
	int i;
	char * flags;

	calcmenuvalues ();

	If menu_width < 3
	Then	return;
	Fi

	gotoxy (flags_pos, 0);
	For i = 0 While i < flags_displayed Step i ++
	Do
		flags = (* Flagmenu [i].dispflag) ();
		If flags [0] == '#'
		Then	menudisp_on ();
			put_char (flags [1]);
			menudisp_off ();
		Else	put_char (flags [0]);
		Fi
	Done
End


/***********************************************************************\
	displaymenuline () displays the menu line
\***********************************************************************/
global void
	displaymenuline ()
Begin
	displaymenuheaders ();
	displayflags ();
	top_line_scrolled = False;
End




/***********************************************************************\
	displaymenu () displays a pulldown menu
\***********************************************************************/
local void
	displaymenu (meni) int meni;
Begin
	int i;
	int menulen = menus [meni].menulen;
	itemtype * item;

	If menulen > YMAX - 1
	Then menulen = YMAX - 1;
	Fi

	gotoxy (meni * menu_width, 0);
	menudisp_on ();
	putnstr ("", 1 + menuborder);
	putnstr (menus [meni].menuname, menu_width - 2 - 2 * menuborder);
	putnstr ("", 1 + menuborder);
	For i = 0 While i < menulen Step i ++
	Do
		gotoxy (meni * menu_width, 1 + i);
		putchar (' ');
		menudisp_off ();
		putnstr ("", menuborder);

		item = & (menus [meni].menuitems [i]);
		If hop_flag > 0 && item->hopitemname != NIL_PTR && item->hopitemname [0] != '\0'
		Then	putnstr (item->hopitemname, menu_width - 2 - 2 * menuborder);
		Else	putnstr (item->itemname, menu_width - 2 - 2 * menuborder);
		Fi

		putnstr ("", menuborder);
		menudisp_on ();
		putchar (' ');
	Done
	gotoxy (meni * menu_width, 1 + menulen);
	putnstr ("", menu_width);
	menudisp_off ();
	gotoxy (meni * menu_width, 0);
	If menulen > last_dirty_line
	Then	last_dirty_line = menulen;
	Fi
	pulldownmenu_displayed = True;
End


/***********************************************************************\
	popupmenu () displays the popup menu
\***********************************************************************/
local void
	popupmenu (column, line) int column; int line;
Begin
	int i;
	int menulen = arrlen (Popupmenu);
	itemtype * item;

	gotoxy (column, line);
	menudisp_on ();
	putnstr ("", popupmenu_width);
	For i = 0 While i < menulen Step i ++
	Do
		gotoxy (column, line + 1 + i);
		putchar (' ');
		menudisp_off ();
		putnstr ("", popupmenuborder);

		item = & (Popupmenu [i]);
		If hop_flag > 0 && item->hopitemname != NIL_PTR && item->hopitemname [0] != '\0'
		Then	putnstr (item->hopitemname, popupmenu_width - 2 - 2 * popupmenuborder);
		Else	putnstr (item->itemname, popupmenu_width - 2 - 2 * popupmenuborder);
		Fi

		putnstr ("", popupmenuborder);
		menudisp_on ();
		putchar (' ');
	Done
	gotoxy (column, line + 1 + menulen);
	putnstr ("", popupmenu_width);
	menudisp_off ();
	gotoxy (column, line);
	first_dirty_line = line - 1;
	last_dirty_line = line + 1 + menulen;
	popupmenu_displayed = True;
End


/***********************************************************************\
	menselected () highlights a selected pulldown menu item
\***********************************************************************/
local void
	menselected (meni, i) int meni; int i;
Begin
	gotoxy (meni * menu_width, 1 + i);
	menudisp_on ();
	putstring ("  ");
	putnstr (menus [meni].menuitems [i].itemname, menu_width - 4);
	putstring ("  ");
	menudisp_off ();
End


/***********************************************************************\
	popupmenselected () highlights a selected popup menu item
\***********************************************************************/
local void
	popupmenselected (column, line, i) int column; int line; int i;
Begin
	gotoxy (column, line + 1 + i);
	menudisp_on ();
	putstring ("  ");
	putnstr (Popupmenu [i].itemname, popupmenu_width - 4);
	putstring ("  ");
	menudisp_off ();
End


/***********************************************************************\
	clear_menus () clears menus which are still displayed
\***********************************************************************/
global void
	clear_menus ()
Begin
	If pulldownmenu_displayed == True
	Then	pulldownmenu_displayed = False;
		display (0, top_line, last_dirty_line, last_dirty_line);
		last_dirty_line = 0;
		set_cursor_xy ();
	Fi
	If popupmenu_displayed == True
	Then	LINE * start_line = proceed (cur_line, first_dirty_line - menu_source_line);
		popupmenu_displayed = False;
		display (first_dirty_line, start_line, 1 + arrlen (Popupmenu), first_dirty_line + 1 + arrlen (Popupmenu));
		last_dirty_line = 0;
		set_cursor_xy ();
	Fi
End


/***********************************************************************\
	MINMEN () handles menu dialog
\***********************************************************************/
global void
	MINMEN (column) int column;
Begin
	int meni;
	character c;

	If menu_width < 3
	Then	return;
	Fi
	meni = column / menu_width;
	If meni < 0 || meni >= arrlen (menus)
	Then	/* check if it's a flag toggle position */
		meni = column - flags_pos;
		If meni >= 0 && meni < flags_displayed
		Then	(* Flagmenu [meni].toggle) ();
			displayflags ();
			set_cursor_xy ();
		Fi
		return;
	Fi

	displaymenu (meni);
	flush ();

	c = readchar ();
	If c == (character) FUNcmd && (* keyproc) == MOUSEescape
	Then	/* mouse click */
		If mouse_xpos / menu_width == meni
		&& mouse_ypos >= 0 && mouse_ypos < menus [meni].menulen
		Then
			menselected (meni, mouse_ypos);
			(* menus [meni].menuitems [mouse_ypos].itemfu) ();
		Elsif mouse_ypos == -1 && mouse_xpos / menu_width != meni
		Then	clear_menus ();
			displaymenuheaders ();
			MINMEN (mouse_xpos);
		Fi
	Else
	Fi
	clear_menus ();
End


/***********************************************************************\
	QUICKMEN () handles right mouse button menu
\***********************************************************************/
global void
	QUICKMEN (column, line) int column; int line;
Begin
	character c;
	int startline;
	int startcolumn;
	int menulen = arrlen (Popupmenu);

	startline = line + 1;
	If startline > YMAX - menulen - 1
	Then	startline = YMAX - menulen - 1;
	Fi
	startcolumn = column;
	If startcolumn > XMAX - popupmenu_width
	Then	startcolumn = XMAX - popupmenu_width;
	Fi
	If startline < 1 || startcolumn < 0 || popupmenu_width < 3
	Then	return;
	Fi

	popupmenu (startcolumn, startline);
	menu_source_line = line;
	flush ();

	c = readchar ();
	If c == (character) FUNcmd && (* keyproc) == MOUSEescape
	Then	/* mouse click */
		mouse_xpos -= startcolumn;
		mouse_ypos -= startline;
		If mouse_xpos > 0 && mouse_xpos < popupmenu_width
		&& mouse_ypos >= 0 && mouse_ypos < arrlen (Popupmenu)
		Then
			popupmenselected (startcolumn, startline, mouse_ypos);
			(* Popupmenu [mouse_ypos].itemfu) ();
		Elsif mouse_ypos + startline == -1
		Then	clear_menus ();
			displaymenuheaders ();
			MINMEN (mouse_xpos + startcolumn);
		Fi
	Else
	Fi
	clear_menus ();
End


/***********************************************************************\
	End
\***********************************************************************/
